# --------------------------------------------------------------------
# backup_make --------------------------------------------------------
#           Make backup file
# --------------------------------------------------------------------
proc backup_make {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # project name is mandatory
   if { $wk(file,project) == "" } {
      tk_messageBox -icon error -type ok \
	  -message "First save all data before making backups."
      return
   }
   # backup directory
   set dir [format "%s/backup" [file dirname $wk(file,project)]]
   if { ![file exists $dir] } {
      set ans [tk_messageBox -icon question -type yesno\
		   -message "You don't have a backup directory.  Create backup directory?" -title "Create backup dir?"]
      if { $ans == "no" } {
	 bar_message "Canceled: making backup file."
	 return
      }
      file mkdir $dir
   }
   if { ![file isdirectory $dir] } {
      tk_messageBox -icon error -type ok\
	  -message "$dir is not a directory.  Cannot make a backup file."\
	  -title "Not a dir"
   }
   # backup filename
   set rootname [file rootname [file tail $wk(file,project)]]
   set extension ".ray86"
   set date [clock format [clock seconds] -format "%Y%m%d_%H%M%S"]
   set fn [format "%s/%s-%s%s" $dir $rootname $date $extension]

   # save backup file
   write_ray86 $fn

   # update backup menu
   backup_menu_update
   
   # message
   bar_message "backup file was made $date"
}

# --------------------------------------------------------------------
# backup_menu_update -------------------------------------------------
#           Update backup filelist menu
# --------------------------------------------------------------------
proc backup_menu_update {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # cleanup menu
   set m $wk(backup,menuwidget)
   $m delete 0 end

   # check backup dir
   set dir [format "%s/backup" [file dirname $wk(file,project)]]
   if { ![file exists $dir] || ![file isdirectory $dir]} {
      return
   }
   
   # list of backup files
   set rootname [file rootname [file tail $wk(file,project)]]
   set extension ".ray86"
   set backuplist [glob -nocomplain\
		       [format "%s/%s-*%s" $dir $rootname $extension]]
   set backuplist [lsort -dictionary -decreasing $backuplist]
   set backuplist [lrange $backuplist 0 $gbl(backup,listmax)]

   foreach fn $backuplist {
      # label
      set rootname [file rootname [file tail $fn]]
      set d [lindex [split $rootname "-"] end]
      if { [string length $d] == 15 } {
	 set date [format "%s/%s/%s %s:%s:%s"\
		       [string range $d 0 3]\
		       [string range $d 4 5]\
		       [string range $d 6 7]\
		       [string range $d 9 10]\
		       [string range $d 11 12]\
		       [string range $d 13 14]]
	 $m add command -label "$date" -command "read_data $fn"
      }
   }
}

# --------------------------------------------------------------------
# backup_cleanup -----------------------------------------------------
#           Cleanup old backup files
# --------------------------------------------------------------------
proc backup_cleanup {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # check backup dir
   set dir [format "%s/backup" [file dirname $wk(file,project)]]
   if { ![file exists $dir] || ![file isdirectory $dir]} {
      return
   }
   
   # list of backup files
   set rootname [file rootname [file tail $wk(file,project)]]
   set extension ".ray86"
   set backuplist [glob -nocomplain\
		       [format "%s/%s-*%s" $dir $rootname $extension]]
   set backuplist [lsort -dictionary -decreasing $backuplist]
   set backuplist [lrange $backuplist 0 $gbl(backup,listmax)]

   # a lot of files?
   if { [llength $backuplist] > $gbl(backup,oldfiles) } {
      bar_message "not so much backup files. (cleanup cancelled)"
      return
   }
   set backuplist [lrange $backuplist $gbl(backup,oldfiles) end]
   # old files?
   set i 0
   foreach fn $backuplist {
      if [file writable $fn] {
	 file delete $fn
	 incr i
      }
   }
   bar_message "Clean up old $i files."
}