# --------------------------------------------------------------------
# update_bgimage ----------------------------------------------------
#            Update background image
# --------------------------------------------------------------------
proc update_bgimage {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # main --------------------------------------------------
   switch -- $gbl(hamen,velimage) {
      0 {
	 if { [$sys(graph) marker exist bgimage] } {
	    $sys(graph) marker configure bgimage -hide 1
	 }
      }
      1 {
	 # velocity field
	 update_velimage
      }
      2 {
	 # aribitrary image file
	 update_arbitrary_image
      }
      3 {
	 # netCDF file
	 update_netcdf_bgimage
      }
   }
}

# --------------------------------------------------------------------
# update_arbitrary_image ---------------------------------------------
#            Update arbitrary image
# --------------------------------------------------------------------
proc update_arbitrary_image {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # main --------------------------------------------------
   if { [file readable $gbl(file,bgimage)] } {
      bar_message "Now reading an image file..."
      image create photo bgimage -file $gbl(file,bgimage)
      $sys(graph) marker create image -image bgimage -name bgimage\
	  -coord "$gbl(xmin) $gbl(zmin) $gbl(xmax) $gbl(zmax)" -under 1
      draw_frame
      draw_model
      bar_message "Done..."
   } else {
      tk_messageBox -icon error -type ok\
	  -message "Cannot read the image file, \"$gbl(file,bgimage)\""
   }
}

# --------------------------------------------------------------------
# update_velimage ----------------------------------------------------
#            Update velocity field image
# --------------------------------------------------------------------
proc update_velimage {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # main --------------------------------------------------
   
   # check file existency and make netCDF file, if required
   set ray86fn [format "modeling.tmp.%d.ray86" [pid]]
   set tmpnetcdf [format "modeling.tmp.%d.grd" [pid]]
   switch -- $gbl(hamen,mformat) {
      1 {
	 # ray86 --------------------
	 if {$gbl(n_interface) < 1} {
	    tk_messageBox -icon error -type ok -message "No model defined."
	    return
	 }
	 # create netCDF file
	 write_ray86 $ray86fn
	 set netcdf $tmpnetcdf
	 set com [format "ray86cdf -d %f %s %s" $gbl(hamen,dx) $ray86fn $netcdf]
	 exec_external_command $com
      }
      2 {
	 # ray86 + gm --------------------
	 
	 if { ![file exist $gbl(file,gmfile)] } {
	    tk_messageBox -icon error -type ok\
		-message "Cannot find gm file, \"$gbl(file,gmfile)\""
	    return
	 }
	 # create netCDF file
	 write_ray86 $ray86fn
	 set netcdf $tmpnetcdf
	 set com [format "model2cdf -d %f -b %d %s %s %s"\
		      $gbl(hamen,dx) $gbl(hamen,mboundary)\
		      $ray86fn $gbl(file,gmfile) $netcdf]
	 exec_external_command $com
      }
      3 {
	 # netCDF --------------------
	 if { ![file exist $gbl(file,cdffile)] } {
	    tk_messageBox -icon error -type ok\
		-message "Cannot find netCDF file, \"$gbl(file,cdffile)\""
	    return
	 }
	 set netcdf $gbl(file,cdffile)
      }
   }

   # check cptfile ------------------------------
   if { ! [file exist $gbl(file,cptfile)] } {
      make_cpt_file
      tk_messageBox -icon warning -type ok\
	  -message "Couldn't find CPT file for velocity structure image.  I made a one for you.  Check it out."
   }

   # GMT must run in EPS mode ------------------------------
   exec_external_command "gmtset PAPER_MEDIA a4+"

   # make image file ------------------------------
   bar_message "Now making velocity field image..."
   set psfn [format "modeling.tmp.%d.ps" [pid]]
   set imagefn [format "modeling.tmp.%d.png" [pid]]
   if { $gbl(bg,contour) > 0.0 } {
      set com [format "grdimage %s -C%s -JX10/-10 -R%f/%f/%f/%f -P -K > %s"\
		   $netcdf $gbl(file,cptfile)\
		   $gbl(xmin) $gbl(xmax) $gbl(zmin) $gbl(zmax)\
		   $psfn ]
      exec_external_command $com
      set com [format "grdcontour %s -C%s -JX10/-10 -R%f/%f/%f/%f -O >> %s"\
		   $netcdf $gbl(bg,contour) \
		   $gbl(xmin) $gbl(xmax) $gbl(zmin) $gbl(zmax)\
		   $psfn]
      exec_external_command $com
      set com [format "convert -density %s %s %s" \
		   $gbl(bg,density) $psfn $imagefn]
      exec_external_command $com
   } else {
      set com [format "grdimage %s -C%s -JX10/-10 -R%f/%f/%f/%f -P | convert -density %s - %s"\
		   $netcdf $gbl(file,cptfile)\
		   $gbl(xmin) $gbl(xmax) $gbl(zmin) $gbl(zmax)\
		   $gbl(bg,density) $imagefn ]
      exec_external_command $com
   }
   bar_message "Done."

   # Show jpeg file ------------------------------
   image create photo bgimage -file $imagefn
   $sys(graph) marker create image -image bgimage -name bgimage\
       -coord "$gbl(xmin) $gbl(zmin) $gbl(xmax) $gbl(zmax)" -under 1
   draw_frame
   draw_model

   # Delete temporary file ------------------------------
   file delete -force $ray86fn
   file delete -force $tmpnetcdf
   file delete -force $psfn
   file delete -force $imagefn

   set gbl(hamen,velimage) 1
}

# --------------------------------------------------------------------
# update_netcdf_bgimage ----------------------------------------------
#            Update background image by a netCDF file
# --------------------------------------------------------------------
proc update_netcdf_bgimage {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # main --------------------------------------------------
   # check file existency
   if { ![file exist $gbl(file,bgcdf)] } {
      tk_messageBox -icon error -type ok\
	  -message "Cannot find netCDF file for backgournd image, \"$gbl(file,bgcdf)\""
      return
   }
   # read netCDF info
   set f [open "| grdinfo -C $gbl(file,bgcdf)"]
   scan [gets $f] "%*s%f%f%f%f" xmin xmax zmin zmax
   close $f

   # check CPT ------------------------------
   set cptfn [format "modeling.tmp.%d.cpt" [pid]]
   if { $gbl(bg,cpt,mode) == 0 } {
      # automatic generation
      set com [format "grd2cpt %s -C%s -Z" $gbl(file,bgcdf) $gbl(bg,cpt,master)]
      if { $gbl(bg,cpt,inverse) } {
	 set com [format "%s -I > %s" $com $cptfn]
      } else {
	 set com [format "%s > %s" $com $cptfn]
      }
      eval exec $com
   } else {
      # use specified CPT file
      if { [string length $gbl(file,bgcdfcpt)] < 1 ||
	   ![file readable $gbl(file,bgcdfcpt)] } {
	 tk_messageBox -message "Cannot find CPT file for background image."\
	     -icon error -type ok
	 return
      }
      file copy $gbl(file,bgcdfcpt) $cptfn
   }

   # GMT must run in EPS mode ------------------------------
   exec_external_command "gmtset PAPER_MEDIA a4+"

   # make image file ------------------------------
   bar_message "Now making velocity field image..."
   set imagefn [format "modeling.tmp.%d.png" [pid]]
   set com [format "grdimage %s -C%s -JX10/-10 -R%f/%f/%f/%f -P | convert -density %s - %s"\
		$gbl(file,bgcdf) $cptfn	$xmin $xmax $zmin $zmax\
		$gbl(bg,density) $imagefn ]
   exec_external_command $com
   bar_message "Done."

   # Show png file ------------------------------
   image create photo bgimage -file $imagefn
   $sys(graph) marker create image -image bgimage -name bgimage\
       -coord "$gbl(xmin) $gbl(zmin) $gbl(xmax) $gbl(zmax)" -under 1
   draw_frame
   draw_model

   # Delete temporary file ------------------------------
   file delete -force $cptfn
   file delete -force $imagefn
}

# --------------------------------------------------------------------
# make_cpt_file ------------------------------------------------------
#           Make a sample CPT file
# --------------------------------------------------------------------
proc make_cpt_file {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   set fp [open $gbl(file,cptfile) "w"]
   puts $fp "# Sample CPT file.
# Created by modeling.
1.51    255     0       255     2       149     0       255
2       149     0       255     3     42      0       255
3       42      0       255     4       0       63      255
4       0       63      255     5       0       170     255
5       0       170     255     5.5     0       255     234
5.5     0       255     234     6       85      255     0
6       85      255     0       6.5     255     213     0
6.5     255     213     0       7       255     106     0
7       255     106     0       8.5     255     0       0
F       255     0       0
B       255     255     255"
   close $fp
}

# --------------------------------------------------------------------
# toggle_bgimage -----------------------------------------------------
#            toggle show/hide background image
# --------------------------------------------------------------------
proc toggle_bgimage {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # main --------------------------------------------------
   if { $gbl(hamen,velimage) == 0 } { 
      if { [$sys(graph) marker exist bgimage] } {
	 $sys(graph) marker configure bgimage -hide 1
      }
      return 
   }

   if { ![$sys(graph) marker exist bgimage] } {
      show_bgimage
      return
   }
   # show/hide background image
   if { [$sys(graph) marker cget bgimage -hide] } {
      # to show
      $sys(graph) marker configure bgimage -hide 0
   } else {
      # to hide
      $sys(graph) marker configure bgimage -hide 1
   }
}

# --------------------------------------------------------------------
# show_bgimage -------------------------------------------------------
#            Show background image
# --------------------------------------------------------------------
proc show_bgimage {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # main --------------------------------------------------
   # turn off
   if { [$sys(graph) marker exist bgimage] } {
      $sys(graph) marker configure bgimage -hide 1
   }

   # show if required
   switch -- $gbl(hamen,velimage) {
      1 {
	 # to show velocity image
	 update_velimage
      }
      2 {
	 # to show an arbitrary image
	 update_arbitrary_image
      }
   }
}