# --------------------------------------------------------------------
# change_bindings ----------------------------------------------------
#            change bindings (next(+1) or prev(-1))
# --------------------------------------------------------------------
proc change_bindings {dir} {
   # Global variables --------------------------------------------------
   global wk
   
   # Main --------------------------------------------------
   set lst [list Node V-Grid]
   set i [lsearch $lst $wk(current_bindings)]
   incr i $dir
   if { $i >= [llength $lst] } { set i 0 }
   if { $i < 0 } { set i [expr [llength $lst] - 1] }
   .message_bar.binding.menu invoke $i
}

# --------------------------------------------------------------------
# update_bindings ----------------------------------------------------
#            Update mouse bindings on the model edit window
# --------------------------------------------------------------------
proc update_bindings {} {
   # Global variables --------------------------------------------------
   global wk
   
   # Main --------------------------------------------------
   switch -- [string tolower $wk(current_bindings)] {
      "node" {
	 node_bindings
      }
      "v-grid" {
	 vgrid_bindings
      }
   }
   update_ALT_bindings
   update_help_keybind
}
# --------------------------------------------------------------------
# clear_bindigs -------------------------------------------------------
#            Clear no-comman mouse bindings.
# --------------------------------------------------------------------
proc clear_bindings {} {
    # Global variables --------------------------------------------------
    global sys

    # Main ------------------------------
    bind $sys(graph) <Button-1> ""
    bind $sys(graph) <Button-2> ""
    bind $sys(graph) <Button-3> ""

    bind $sys(graph) <Shift-Button-1> ""
    bind $sys(graph) <Shift-Button-2> ""
    bind $sys(graph) <Shift-Button-3> ""
    bind $sys(graph) <Double-Button-1> ";"

    bind $sys(graph) <Triple-Button-1> ""
    bind $sys(graph) <Double-ButtonPress-3> ""
}
# --------------------------------------------------------------------
# common_bindigs -----------------------------------------------------
#            Key and Mouse bindings on Graph widget (common)
# --------------------------------------------------------------------
proc common_bindings {} {
    # Global variables --------------------------------------------------
    global sys

    # Main ------------------------------
    # Model exists? ----------
    if { ![$sys(graph) element exist graph_frame] } {
	return
    }
    clear_bindings

    # Select interfaces
    bind $sys(graph) <Button-1> "select_interface %x %y"
    # Edit velocity grid
    bind $sys(graph) <Button-2> "select_grid %x %y %X %Y edit"

    # Double click of Mouse-1
    bind $sys(graph) <Double-Button-1> "bind_graph_double1 %x %y %X %Y"

    # Add velocity grid
    bind $sys(graph) <Triple-Button-1> "create_grid %x %y"

    # Move velocity grid
    bind $sys(graph) <Double-ButtonPress-3>\
	    "select_grid %x %y %X %Y move"
}

# --------------------------------------------------------------------
# bind_graph_double1 -------------------------------------------------
#            Double click on the graph widget
# --------------------------------------------------------------------
# [$B0z?t(B]
#   x   : x coordinate of Mouse (Graph widget coodinate)
#   y   : y coordinate of Mouse (Graph widget coodinate)
#   X   : x coordinate of Mouse (root window coodinate)
#   Y   : y coordinate of Mouse (root window coodinate)
# --------------------------------------------------------------------
proc bind_graph_double1 {x y X Y} {
    # Global variables --------------------------------------------------
    global sys

    # Get click point
    if [$sys(graph) inside $x $y] {
	# Inside of plot area
	select_point $x $y $X $Y
    } else {
	# Outside of plot area
	select_axis $x $y $X $Y
    }
}

# --------------------------------------------------------------------
# update_ALT_bindings ----------------------------------------------------
#            (Re)binding of using option key.
# --------------------------------------------------------------------
proc update_ALT_bindings {} {
    # Global variables --------------------------------------------------
    global gbl sys

    # Clear ----------
    foreach i {Alt Meta Lock} {
	bind $sys(graph) <$i\-Button-1> ""
	bind $sys(graph) <$i\-Button-2> ""
	bind $sys(graph) <$i\-Button-3> ""
	bind . <$i\-o> ""
	bind . <$i\-s> ""
	bind . <$i\-a> ""
	bind . <$i\-u> ""
    }

    # menu bar ----------
    bind . <$gbl(option,option_key)-o> "open_model ray86"
    bind . <$gbl(option,option_key)-s> "save_model ray86"
    bind . <$gbl(option,option_key)-u> "undo"
    bind . <$gbl(option,option_key)-b> "backup_make"
}
# --------------------------------------------------------------------
# node_bindigs -------------------------------------------------------
#            Mouse bindings on Graph widget for "Node" mode.
# --------------------------------------------------------------------
proc node_bindings {} {
    # Global variables --------------------------------------------------
    global sys

    # Main ------------------------------
    # Model exists? ----------
    if { ![$sys(graph) element exist graph_frame] } {
	return
    }
    common_bindings

    # Add a point
    bind $sys(graph) <Shift-Button-1> "create_point %x %y"
    # Delete a point
    bind $sys(graph) <Shift-Button-2> "delete_point %x %y"
    # Move closest point
    bind $sys(graph) <Button-3> "move_start %x %y"
    bind $sys(graph) <Shift-Button-3> "move_start %x %y"
}
# --------------------------------------------------------------------
# vgrid_bindigs ------------------------------------------------------
#            Mouse bindings on Graph widget for "V-Grid" mode.
# --------------------------------------------------------------------
proc vgrid_bindings {} {
    # Global variables --------------------------------------------------
    global sys

    # Main ------------------------------
    # Model exists? ----------
    if { ![$sys(graph) element exist graph_frame] } {
	return
    }
    common_bindings

    # Add velocity grid
    bind $sys(graph) <Shift-Button-1> "create_grid %x %y"
    # Delete velocit grid
    bind $sys(graph) <Shift-Button-2> "select_grid %x %y %X %Y delete"
    # Move velocity grid
    bind $sys(graph) <Button-3> "select_grid %x %y %X %Y move"
    bind $sys(graph) <Shift-Button-3> "select_grid %x %y %X %Y move"
}
