# --------------------------------------------------------------------
# draw_frame ----------------------------------------------------------
#            Draw outside line of velocity structure
# --------------------------------------------------------------------
proc draw_frame {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # Main --------------------------------------------------
   catch {$sys(graph) element delete graph_frame}
   # Model edit window
   $sys(graph) element create graph_frame\
       -xdata "$gbl(xmin) $gbl(xmin) $gbl(xmax) $gbl(xmax) $gbl(xmin)"\
       -ydata "$gbl(zmax) $gbl(zmin) $gbl(zmin) $gbl(zmax) $gbl(zmax)"\
       -color black -dashes "5 5" -symbol "" -label ""
   set xrange [$sys(graph) xaxis limit]
   set yrange [$sys(graph) yaxis limit]
   $sys(graph) x2axis conf -min [lindex $xrange 0] -max [lindex $xrange 1]
   $sys(graph) y2axis conf -min [lindex $yrange 0] -max [lindex $yrange 1]

   # T-X graph window
   catch {$sys(tx) element delete graph_frame}
   set trange [$sys(tx) yaxis limit]
   $sys(tx) element create graph_frame\
       -xdata "$gbl(xmin) $gbl(xmin) $gbl(xmax) $gbl(xmax) $gbl(xmin)"\
       -color black -dashes "5 5" -symbol "" -label ""
   $sys(tx) xaxis conf -min [lindex $xrange 0] -max [lindex $xrange 1]
   $sys(tx) x2axis conf -min [lindex $xrange 0] -max [lindex $xrange 1]
   if { $gbl(option,t_axis) } {
      # auto-scale
      $sys(tx) yaxis conf -min {} -max {}
      $sys(tx) y2axis conf -min {} -max {}
   } else {
      # set scale
      $sys(tx) yaxis conf -min $gbl(tmin) -max $gbl(tmax)
      $sys(tx) y2axis con -min $gbl(tmin) -max $gbl(tmax)
   }

}
# --------------------------------------------------------------------
# draw_model ----------------------------------------------------------
#            Draw velocity structure model
# --------------------------------------------------------------------
proc draw_model {} {
   # Global variables --------------------------------------------------
   global gbl sys wk
   for {set i 1} {$i < $sys(MAX_LAYER)} {incr i} {
      global vec_x$i vec_z$i
   }
   
   # Main --------------------------------------------------
   # Clear all the vector --------------------
   for {set i 1} {$i < $sys(MAX_LAYER)} {incr i} {
      vec_x$i clear
      vec_z$i clear
   }
   for {set i 1} {$i <= $gbl(n_interface)} {incr i} {
      catch {$sys(graph) element delete interface($i)}
      vec_x$i set $gbl(x,$i)
      vec_z$i set $gbl(z,$i)
      $sys(graph) element create interface($i)\
	  -xdata vec_x$i -ydata vec_z$i -pixe $wk(mark,size)\
	  -outline $wk(col,normal) -fill $wk(col,normal)\
	  -color $wk(col,normal) -label "" -activepen interfacePen
   }
   # Hilight the current interface ------------------------------
   selection_flash
   # Display velocity grid
   draw_grid
   # Display sources
   draw_source
}
# --------------------------------------------------------------------
# draw_grid ----------------------------------------------------------
#            Display all the velocity grid and labels
# --------------------------------------------------------------------
proc draw_grid {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   for {set i 1} {$i < $gbl(n_interface)} {incr i} {
      # $i-th layer ----------
      draw_grid_layer $i
   }
}
# --------------------------------------------------------------------
# draw_grid_layer ----------------------------------------------------
#            Display velocity grid and labels in $i layer.
# --------------------------------------------------------------------
# [arguments]
#   i   : Layer number.
# --------------------------------------------------------------------
proc draw_grid_layer {i} {
   # Global variables --------------------------------------------------
   global gbl sys

   # Main --------------------------------------------------
   # Delete old velocity grid and label
   foreach name [$sys(graph) marker names m($i,*,*)] {
      $sys(graph) marker delete $name
   }
   if {$gbl(option,grid) == 0} {
      return 
   }
   for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
      set x [lindex $gbl(vx,$i) $j]
      # $i-layer $j-th grid  ----------
      # Upper Velocity grid
      set top_depth [interpolate_depth $i $x]
      # Lower velocity grid
      set bot_depth [interpolate_depth [expr $i+1] $x]
      # Draw a velocity grid ----------
      $sys(graph) marker create line -coords "$x $top_depth $x $bot_depth"\
	  -outline gray -name m($i,$j,v)
      # Draw velocity labels ----------
      if {$gbl(option,label) == 1} {
	 $sys(graph) marker create text\
	     -text [format "%.2f" [lindex $gbl(vtop,$i) $j]]\
	     -coords "$x $top_depth" -anchor n -name m($i,$j,lt)
	 $sys(graph) marker create text\
	     -text [format "%.2f" [lindex $gbl(vbot,$i) $j]]\
	     -coords "$x $bot_depth" -anchor s -name m($i,$j,lb)
      }
   }
}
# --------------------------------------------------------------------
# show_positon -------------------------------------------------------
#            Show the mouse corsor position.
# --------------------------------------------------------------------
proc show_position {x y} {
    # Global variables --------------------------------------------------
    global gbl sys wk

    # Main --------------------------------------------------
    set xz [$sys(graph) invtransform $x $y]
    set gx [lindex $xz 0]
    set gz [lindex $xz 1]
    catch {$wk(w,pos).message configure -text [format "(%8.3fkm, %8.3fkm)" $gx $gz]}
    if {$gbl(option,vel)} {
	show_velocity $x $y
    }
}
# --------------------------------------------------------------------
# draw_source --------------------------------------------------------
#            Display selected sources.
# --------------------------------------------------------------------
proc draw_source {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # Main --------------------------------------------------
   # Erase old sources.
   foreach name [$sys(graph) marker names source(*)] {
      $sys(graph) marker delete $name
   }
   # Display sources.
   if {$gbl(option,source) == 0} {
      return
   }
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      if {$gbl(src,$i,onoff)} {
	 catch {$sys(graph) marker create bitmap\
		    -bitmap @$sys(SOURCE_IMAGE) -outline green\
  		    -name source($i) -anchor center\
  		    -coords "$gbl(src,$i,x) $gbl(src,$i,z)"}
      }
   }
}
# --------------------------------------------------------------------
# Zooming related:  from BLT library, slightly modified.
# --------------------------------------------------------------------
proc blt::SetZoomPoint { graph x y } {
   global zoomInfo zoomMod
   global gbl sys
   
   if { ![info exists zoomInfo($graph,corner)] } {
      blt::InitStack $graph
   }
   blt::GetCoords $graph $x $y $zoomInfo($graph,corner)
   if { [info exists zoomMod] } {
      set modifier $zoomMod
   } else {
      set modifier "Any-"
   }
   bind select-region-$graph <${modifier}Motion> { 
      blt::GetCoords %W %x %y B
      #blt::MarkPoint $graph B
      blt::Box %W
   }
   if { $zoomInfo($graph,corner) == "A" } {
      if { ![$graph inside $x $y] } {
	 return
      }
      # First corner selected, start watching motion events
      
      #blt::MarkPoint $graph A
      blt::ZoomTitleNext $graph 
      
      blt::AddBindTag $graph select-region-$graph
      set zoomInfo($graph,corner) B
   } else {
      # Delete the modal binding
      blt::RemoveBindTag $graph select-region-$graph
      blt::PushZoom $graph 
      # Modified by FUJIE: BEGIN ------------------------------
      # update another graph
      if {$graph == $sys(graph)} {
	 # update T-X
	 set g $sys(tx)
      } else {
	 # update model
	 set g $sys(graph)
      }
      # set current region
      # xmin and xmax
      set axis [$graph xaxis use]
      set xlimit [$graph axis limit $axis]
      if {$xlimit == ""} {
	 set xmin 0
	 set xmax 10
      } else {
	 set xmin [lindex $xlimit 0]
	 set xmax [lindex $xlimit 1]
      }
      # ymin and ymax
      set axis [$g yaxis use]
      set ylimit [$g axis limit $axis]
      if {$ylimit == ""} {
	 set ymin 0
	 set ymax 10
      } else {
	 set ymin [lindex $ylimit 0]
	 set ymax [lindex $ylimit 1]
      }
      # convert to widget coordinates
      set zoomInfo($g,A,x) [$g xaxis transform $xmin]
      set zoomInfo($g,A,y) [$g yaxis transform $ymin]
      set zoomInfo($g,B,x) [$g xaxis transform $xmax]
      set zoomInfo($g,B,y) [$g yaxis transform $ymax]
      blt::PushZoom $g
      # Modified by FUJIE: END ------------------------------
      set zoomInfo($graph,corner) A
   }
}

proc blt::ResetZoom { graph } {
   global zoomInfo 
   global gbl sys

   if { ![info exists zoomInfo($graph,corner)] } {
      blt::InitStack $graph 
   }
   eval $graph marker delete [$graph marker names "zoom*"]
   
   if { $zoomInfo($graph,corner) == "A" } {
      # Reset the whole axis
      blt::PopZoom $graph
      # Added by FUJIE : BEGIN ------------------------------
      if {$graph == $sys(graph)} {
	 # update T-X
	 set g $sys(tx)
      } else {
	 # update model
	 set g $sys(graph)
      }
      blt::PopZoom $g
      # Added by FUJIE : END ------------------------------
   } else {
      global zoomMod
      
      if { [info exists zoomMod] } {
	 set modifier $zoomMod
      } else {
	 set modifier "Any-"
      }
      set zoomInfo($graph,corner) A
      blt::RemoveBindTag $graph select-region-$graph
   }
}
