# --------------------------------------------------------------------
# select_axis --------------------------------------------------------
#            Invoke axis edit window.
# --------------------------------------------------------------------
# [arguments]
#   x   : x coordinate of mouse cursor. (Graph widget coordiantes.)
#   y   : y coordinate of mouse cursor. (Graph widget coordiantes.)
#   XX  : x coordinate of mouse cursor. (root window coordiantes.)
#   YY  : y coordinate of mouse cursor. (root window coordiantes.)
# --------------------------------------------------------------------
proc select_axis {x y XX YY} {
   # Global variables --------------------------------------------------
   global sys
   
   # Main --------------------------------------------------
   # Range of axis
   set xx [$sys(graph) axis limits x]
   set x1 [lindex $xx 0]
   set x2 [lindex $xx 1]
   set zz [$sys(graph) axis limits y]
   set z1 [lindex $zz 0]
   set z2 [lindex $zz 1]
   
   # Get cursor postion.
   set xz [$sys(graph) invtransform $x $y]
   set x [lindex $xz 0]
   set z [lindex $xz 1]
   
   if { $z2 <= $z || $z1 >= $z} {
      # Set horizontal position
      set_axis x $XX $YY
   } elseif { $x1 >= $x || $x2 <= $x} {
      # Set vertical position
      set_axis y $XX $YY
   }
}
# --------------------------------------------------------------------
# set_axis -----------------------------------------------------------
#            Invoke axis edit window
# --------------------------------------------------------------------
# [arguments]
#   a   : Axis mode
#   XX  : x coordinate of mouse cursor. (root window coordiantes.)
#   YY  : y coordinate of mouse cursor. (root window coordiantes.)
# --------------------------------------------------------------------
proc set_axis {a XX YY} {
   # Global variables --------------------------------------------------
   global sys gbl
   
   # Main --------------------------------------------------
   # Identify axis
   switch $a {
      "x" {
	 set axis_name "Distance"
	 set axis x
	 set g $sys(graph)
	 set unit "km."
	 set name x
      }
      "y" {
	 set axis_name "Depth"
	 set axis y
	 set g $sys(graph)
	 set unit "km."
	 set name z
      }
      "t" {
	 set axis_name "Time"
	 set axis y
	 set g $sys(tx)
	 set unit "sec."
	 set name t
      }
      default {
	 set axis_name "undefined"
	 set g $sys(graph)
	 set name "unknown"
      }
   }
   
   # Get current settings
   set limit [$g axis limits $axis]
   set limit1 [lindex $limit 0]
   set limit2 [lindex $limit 1]
   
   # Invoke window
   set w .set_axis_window
   catch {destory $w}
   toplevel $w
   wm geometry $w [format "+%d+%d" $XX $YY]
   
   # Title frame ------------------------------
   title_frame $w.title "Edit Axis ($axis_name) "
   
   # Parameter frame ------------------------------
   set pf $w.parameter
   frame $pf -relief flat -bd 2

   # reduction
   if { $a == "t" } {
      make_entry $pf rv "Reduction Velocity" gbl(plot,vred) "km/sec"
   }
   # auto scale
   make_check_label $pf "auto scaling" autoscale gbl(option,$a\_axis) \
       "axis_entry_status $pf $a"
   label $pf.left -text "$name = ("
   entry $pf.emin -width 10
   $pf.emin insert 0 $limit1
   label $pf.comma -text ","
   entry $pf.emax -width 10
   $pf.emax insert 0 $limit2
   label $pf.unit -text ")$unit,  tick interval="
   entry $pf.estep -width 5
   label $pf.ustep -text $unit
   $pf.estep insert 0 [$g axis cget $axis -stepsize]
   grid x $pf.left $pf.emin $pf.comma $pf.emax $pf.unit $pf.estep $pf.ustep\
       -sticky news
   # update status
   axis_entry_status $pf $a

   # Button frame ------------------------------
   frame $w.button
   # OK button
   button $w.button.ok -text "OK" -width 10\
       -command "update_axis $a $pf; destroy $w"
   # Cancel button
   button $w.button.cancel -text "Cancel" -width 10\
       -command "busy release . ; destroy $w"
   # Packing
   pack $w.button.ok $w.button.cancel\
	    -in $w.button -side left -expand 1
   
   # Packing ------------------------------
   pack $w.title $pf $w.button -side top -fill x
   busy hold .
}

# --------------------------------------------------------------------
# update_axis --------------------------------------------------------
#            Update axis setting.
# --------------------------------------------------------------------
# [arguments] 
#   a   :  Axis mode, "x", "y" or "t"
#   w   :  Widget name of axis edit.
# --------------------------------------------------------------------
proc update_axis {a w} {
    # Global variables --------------------------------------------------
    global gbl sys
    
    # Main --------------------------------------------------
    busy release .
    if {$gbl(option,$a\_axis)} {
       # in case of autoscale
       auto_axis $a
       return
    }
    
    switch $a {
       "x" {
	  set g $sys(graph)
	  set axis x
       }
       "y" {
	  set g $sys(graph)
	  set axis y
       }
       "t" {
	  set g $sys(tx)
	  set axis y
	  set gbl(tmin) [$w.emin get]
	  set gbl(tmax) [$w.emax get]
	  set gbl(option,t_tic) [$w.estep get]
       }
    }

    # Get read value ------------------------------
    set new_min [format "%.3f" [$w.emin get]]
    set new_max [format "%.3f" [$w.emax get]]
    set new_step [format "%.3f" [$w.estep get]]
    
    # Update ------------------------------
    set axis2 [format "%s2" $axis]
    if { $a == "x"} {
       # in case of x-axis.
       $sys(tx) axis conf $axis -min $new_min -max $new_max -step $new_step
       $sys(tx) axis conf $axis2 -min $new_min -max $new_max -step $new_step
    }

    $g axis conf $axis -min $new_min -max $new_max -step $new_step
    $g axis conf $axis2 -min $new_min -max $new_max -step $new_step
}

# --------------------------------------------------------------------
# auto_axis ----------------------------------------------------------
#            Set auto scale.
# --------------------------------------------------------------------
# [arguments] 
#   a  :  Axis mode
# --------------------------------------------------------------------
proc auto_axis {a} {
    # Global variables --------------------------------------------------
    global gbl sys
    
    # Main --------------------------------------------------
    switch $a {
       "x" {
	  set g $sys(graph)
	  set axis x
       }
       "y" {
	  set g $sys(graph)
	  set axis y
       }
       "t" {
	  set g $sys(tx)
	  set axis y
       }
    }
    $g axis configure $axis -min "" -max "" -stepsize -1
    if {$a == "x"} {
       $sys(tx) axis configure $axis -min "" -max "" -stepsize -1
       draw_frame
    }
}

# --------------------------------------------------------------------
# update_hide_axis ------------------------------------------------
#            Update hide axis
# --------------------------------------------------------------------
# [arguments] 
#   axis  :  Axis name
# --------------------------------------------------------------------
proc update_hide_axis {axis} {
    # Global variables --------------------------------------------------
    global gbl sys
    
    # Main --------------------------------------------------
    $sys(graph) axis configure $axis -hide $gbl(option,$axis\_axis)
    $sys(tx) axis configure $axis -hide $gbl(option,$axis\_axis)
}

# --------------------------------------------------------------------
# axis_entry_status --------------------------------------------------
#            Update entry widget status of axis edit window
# --------------------------------------------------------------------
# [arguments] 
#   w   :  Widget name of parent frame widget of entries
#   a   :  axis name, "x", "y", or "t"
# --------------------------------------------------------------------
proc axis_entry_status {w a} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   switch -- $gbl(option,$a\_axis) {
      1 {
	 # Auto scale mode
	 foreach {i} {emin emax estep} {
	    $w.$i conf -state disabled
	    $w.$i conf -foreground gray
	 }
	 auto_axis $a
      }
      0 {
	 # specified mode
	 foreach {i} {emin emax estep} {
	    $w.$i conf -state normal
	    $w.$i conf -foreground black
	 }
      }
   }
}