namespace eval HAMEN {
   namespace export *

   # --------------------------------------------------------------------
   # parameter window for HAMEN
   # --------------------------------------------------------------------
   proc parameter_window {} {
      # Global variables --------------------------------------------------
      global sys gbl
      
      # Main --------------------------------------------------
      set w .hamen_parameter
      
      # check backend program
      if { $gbl(option,backend) != "hamen" } {
	 tk_messageBox -icon warning -type ok\
	     -message "Now the current backend program is the \"RAYINVR\".  If you want to use \"Wavefront Method\" for a backend program, change the backend program before ray-trace."
      }

      if { [winfo exists $w] } {
	 wm deiconify $w
	 raise $w
	 return
      }
      toplevel $w
      wm title $w "Parameters for Wavefront Method"
      
      # Title frame ------------------------------
      title_frame $w.title "Parameters for Wavefront Method"
      
      # tabset frame ------------------------------
      set ts [tabset $w.tabset -relief flat]
      $ts conf -borderwidth 1 -highlightthickness 0 -side top
      foreach i {"Model" "RayTrace" "Plot" "BGimage"} {
	 $ts insert end $i -text $i -background gray70 -anchor nw
      }
      
      # parameter tab ------------------------------
      model_parameter $ts
      trace_parameter $ts
      plot_parameter $ts
      bgimage_parameter $ts

      # Button frame ------------------------------
      set bf [button_frame $w $w.button_frame]
      
      # Packing ------------------------------
      pack $w.title $ts $bf -side top -fill x
   }

   # --------------------------------------------------------------------
   # hamen_model_parameter ----------------------------------------------
   #            Hamen model parameter tab
   # --------------------------------------------------------------------
   proc model_parameter {ts} {
      # Global variables ------------------------------------------------
      global gbl sys wk
      
      # Main --------------------------------------------------
      set w [frame $ts.model]
      $ts tab conf "Model" -window $w -fill x

      # ------------------------------------------------------------
      # parameter frame --------------------------------------------
      set wf [frame $w.param]

      # initialize
      set wk(xmin) $gbl(xmin)
      set wk(xmax) $gbl(xmax)
      set wk(zmin) $gbl(zmin)
      set wk(zmax) $gbl(zmax)

      # format & file
      set ff [labelframe $wf.formatfile -text "format & file"]
      set u [frame $ff.upper]
      make_menu_com $u mformat "Model format" gbl(hamen,mformat) ""\
	  "Ray86" 1 "Ray86+gm" 2 "netCDF" 3
      make_spinbox $u mb "Boundary between Ray86 and gm" gbl(hamen,mboundary)\
	  1 $sys(MAX_LAYER) 1 "(effective for ray86+gm)"
      make_entry $u dx "Delta x and z (dx)" gbl(hamen,dx) km

      set l [frame $ff.lower]
      filename_hscroll_frame $l gm "Grid Model (gm) file" gbl(file,gmfile) 40\
	  load { {"gm"   *.gm} {"All"   *} }
      filename_hscroll_frame $l cdf "netCDF (grd) file" gbl(file,cdffile) 40\
	  load { {"grd"  *.grd} {"All"   *} }
      pack $u $l -side top -expand 1 -fill x

      # model size
      set ms [labelframe $wf.modelsize -text "model size"]
      foreach {i j} {xmin "X min" xmax "X max" \
			 zmin "depth min" zmax "depth max"} {
	 make_entry $ms $i $j wk($i) Km
      }
      make_command_button $ms update "" "update" update_model_size

      # packing ------------------------------
      pack $ff $ms -side top -pady 5 -padx 1 -ipady 5 -fill x
      pack $wf -side top -expand 1 -fill x
   }

   # --------------------------------------------------------------------
   # plot parameter tab
   # --------------------------------------------------------------------
   proc plot_parameter {ts} {
      # Global variables ------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      set w [frame $ts.plot]
      $ts tab conf "Plot" -window $w -fill x

      # ------------------------------------------------------------
      # parameter frame --------------------------------------------
      set wf [frame $w.param]

      # plot style of tx-graph
      set ps [labelframe $wf.plotstyle -text "plot style of T-X graph"]
      make_menu_com $ps itx "T-X plot" gbl(plot,tx) ""\
	  "no plot" 0 "with line" 1 "with symbols" 2
      make_entry $ps vred "reduction velocity" gbl(plot,vred) "km/sec."
      make_checkbutton $ps "Show observed picks" idata gbl(param,plot,idata)
      make_entry $ps observe_thinout "Plot observed picks every" \
	  gbl(plot,observe_thinout) "picks"

      # plot sytle of rays
      set rs [labelframe $wf.raystyle -text "plot style of raypaths"]
      make_menu_com $rs ircol "Ray color" gbl(plot,ray_color)\
	  "" "Black" 0 "Ray group" 3 "Shot(OBS)" 2

      # graph axis
      set ga [labelframe $wf.graphaxis -text "T-X graph axis"]
      make_entry $ga tmin "Time min" gbl(tmin) sec
      make_entry $ga tmax "Time max" gbl(tmax) sec
      make_entry $ga vred "Reduction velocity" gbl(plot,vred) "km/sec."
      make_menu_com $ga itrev "Time axis direction" gbl(plot,taxis_dir)\
	  {} Upward 0 Downward 1

      pack $ps $rs $ga -side top -fill x -pady 10 -padx 10 -ipady 5

      # packing ------------------------------
      pack $wf -side top -expand 1 -fill x
   }

   # --------------------------------------------------------------------
   # trace parameter tab
   # --------------------------------------------------------------------
   proc trace_parameter {ts} {
      # Global variables ------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      set w [frame $ts.trace]
      $ts tab conf "RayTrace" -window $w -fill x

      # ------------------------------------------------------------
      # parameter frame --------------------------------------------
      set wf [frame $w.param]

      # ray types
      set rt [labelframe $wf.raytypes -text "Ray types for calculation"]
      make_checkbutton $rt "First arrival calculation" first gbl(hamen,first)
      make_command_button $rt raygroup "Reflection waves"\
	  "config" "ray_parameter"

      # Shot (receiver)
      set sh [labelframe $wf.shot -text "Shots (or hydrophone/geophone)"]
      make_entry $sh dsx "spacing (dsx)" gbl(hamen,dsx)\
	  "km (if dsx<0 then dsx=dx)"
      make_spinbox $sh rayout "show raypath every" gbl(hamen,rayout)\
	  0 100 1 "shots (0:none, 1:all, 2:half)"

      set ff [labelframe $wf.filenames -text "output files"]
      make_menu_com $ff txformat "T-X data format" gbl(hamen,txformat)\
	  "" "MacRay(pick)" 2 "Traveltime-field" 3 "(x,t)" 1 "None" 0
      filename_hscroll_frame $ff tx "T-X file(OUTPUT)" \
	  gbl(file,calculated) 37 save \
	  { {"macray" *.macray} {"pick" *.pick} {"dat" *.dat} {"All"   *} }
      filename_hscroll_frame $ff ray "raypath file(OUTPUT)" gbl(file,rayout) 37\
	  save { {"dat"   "*.dat"}  {"out"    *.out} {"All"   *} }

      # packing ------------------------------
      pack $rt $sh $ff -side top -fill x -pady 5 -padx 3 -ipady 5
      pack $wf -side top -expand 1 -fill x
   }
}