# --------------------------------------------------------------------
# help_mousebind -----------------------------------------------------
#            help for mouse bindings
# --------------------------------------------------------------------
proc help_mousebind {} {
   # Global  --------------------------------------------------
   global sys gbl
   
   # Main --------------------------------------------------
   # Make window?
   set w .mousebind
   set hw $w.help
   if { [winfo exists $w] } {
      # Already exists
      raise $w
   } else {
      # Newly invoke help window
      toplevel $w
      wm title $w "HELP : Mouse Bindings"
      text $hw -width 45 -height 15 -font help_font
      $hw tag configure keycolor -foreground red
      
      # Button frame
      frame $w.button
      button $w.button.ok -text "Dismiss" -width 15 -command "destroy $w"
      pack $w.button.ok -side left -expand 1
      
      # Packing
      pack $hw $w.button -side top -fill x
   }
   
   # Help area ------------------------------
   update_help_keybind
}
# --------------------------------------------------------------------
# update_help_keybind ------------------------------------------------
#            Mouse and Key bind help, update.
# --------------------------------------------------------------------
proc update_help_keybind {} {
   # Global  --------------------------------------------------
   global wk
   
   # Main --------------------------------------------------
   set w .mousebind
   set hw $w.help
   if { ![winfo exists $w] } {
      # Non help window.
      return
   }
   
   $hw configure -state normal
   $hw delete 0.0 end
   switch -- $wk(current_bindings) {
      "Node" {
	 foreach {a b c} "\
	    { }        Button1   {Select Interface}\
	    { }        Button2   {Edit a Velocity Grid}\
	    { }        Button3   {Move a Node}\
	    {} {} {}\
	    Shift-     Button1   {Add a Node}\
	    Shift-     Button2   {Delete a Node}\
	    Shift-     Button3   {Move a Node}\
	    {} {} {}\
	    Double-    Button1   {Edit a Node}\
            Double-    Button3   {Move a Velocity Grid}\
	    {} {} {}\
	    Control-   Button1   {Zoom in}\
	    Control-   Button2   {Undo}\
	    Control-   Button3   {Zoom out}" {
	    eval {$hw insert end [format "%10s%7s" $a $b] keycolor}
	    $hw insert end [format "  :  %s\n" $c]
	 }
      }
      "V-Grid" {
	 foreach {a b c} "\
	    { }        Button1   {Select Interface}\
	    { }        Button2   {Edit a Velocity Grid}\
	    { }        Button3   {Move a Velocity Grid}\
	    {} {} {}\
	    Shift-     Button1   {Add a Velocity Grid}\
	    Shift-     Button2   {Delete a Velocity Grid}\
	    Shift-     Button3   {Move a Velocity Grid}\
	    {} {} {}\
	    Double-    Button1   {Edit a Node}\
            Double-    Button3   {Move a Velocity Grid}\
	    {} {} {}\
	    Control-   Button1   {Zoom in}\
	    Control-   Button2   {Undo}\
	    Control-   Button3   {Zoom out}" {
	    eval {$hw insert end [format "%10s%7s" $a $b] keycolor}
	    $hw insert end [format "  :  %s\n" $c]
	 }
      }
   }
   $hw configure -state disabled
}

# --------------------------------------------------------------------
# help_history -------------------------------------------------------
#            Show program development history
# --------------------------------------------------------------------
proc help_history {} {
   # global --------------------------------------------------
   global sys
   
   # main --------------------------------------------------
   set h .history
   catch {destroy $h}
   toplevel $h
   wm title $h "History of Modeling application"
   wm resizable $h 0 1
   
   # show ----------
   display_help  $h $sys(HISTORY_FILE)
}
# --------------------------------------------------------------------
# help_file_format ---------------------------------------------------
#            Description about data format
# --------------------------------------------------------------------
proc help_file_format {} {
   # global --------------------------------------------------
   global sys
   
   # main --------------------------------------------------
   set h .help_file_format
   catch {destroy $h}
   toplevel $h
   wm title $h "Describe File Formats"
   wm resizable $h 0 1
   
   # show ----------
   display_help $h $sys(FILE_FORMAT)
}

# --------------------------------------------------------------------
# display_help -------------------------------------------------------
#            Show help data from a text file
# --------------------------------------------------------------------
#   h         : widget name to display help
#   filename  : a text file to show
# --------------------------------------------------------------------
proc display_help {h filename} {
   # main --------------------------------------------------
   frame $h.help_frame
   text $h.help_frame.text -width 80 -height 20\
       -yscrollcommand "$h.help_frame.scroll set" -setgrid 1 
   scrollbar $h.help_frame.scroll -command "$h.help_frame.text yview"
   pack $h.help_frame.text $h.help_frame.scroll -side left -fill y
   
   # read the help file
   set fp [open $filename]
   while {![eof $fp]} {
      $h.help_frame.text insert end [read $fp 10000]
   }
   close $fp
   $h.help_frame.text configure -state disabled
   
   # OK button ----------
   button $h.ok -text "OK" -width 15 -command "destroy $h"
   
   # packing
   pack $h.help_frame -side top -expand yes -fill both
   pack $h.ok -side top
}
# --------------------------------------------------------------------
# bug_report ---------------------------------------------------------
#            show bug-report information
# --------------------------------------------------------------------
proc bug_report {} {
   # global --------------------------------------------------
   global sys gbl

   # main --------------------------------------------------
   # invoke window
   set w .bug_report
   catch {destroy $w}
   toplevel $w
   wm title $w "BUG Report"
   
   # show info ------------------------------
   set hw $w.text
   text $hw -width 80 -height 10
   $hw tag configure BOLD -font help_font
   
   $hw insert end "Please send bug reports to\n\n"
   $hw insert end "\t fujie@jamstec.go.jp\n" BOLD
   $hw configure -state disabled
   
   # button ------------------------------
   frame $w.button
   button $w.button.ok -text "Dismiss" -width 15 -command "destroy $w"
   pack $w.button.ok -side left -expand 1
   
   # packing
   pack $hw $w.button -side top -fill x
}
# --------------------------------------------------------------------
# show_version -------------------------------------------------------
#            Show version and logo image
# --------------------------------------------------------------------
proc show_version {} {
   # global --------------------------------------------------
   global sys gbl
   
   # main --------------------------------------------------
   # invoke new window
   set w .shou_version
   catch {destroy $w}
   toplevel $w
   wm title $w "Modeling: Version"
   
   # Copyright  ------------------------------
   label $w.copyright -text "(C)1998-2008, FUJIE Gou."\
       -relief groove -bd 2 -anchor w
   
   # generate logo image
   image create photo logo -file $sys(logo_image)
   label $w.logo -image logo -relief groove -bd 5
   
   # show version ------------------------------
   label $w.version -text "Version $sys(VERSION)"\
       -font bold_font -relief groove -bd 2
   
   # buttons ------------------------------
   frame $w.button
   button $w.button.ok -text "Dismiss" -width 15 -command "destroy $w"
   pack $w.button.ok -side left -expand 1
   
   # packing
   pack $w.copyright $w.logo $w.version $w.button -side top -fill x
}