proc initialize {} {
   global sys gbl wk custom tcl_version

   # --------------------------------------------------------------------
   # System dependent definition
   # --------------------------------------------------------------------
   set sys(SOURCE_IMAGE) "$sys(INSTALL_DIR)/source.xbm"
   set sys(HISTORY_FILE) "$sys(INSTALL_DIR)/HISTORY.txt"
   set sys(FILE_FORMAT)  "$sys(INSTALL_DIR)/FORMAT.txt"
   set sys(logo_image)   "$sys(INSTALL_DIR)/logo.gif"

   set sys(MAX_LAYER) 20
   set sys(MAX_UNDO)  100
   set sys(MAX_COMPLEX_WAVE) 5

   set sys(colors) {"\#ff00ff" "\#00ff00" "\#00ffff" "\#e0e000"\
			"\#800080" "\#008000" "\#d0d040"}
   set sys(ncolor) [llength $sys(colors)]
   image create bitmap source_image -file $sys(SOURCE_IMAGE) -foreground green

   # --------------------------------------------------------------------
   # font setting
   # --------------------------------------------------------------------
   declare_fonts
   
   # --------------------------------------------------------------------
   # Common widgets options
   # --------------------------------------------------------------------
   # tk setting
   if { $tcl_version >= 8.3 } {
      tk useinputmethods 0
   }

   set bgcolor [option get . background background]
   if { $bgcolor != "" } {
      option add *selectBackground         $bgcolor
   }
   option add *Graph.title                 "Velocity Structure"
   option add *Graph.x.title		   "Distance(Km)"
   option add *Graph.y.title		   "Depth(Km)" 
   option add *Graph.TextMarker.background ""
   option add *Graph.TextMarker.Font       [lindex $sys(VELOCITY_FONTS) 3]
   option add *Graph.TextMarker.Under      true
   option add *Graph.Axis.subdivisions     5
   option add *Button.Foreground           "black"
   option add *Button.Background           "#d0ffff"
   option add *Button.activeBackground     "#80ffff"
   option add *Button.activeForeground     "red"
   option add *Radiobutton.activeForeground "red"
   option add *Button.Cursor               "hand2"
   option add *Checkbutton.Cursor          "hand2"
   option add *Text.Cursor                 "left_ptr"
   option add *Entry.Cursor                "pencil"
   option add *Entry.Background            "white"
   option add *Entry.selectBackground      "navyblue"
   option add *Menubutton.Cursor           "hand2"
   option add *Listbox.Background          "white"
   option add *Spinbox.Background          "white"
   option add *Spinbox.selectBackground    "navyblue"

   # --------------------------------------------------------------------
   # Global variable declaration
   # --------------------------------------------------------------------

   set gbl(backup,dir) "backup"
   set gbl(backup,listmax) 15
   set gbl(backup,oldfiles) 30
   set gbl(backup,olddays) 14

   # ------------------------------
   # file names
   # ------------------------------
   set gbl(file,calculated) "./tx-out.macray"
   set gbl(file,gmfile) ""
   set gbl(file,cdffile) ""
   set gbl(file,cptfile) "./vel.cpt"
   set gbl(file,rayout) "./ray.dat"
   set gbl(file,wavelet) "./wavelet.wav"
   set gbl(file,bgimage) ""
   set gbl(file,bgcdf) ""
   set gbl(file,bgcdfcpt) ""

   # ------------------------------
   # velocity structure 
   # ------------------------------
   for {set i 1} { $i <= $sys(MAX_LAYER) } { incr i } {
      global vec_x$i vec_z$i
   }
   set gbl(n_interface) 0
   set gbl(xmin) 0.0
   set gbl(xmax) 10.0
   set gbl(zmin) 0.0
   set gbl(zmax) 20.0
   for {set i 1} {$i <= $sys(MAX_LAYER)} {incr i} {
      vector create vec_x$i vec_z$i
   }
   set gbl(tmin) 0.0
   set gbl(tmax) 10.0

   # Converted/Multiple wave related
   for {set i 1} {$i <= $sys(MAX_COMPLEX_WAVE)} {incr i} {
      foreach {j k} {\
			 onoff 0 \
			 mode "     Refract in          "\
			 layer 1 \
			 interface 0,1 \
			 rbnd "" \
		      } {
	 set gbl(ray,convert,$i,$j) $k
      }
   }

   # Option related
   set gbl(option,option_key) "Alt"
   set gbl(option,exec_condition) 0
   set gbl(option,vel) 0
   set gbl(option,source) 1
   set gbl(option,label) 1
   set gbl(option,grid) 1
   set gbl(option,label_font) small
   set gbl(option,x2_axis) 1
   set gbl(option,y2_axis) 1
   set gbl(option,tauto) 1
   set gbl(option,tfrom) 0
   set gbl(option,tto) 19
   set gbl(option,x_axis) 1
   set gbl(option,y_axis) 1
   set gbl(option,t_axis) 1
   set gbl(option,t_tic) ""
   set gbl(option,backend) rayinvr
   set gbl(option,use_relative_path) 1

   # ------------------------------
   # source (receiver) parameters
   # ------------------------------
   set gbl(src,n_source) 5
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      foreach {j k} {x "" z "" name "" onoff 0 pick ""} {
	 set gbl(src,$i,$j) $k
      }
   }
   set gbl(src,single) 1

   # common plot parameters
   set gbl(plot,pick_color) 0
   set gbl(plot,pick_acc)   0.100
   set gbl(plot,ray_color)  3
   set gbl(plot,taxis_dir)  0
   set gbl(plot,observe_thinout) 1
   set gbl(plot,vred) 0.0
   set gbl(plot,tx) 1

   # common trace parameters
   set gbl(trace,2pt) "0"        ; # flag for two-point raytracing
   set gbl(trace,dx2pt) "0.5"    ; # delta-x for automatic two-point raytracing

   # background image
   set gbl(bg,cpt,mode) 0
   set gbl(bg,cpt,master) "gray"
   set gbl(bg,cpt,inverse) 0
   set gbl(bg,contour) 0.5
   set gbl(bg,density) 72

   # Extension for Ray86 format for Inversion
   set gbl(ERI,inv) 0
   set gbl(ERI,poisson) 0
   set gbl(ERI,poisson_orig) 0

   # Movable Interface.
   for {set i 1} {$i <= $sys(MAX_LAYER)} {incr i} {
      set gbl(ERI,movable,$i) 0
      set gbl(ERI,movable_sd,$i) 0.0
   }

   # ------------------------------
   # working variables
   # ------------------------------
   set wk(file,project) ""
   set wk(undo,n) 0
   set wk(current_bindings) Node
   set wk(halo) 2                  ; # mouse pick tolerance
   set wk(selected) -1             ; # selected interface number

   # color
   set wk(col,normal) blue
   set wk(col,tmp) purple
   set wk(col,select) red

   set wk(mark,size) 3
   set wk(mark,select_size) 6

   # for plotting raypaths
   set wk(ray,n)  0
   set wk(ray,ray_list) ""
   set wk(ray,color_list) ""
   set wk(ray,n_color) 0

   # default values
   set wk(def,poi) 0.250          ; # default for Poisson's ratio
   set wk(def,SDps) 1.732         ; # default for (SD of Vp)/(SD of Vs)

   set wk(read_status) 0

   # widget paths
   set wk(w,bar) .message_bar
   set wk(w,pos) .position_bar
   set wk(w,backend_menu) ""
   set wk(w,parameter_menu) ""
   set wk(w,src_param) .src_param

   # Custom
   set custom(DismissButtonBackground) "#d0ffff"

   # fail-safe
   RAYPATH::define_color     ; # for fail-safe
}

proc declare_fonts {} {
   global sys
   
   # Which font family do you have?
   set fl [font families]
   foreach fn {{times new roman} arial utopia times helvetica fixed courier} {
      if { [lsearch -exact $fl $fn] >= 0 } {
	 break
      }
   }
   
   # bold font
   font create bold_font -family $fn -size 18 -weight bold

   # help font
   font create help_font -family fixed -size 10 -weight bold
   
   # velocity font
   set sys(VELOCITY_FONTS) {\
			tiny   "-*-*-*-*-*-*-7-*-*-*-*-*-iso8859-*" \
			small  "-*-fixed-*-*-*-*-10-*-*-*-*-*-iso8859-*"\
			medium "-*-times-*-*-*-*-14-*-*-*-*-*-iso8859-*"\
			large  "-*-times-*-*-*-*-18-*-*-*-*-*-iso8859-*"}
}
