# --------------------------------------------------------------------
# select_interface ---------------------------------------------------
#            Select interface nearest mouse cursor.
# --------------------------------------------------------------------
# [Arguments] 
#   x  :  X coordinate of Mouse.
#   y  :  Y coordinate of Mouse.
# --------------------------------------------------------------------
proc select_interface {x y} {
    # Global variables --------------------------------------------------
    global gbl sys wk
    
    # Main --------------------------------------------------
    # Get the closest point --------------------
    $sys(graph) element conf graph_frame -hide yes
    $sys(graph) element closest $x $y point -interpolate yes
    $sys(graph) element conf graph_frame -hide no
    if {![string match "interface*" $point(name)]} {
	# if the closest element is not interface, do nothing.
	return
    }
    unselect_interface
    # Get interface number -----
    set wk(selected) [string trim $point(name) "interface()"]
    selection_flash
    # report message -----
    bar_message "Interface $wk(selected) is selected."
}
# --------------------------------------------------------------------
# unselect_interface -------------------------------------------------
#           Un-highlight the current interface
# --------------------------------------------------------------------
proc unselect_interface {} {
    # Global variables --------------------------------------------------
    global gbl sys wk

    # Main --------------------------------------------------
    if {[$sys(graph) element exist interface($wk(selected))]} {
	# Un-highlight all the selected interfaces.
	$sys(graph) element conf interface($wk(selected)) -pi $wk(mark,size)\
	    -outline $wk(col,normal) -fill $wk(col,normal) -colo $wk(col,normal)
    }
}

# --------------------------------------------------------------------
# selection_flash ----------------------------------------------------
#           Highlight the selected interfaces
# --------------------------------------------------------------------
proc selection_flash {} {
    # Global variables --------------------------------------------------
    global gbl sys wk

    # Main --------------------------------------------------
    $sys(graph) elem conf interface($wk(selected)) -fill $wk(col,select)\
	    -outline $wk(col,select) -pix $wk(mark,select_size)\
	    -color $wk(col,select)
}
# --------------------------------------------------------------------
# insert_interface ---------------------------------------------------
#            Insert new interface
# --------------------------------------------------------------------
# [Arguments]
#   mode  :  Insert mode, "up" or "down"
# --------------------------------------------------------------------
proc insert_interface {mode} {
   # Global variables --------------------------------------------------
   global gbl sys wk
   for {set i 1} {$i < $sys(MAX_LAYER)} {incr i} {
      global vec_x$i vec_z$i
   }
   
   # Main --------------------------------------------------
   # Check the total interface number --------------------
   if {[expr $sys(MAX_LAYER) - $gbl(n_interface) - 1] <= 0} {
      bar_message "Too many interfaces.  Can't make any more interfaces!"
      return
   }
   # Check Insert mode --------------------
   if ![string compare $mode "up"] {
      # Insert upside
      if {$wk(selected) <= 1} {
	 bar_message "Can't insert a new interface above the top."
	 return
      }
      set downside $wk(selected)
   } else {
      # Insert downside
      if {$wk(selected) >= $gbl(n_interface)} {
	 bar_message "Can't insert a new interface beneath the bottom."
	 return
      }
      set downside [expr $wk(selected) + 1]
   }
   set upside [expr $downside - 1]
   # Undo information
   set_interface_undo_info interface.create $downside
   
   # shift interfaces and layers
   shift_interface $wk(selected) down
   
   # Give appropriate velocity values to new layer --------------------
   #     ----- Make constant velocity layer.
   switch -- $mode {
      "up" {
	 foreach j {vx vbot n_vnode} {
	    set gbl($j,$downside) $gbl($j,$upside)
	 }
	 foreach j {vtop vbot} {
	    set gbl($j,SD,$downside) $gbl($j,SD,$upside)
	 }
	 set gbl(vbot,$upside) $gbl(vtop,$upside)
	 set gbl(vtop,$downside) $gbl(vbot,$downside)
	 set gbl(poi,$downside,bot) $gbl(poi,$upside,bot)
	 set gbl(poi,SD,$downside,bot) $gbl(poi,SD,$upside,bot)
	 set gbl(poi,$upside,bot) $gbl(poi,$upside,top)
	 set gbl(poi,SD,$upside,bot) $gbl(poi,SD,$upside,top)
	 set gbl(poi,$downside,top) $gbl(poi,$downside,bot)
	 set gbl(poi,SD,$downside,top) $gbl(poi,SD,$downside,bot)
      }
      "down" {
	 set gbl(vbot,$upside) $gbl(vtop,$upside)
	 set gbl(vtop,$downside) $gbl(vbot,$downside)
	 set gbl(vbot,SD,$upside) $gbl(vtop,SD,$upside)
	 set gbl(vtop,SD,$downside) $gbl(vbot,SD,$downside)
	 set gbl(poi,$upside,bot) $gbl(poi,$upside,top)
	 set gbl(poi,$downside,top) $gbl(poi,$downside,bot)
	 set gbl(poi,SD,$upside,bot) $gbl(poi,SD,$upside,top)
	 set gbl(poi,SD,$downside,top) $gbl(poi,SD,$downside,bot)
      }
   }
   # Interpolate the new interface depth --------------------
   set gbl(z,$downside) ""
   for {set i 0} {$i < $gbl(n_node,$downside)} {incr i} {
      set x [lindex $gbl(x,$downside) $i]
      lappend gbl(z,$downside)\
	  [format "%.3f" \
	   [expr ([interpolate_depth $upside $x] +\
		      [interpolate_depth [expr $downside + 1] $x])/2.0]]
   }
   switch -- $mode {
      "up" {
	 set gbl(z,SD,$downside) $gbl(z,SD,[expr $downside + 1])
      }
      "down" {
	 set gbl(z,SD,$downside) $gbl(z,SD,$upside)
      }
   }

   # Declare ray  group for new interface(layer)
   foreach i {refract reflect head} {
      set gbl(ray,$i,$gbl(n_interface)) 0
   }
   
   # Selecte new interface
   set wk(selected) $downside
   incr gbl(n_interface)
   
   # Redraw --------------------
   draw_model
}
# --------------------------------------------------------------------
# delete_interface ---------------------------------------------------
#            Delte an interface
# --------------------------------------------------------------------
proc delete_interface {} {
    # Global variables --------------------------------------------------
    global gbl sys wk
    for {set i 1} {$i < $sys(MAX_LAYER)} {incr i} {
	global vec_x$i vec_z$i
    }
    
    # Main --------------------------------------------------
    # Check --------------------
    if {$gbl(n_interface) <= 2} {
	bar_message "You can't delete any more interfaces!"
	return
    }
    # Confirmation window --------------------
    set ans [tk_messageBox -type yesno -title "Delete or not?"\
		 -message "Delete interface $wk(selected). Are you sure?"\
		 -icon question]
    if {$ans=="no"} {
	# Canceled
	bar_message "Delete canceled."
	return
    }
    # Undo information --------------------
    set_interface_undo_info interface.delete $wk(selected)

    # Give appropriate velocity to the layer which is newly compounded.
    set upper [expr $wk(selected) - 1]
    set gbl(vbot,$upper) {}
    for {set i 0} {$i < $gbl(n_vnode,$upper)} {incr i} {
	lappend gbl(vbot,$upper) [lindex [calc_v_layer $wk(selected)\
					      [lindex $gbl(vx,$upper) $i]] 1]
    }

    # Exec delete interface --------------------
    # Delete velocity grid and bottom interface
    foreach name [$sys(graph) marker names m($wk(selected),*,*)] {
	$sys(graph) marker delete $name
    }
    $sys(graph) element delete interface($gbl(n_interface))

    # shift
    shift_interface $wk(selected) up

    # Select new interface
    incr gbl(n_interface) -1
    if {$wk(selected) > $gbl(n_interface)} {
	set wk(selected) $gbl(n_interface)
    }

    # Redraw --------------------
    draw_model
    bar_message "An interface was deleted. ( $gbl(n_interface) remained)"
}
# --------------------------------------------------------------------
# move_interface -----------------------------------------------------
#           Parallel displacement of interfaces.
# --------------------------------------------------------------------
proc move_interface {} {
    # Global variables --------------------------------------------------
    global sys gbl wk

    # Main --------------------------------------------------
    # You can't move top and bottom interfaces.
    if {$wk(selected) == 0 || $wk(selected) == $gbl(n_interface)} {
	bar_message "You can't move top and bottom interfaces."
	return
    }

    # Invoke window to enter the displacement distance --------------------
    set w .move_interface
    catch {destroy $w}
    toplevel $w
    wm title $w "Parallel displacement of Interface"

    # Title frame ------------------------------
    title_frame $w.title "Parallel displacement : Interface $wk(selected)"

    # Parameter frame ------------------------------
    set pf $w.parameter_frame
    frame $pf
    label $pf.label -text "Displacement :"
    entry $pf.entry -width 12
    $pf.entry insert 0 0.0
    label $pf.unit -text "Km"
    pack $pf.label $pf.entry $pf.unit -side left -in $pf

    # Botton frame ------------------------------
    frame $w.button
    # OK button
    button $w.button.ok -text "OK" -width 10\
	    -command "exec_move_interface $w ok"
    # Cancel button
    button $w.button.cancel -text "Cancel" -width 10\
	    -command "exec_move_interface $w cancel"

    # Packing of button frame
    pack $w.button.ok $w.button.cancel\
	    -in $w.button -side left -expand 1

    # Packing ------------------------------
    pack $w.title $pf $w.button -side top -fill x
    busy hold .
}
# --------------------------------------------------------------------
# exec_move_interface -----------------------------------------------
#           Exec move interface parallel
# --------------------------------------------------------------------
# []
#    w    :  Widget of set displacement value.
#    mode :  mode
# --------------------------------------------------------------------
proc exec_move_interface {w mode} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # busy --------------------
   busy release .
   
   # Return "cancel" --------------------
   if ![string compare $mode "cancel"] {
      bar_message "Interface was not moved."
      # Delete displacement window
      destroy $w
      return
   }
   
   # Get the displacement
   set dz [format "%.3f" [$w.parameter_frame.entry get]]
   
   # Undo information
   set_interface_undo_info interface.move $wk(selected)
   
   # Parallel displacement ------------------------------
   set upper [expr $wk(selected) - 1]
   set lower [expr $wk(selected) + 1]
   for {set i 0} {$i < $gbl(n_node,$wk(selected))} {incr i} {
      # move ----------
      set new_z [expr [lindex $gbl(z,$wk(selected)) $i] + $dz]
      # Check crossing of interfaces ----------
      set x [lindex $gbl(x,$wk(selected)) $i] ; # x coordinate
      set top [interpolate_depth $upper $x]    ; # upper 
      set bottom [interpolate_depth $lower $x] ; # lower
      # Check
      if {$new_z > $bottom} {
	 set new_z $bottom
      } elseif {$new_z < $top} {
	 set new_z $top
      }
      # Set new value ----------
      set gbl(z,$wk(selected)) [lreplace $gbl(z,$wk(selected))\
				    $i $i $new_z]
   }
   
   # Redraw
   draw_model
   
   # Close displacement set window
   destroy $w
   
   # message
   bar_message "Interface $wk(selected) was moved."
}
# --------------------------------------------------------------------
# set_interface_undo_info --------------------------------------------
#        Update undo information, when you edit a interface.
# --------------------------------------------------------------------
# [arguments]
#   mode  : edit mode (interface.move, interface.delete, interface.create,
#                      interface.pinchout )
#   i     : Interface number.
# --------------------------------------------------------------------
proc set_interface_undo_info {mode i} {
   # Global variables --------------------------------------------------
   global gbl wk

   # Main --------------------------------------------------
   # Common undo information
   set n $wk(undo,n)
   set wk(undo,x,$n) $gbl(x,$i)
   set wk(undo,z,$n) $gbl(z,$i)
   set wk(undo,z,SD,$n) $gbl(z,SD,$i)
   set wk(undo,interface,$n) $i
   set wk(undo,index,$n) -1
   set wk(undo,mode,$n) $mode
   # dummy values
   set wk(undo,vtop,$n) dum
   set wk(undo,vbot,$n) dum
   set wk(undo,vtop,SD,$n) dum
   set wk(undo,vbot,SD,$n) dum
   set wk(undo,poi,top,$n) dum
   set wk(undo,poi,bot,$n) dum
   set wk(undo,poi,SD,top,$n) dum
   set wk(undo,poi,SD,bot,$n) dum
   # Undo informatio which depends on edit mode.
   switch $mode {
      interface.move -
      interface.pinchout {
      }
      interface.create {
      }
      interface.delete {
	 # upper layer bottom velocity
	 set wk(undo,vtop,$n) $gbl(vbot,[expr $i-1])
	 # lower layer all v-grid information
	 set wk(undo,vbot,$n)\
	     [list $gbl(vx,$i) $gbl(vtop,$i) $gbl(vbot,$i)]
	 # upper layer bottom sd values
	 set wk(undo,vtop,SD,$n) $gbl(vbot,SD,[expr $i-1])
	 # lower layer sd values
	 set wk(undo,vbot,SD,$n)\
	     [list $gbl(vtop,SD,$i) $gbl(vbot,SD,$i)]
	 # upper layer bottom poisson's ratio and its SD
	 set wk(undo,poi,top,$n) $gbl(poi,[expr $i-1],bot)
	 set wk(undo,poi,SD,top,$n) $gbl(poi,SD,[expr $i-1],bot)
	 # lower layer poisson's ratio and its SD
	 set wk(undo,poi,bot,$n)\
	     [list $gbl(poi,$i,top) $gbl(poi,$i,bot)]
	 set wk(undo,poi,SD,bot,$n)\
	     [list $gbl(poi,SD,$i,top) $gbl(poi,SD,$i,bot)]
      }
   }
   # Update undo-list.
   incr wk(undo,n)
   undo_check
}
# --------------------------------------------------------------------
# shift_interface ----------------------------------------------------
#        Shift interface and layers iterative.
# --------------------------------------------------------------------
# [arguments]
#   inter  : Interface number
#   d      : Shift direction, "up" => upward, "down" => downward.
# --------------------------------------------------------------------
proc shift_interface {inter d} {
   # Global variables --------------------------------------------------
   global gbl sys
   for {set i 1} {$i < $sys(MAX_LAYER)} {incr i} {
      global vec_x$i vec_z$i
   }

   # Main --------------------------------------------------
   switch -- $d {
      "up" {
	 # Shift interface
	 set n $gbl(n_interface)
	 for {set i $inter} {$i < $n} {incr i} {
	    foreach j {x z z,SD n_node} {
	       set gbl($j,$i) $gbl($j,[expr $i+1])
	    }
	    set j [expr $i+1]
	    vec_x$j dup vec_x$i
	    vec_z$j dup vec_z$i
	 }
	 # Shift layer
	 for {set i $inter} {$i < [expr $n - 1]} {incr i} {
	    foreach j {vx vtop vbot n_vnode} {
	       set gbl($j,$i) $gbl($j,[expr $i + 1])
	    }
	    foreach j {vtop vbot} {
	       set gbl($j,SD,$i) $gbl($j,SD,[expr $i + 1])
	    }
	    foreach j {top bot} {
	       set gbl(poi,$i,$j) $gbl(poi,[expr $i + 1],$j)
	       set gbl(poi,SD,$i,$j) $gbl(poi,SD,[expr $i + 1],$j)
	    }
	 }
      }
      "down" {
	 # Shift interface
	 set n $gbl(n_interface)
	 for {set i $n} {$i >= $inter} {incr i -1} {
	    foreach j {x z z,SD n_node} {
	       set gbl($j,[expr $i + 1]) $gbl($j,$i)
	    }
	    set j [expr $i+1]
	    vec_x$i dup vec_x$j
	    vec_z$i dup vec_z$j
	 }
	 # Shift layer
	 for {set i $n} {$i > $inter} {incr i -1} {
	    foreach j {vx vtop vbot n_vnode} {
	       set gbl($j,$i) $gbl($j,[expr $i - 1])
	    }
	    foreach j {vtop vbot} {
	       set gbl($j,SD,$i) $gbl($j,SD,[expr $i - 1])
	    }
	    foreach j {top bot} {
	       set gbl(poi,$i,$j) $gbl(poi,[expr $i - 1],$j)
	       set gbl(poi,SD,$i,$j) $gbl(poi,SD,[expr $i - 1],$j)
	    }
	 }
      }
      default {
	 puts stderr "Error: Invalid interface shift mode."
      }
   }
}
# --------------------------------------------------------------------
# pinchout_interface -------------------------------------------------
#            Pinch out interface
# --------------------------------------------------------------------
proc pinchout_interface {} {
    # Global variables --------------------------------------------------
    global sys gbl sk wk

    # Main --------------------------------------------------
    # You can't move top and bottom interfaces.
    if {$wk(selected) == 0 || $wk(selected) == $gbl(n_interface)} {
	bar_message "You can't move top and bottom interfaces."
	return
    }

    # Invoke window to enter the displacement distance --------------------
    set w .pinchout_interface
    catch {destroy $w}
    toplevel $w
    wm title $w "Pinch Out Interface"

    # Title frame ------------------------------
    title_frame $w.title "Pinch Out : Interface $wk(selected)"

    # Parameter frame ------------------------------
    set pf $w.parameter_frame
    frame $pf
    set gbl(pinch,direction) -1
    make_many_radio $pf "Pinch out direction" direction gbl(pinch,direction)\
	upper -1 lower 1
    label $pf.rlabel -text "X Range"
    label $pf.rmiddle -text "(km) to "
    label $pf.rright -text "(km)"
    entry $pf.rfrom -width 8
    entry $pf.rto -width 8
    grid $pf.rlabel $pf.rfrom $pf.rmiddle $pf.rto $pf.rto
    
    # initial value
    $pf.rfrom insert 0 $gbl(xmin)
    $pf.rto insert 0 $gbl(xmax)

    # Botton frame ------------------------------
    frame $w.button
    # OK button
    button $w.button.exec -text "Exec" -width 10\
	    -command "exec_pinchout_interface $w exec"
    # Cancel button
    button $w.button.cancel -text "Cancel" -width 10\
	    -command "exec_pinchout_interface $w cancel"

    # Packing of button frame
    pack $w.button.exec $w.button.cancel\
	    -in $w.button -side left -expand 1

    # Packing ------------------------------
    pack $w.title $pf $w.button -side top -fill x
    busy hold .
}
# --------------------------------------------------------------------
# exec_pinchout_interface --------------------------------------------
#           Exec pinch out interface
# --------------------------------------------------------------------
# [Arguments]
#    w    :  Window name of pinch out setting window.
#    mode :  mode
# --------------------------------------------------------------------
proc exec_pinchout_interface {w mode} {
    # Global variables --------------------------------------------------
    global gbl wk

    # Main --------------------------------------------------
    # busy --------------------
    busy release .

    # Return "cancel" --------------------
    if ![string compare $mode "cancel"] {
	bar_message "Pinch out canceled"
	# Delete parent window
	destroy $w
	return
    }

    # Get the range
    set from [$w.parameter_frame.rfrom get]
    set to [$w.parameter_frame.rto get]
    
    # Check range
    if { $from >= $to || $from < $gbl(xmin) || $to > $gbl(xmax) } {
	bar_message "Invalid range, pinch out canceled."
	return
    }

    # Check direction
    set target [expr $gbl(pinch,direction) + $wk(selected)]
    if { $target < 1 || $target > $gbl(n_interface) } {
	bar_message "You cannot pinchout to that direction."
	return
    }

    # Undo information
    set_interface_undo_info interface.pinchout $wk(selected)

    # Pinch Out ------------------------------
    # Delete all the point in the range.
    set new_x {}
    set new_z {}
    set new_sd {}
    # Recover points of outside of range
    for {set i 0} {$i < $gbl(n_node,$wk(selected))} {incr i} {
       set x [lindex $gbl(x,$wk(selected)) $i]
       if { $x < $from || $to < $x } {
	  lappend new_x $x
	  lappend new_z [lindex $gbl(z,$wk(selected)) $i]
	  lappend new_sd [lindex $gbl(z,SD,$wk(selected)) $i]
       }
    }
    
    # Copy all the point in the range.
    for {set i 0} {$i < $gbl(n_node,$target)} {incr i} {
	set x [lindex $gbl(x,$target) $i]
	if { $from <= $x && $x <= $to } {
	    # search appropriate locatin to insert
	    for {set j 0} {$j < [llength $new_x]} {incr j} {
		if { [lindex $new_x $j] > $x } {
		    break
		}
	    }
	    # insert
	    set new_x [linsert $new_x $j $x]
	    set new_z [linsert $new_z $j [lindex $gbl(z,$target) $i]]
	    set new_sd [linsert $new_sd $j [lindex $gbl(z,SD,$target) $i]]
	}
    }
    set gbl(x,$wk(selected)) $new_x
    set gbl(z,$wk(selected)) $new_z
    set gbl(z,SD,$wk(selected)) $new_sd
    set gbl(n_node,$wk(selected)) [llength $new_x]

    # Redraw
    draw_model

    # Close displacement set window
    destroy $w

    # message
    bar_message "Interface $wk(selected) was pinched out."
}
# --------------------------------------------------------------------
# interpolate_depth --------------------------------------------------
#            Compute depth of $i-th interface at position $x
#             (Linear interpolation)
# --------------------------------------------------------------------
# [arguments]
#  i          : interface number
#  x          : x coordinate.
# --------------------------------------------------------------------
proc interpolate_depth {i x} {
   # Global variables --------------------------------------------------
   global vec_x$i vec_z$i
   
   # Main --------------------------------------------------
   vector create wx wz
   # Search the velocity grid of both end of $x.
   vec_x$i dup wx
   wx variable wx
   wz expr "wx - $x"
   # avoid BLT bugs.....,  "$wz(min)" sometimes fails...
   set n1 [lindex [wz search -1.0e4 0.0] end]
   if {[expr $n1 + 1] < [wx length]} {
      set n2 [expr $n1 + 1]
   } else {
      set n2 [expr $n1 - 1]
   }
   vec_z$i dup wz
   wz variable wz
   return [expr ($wz($n2)*($x - $wx($n1)) + $wz($n1)*($wx($n2) - $x))\
	       / ( $wx($n2) - $wx($n1) )]
}

# --------------------------------------------------------------------
# interpolate_zSD ----------------------------------------------------
#         Calculate SD of node depth at $i-th interface at position $x
# --------------------------------------------------------------------
# [arguments]
#  i          : interface number
#  x          : x coordinate.
# --------------------------------------------------------------------
proc interpolate_zSD {i x} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   for {set j 1} {$j < [expr $gbl(n_node,$i) -1]} {incr j} {
      if {$x <= [lindex $gbl(x,$i) $j]} {
	 break
      }
   }
   # "x" is between node "j-1" and "j".
   set j1 [expr $j - 1]
   set j2 $j
   set x1  [expr $x - [lindex $gbl(x,$i) $j1]]
   set x2  [expr [lindex $gbl(x,$i) $j2] - $x]
   set sd1 [lindex $gbl(z,SD,$i) $j1]
   set sd2 [lindex $gbl(z,SD,$i) $j2]
   return [expr ($sd2 * $x1 + $sd1 * $x2) / ($x1 + $x2) ]
}

# --------------------------------------------------------------------
# read_interface -----------------------------------------------------
#         Read interface from (x,z) data file
# --------------------------------------------------------------------
proc read_interface {} {
   # Global variables --------------------------------------------------
   global gbl wk

   # Main --------------------------------------------------
   # select (x, z) file
   set types {
      {"All files"		*}
      {"xz files"	{.xz}	}
      {"dat files"	{.dat}	}
   }
   set fn [tk_getOpenFile -filetypes $types -parent .]
   if {$fn == ""} {
      bar_message "No (x,z) interface file is selected."
      return
   }
   
   # read the file
   set fp [open $fn]
   set xz {}
   while { [gets $fp line] != -1} {
      if { [llength $line] < 2 } { continue }
      set x [lindex $line 0]
      if { $x < $gbl(xmin) || $x > $gbl(xmax) } {
	 continue
      }
      lappend xz $line
   }
   # sort
   set xz [lsort -real -index 0 -unique $xz]
   set xarray {}
   set zarray {}
   foreach {d} $xz {
      lappend xarray [lindex $d 0]
      lappend zarray [lindex $d 1]
   }
   # edge
   if { [lindex $xarray 0] > $gbl(xmin) } {
      set xarray "$gbl(xmin) $xarray"
      set zarray [format "%f %s" [lindex $zarray 0] $zarray]
   }
   if { [lindex $xarray end] < $gbl(xmax) } {
      lappend xarray $gbl(xmax)
      lappend zarray [lindex $zarray end]
   }
	
   # determine layer
   set z [lindex $zarray 0]
   for {set i 1} { $i <= $gbl(n_interface)} {incr i} {
      if { $z < [lindex $gbl(z,$i) 0] } {
	 break
      }
   }
   if {$i <= 0} {
      tk_messageBox -icon error -type ok \
	  -message "You cannot insert interface above the surface"
      return
   }
   if {$i > $gbl(n_interface)} {
      tk_messageBox -icon error -type ok \
	  -message "You cannot insert interface below the bottom"
      return
   }
   
   # insert ------------------------------
   # undo
   set_interface_undo_info interface.create $i
   # shift
   shift_interface $i down

   # Give appropriate velocity values to new layer
   #     ----- Make constant velocity layer.
   set upside [expr $i - 1]
   foreach j {vx vbot n_vnode} {
      set gbl($j,$i) $gbl($j,$upside)
   }
   foreach j {vtop vbot} {
      set gbl($j,SD,$i) $gbl($j,SD,$upside)
   }
   set gbl(vbot,$upside) $gbl(vtop,$upside)
   set gbl(vtop,$i) $gbl(vbot,$i)
   set gbl(poi,$i,bot) $gbl(poi,$upside,bot)
   set gbl(poi,SD,$i,bot) $gbl(poi,SD,$upside,bot)
   set gbl(poi,$upside,bot) $gbl(poi,$upside,top)
   set gbl(poi,SD,$upside,bot) $gbl(poi,SD,$upside,top)
   set gbl(poi,$i,top) $gbl(poi,$i,bot)
   set gbl(poi,SD,$i,top) $gbl(poi,SD,$i,bot)

   # give coordinates
   set gbl(x,$i) $xarray
   set gbl(z,$i) $zarray
   set gbl(n_node,$i) [llength $xarray]
   set gbl(z,SD,$i) ""
   set default_sd 0.0
   for {set j 0} { $j < [llength $gbl(z,$i)] } { incr j } {
      lappend gbl(z,SD,$i) $default_sd
   }

   # Declare ray  group for new interface(layer)
   foreach j {refract reflect head} {
      set gbl(ray,$j,$gbl(n_interface)) 0
   }
   
   # Selecte new interface
   set wk(selected) $i
   incr gbl(n_interface)
   
   # Redraw --------------------
   draw_model
}

# --------------------------------------------------------------------
# read_layer ---------------------------------------------------------
#         Read layer from a (x, vtop, vbot) file
# --------------------------------------------------------------------
proc read_layer {} {
   # Global variables --------------------------------------------------
   global gbl wk

   # Main --------------------------------------------------
   # valid interface?
   if { $wk(selected) == $gbl(n_interface) } {
      tk_messageBox -icon warning -type ok \
	  -message "You cannot edit below the bottom."
      return
   }

   # select (x, z) file
   set types {
      {"All files"		*}
      {"dat files"	{.dat}	}
      {"xz files"	{.xvv}	}
   }
   set fn [tk_getOpenFile -filetypes $types -parent .]
   if {$fn == ""} {
      bar_message "No (x,vtop,vbot) file is selected."
      return
   }
   
   # read the file
   set fp [open $fn]
   set xvv {}
   while { [gets $fp line] != -1} {
      if { [llength $line] < 3 } { continue }
      set x [lindex $line 0]
      if { $x < $gbl(xmin) || $x > $gbl(xmax) } {
	 continue
      }
      lappend xvv $line
   }
   # sort
   set xvv [lsort -unique -index 0 -real $xvv]
   set xarray {}
   set vtoparray {}
   set vbotarray {}
   foreach {d} $xvv {
      lappend xarray [lindex $d 0]
      lappend vtoparray [lindex $d 1]
      lappend vbotarray [lindex $d 2]
   }
   
   # edge
   if { [lindex $xarray 0] > $gbl(xmin) } {
      set xarray "$gbl(xmin) $xarray"
      set vtoparray [format "%f %s" [lindex $vtoparray 0] $vtoparray]
      set vbotarray [format "%f %s" [lindex $vtoparray 0] $vbotarray]
   }
   if { [lindex $xarray end] < $gbl(xmax) } {
      lappend xarray $gbl(xmax)
      lappend vtoparray [lindex $vtoparray end]
      lappend vbotarray [lindex $vbotarray end]
   }
	
   # Replace ------------------------------
   # undo
   set layer [expr $wk(selected)]
   set_vgrid_undo_info vgrid.all_layer $layer -2

   set orig_vtop_SD [lindex $gbl(vtop,SD,$layer) 0]
   set orig_vbot_SD [lindex $gbl(vbot,SD,$layer) 0]
   set orig_poi_top [lindex $gbl(poi,$layer,top) 0]
   set orig_poi_bot [lindex $gbl(poi,$layer,bot) 0]
   set orig_SDps_top [lindex $gbl(poi,SD,$layer,top) 0]
   set orig_SDps_bot [lindex $gbl(poi,SD,$layer,bot) 0]

   set gbl(vx,$layer) $xarray
   set gbl(vtop,$layer) $vtoparray
   set gbl(vbot,$layer) $vbotarray

   set gbl(n_vnode,$layer) [llength $xarray]

   set gbl(vtop,SD,$layer) {}
   set gbl(vbot,SD,$layer) {}
   set gbl(poi,$layer,top) {}
   set gbl(poi,$layer,bot) {}
   set gbl(poi,SD,$layer,top) {}
   set gbl(poi,SD,$layer,bot) {}
   for {set i 0} {$i < $gbl(n_vnode,$layer)} {incr i} {
      lappend gbl(vtop,SD,$layer) $orig_vtop_SD
      lappend gbl(vbot,SD,$layer) $orig_vbot_SD
      lappend gbl(poi,$layer,top) $orig_poi_top
      lappend gbl(poi,$layer,bot) $orig_poi_bot
      lappend gbl(poi,SD,$layer,top) $orig_SDps_top
      lappend gbl(poi,SD,$layer,bot) $orig_SDps_bot
   }

   # Redraw --------------------
   draw_model
}