# --------------------------------------------------------------------
# Display message in the message value.
# --------------------------------------------------------------------
proc bar_message {message} {
    # Global variables --------------------------------------------------
    global wk

    # Main --------------------------------------------------
    $wk(w,bar).message configure -text $message
    update
}
# --------------------------------------------------------------------
# Quit application
# --------------------------------------------------------------------
proc quit_application {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # Save or not?
   if {$gbl(n_interface) > 0} {
      # query
      set ans [tk_messageBox -title "Save Project?" -type yesnocancel\
		   -message "Do you want to save your Project?"\
		   -icon question]
      if {$ans=="yes"} {
	 save_project overwrite
      } elseif {$ans=="cancel"} {
	 return
      }
   }
   destroy .
}
# --------------------------------------------------------------------
# Change ray tracing window size
# --------------------------------------------------------------------
#   mode  :  Window size mode, "800x600", "a4" and so on.
# --------------------------------------------------------------------
proc change_size {mode} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    set ls(plot) {xwndow ywndow symht}
    set ls(axis) {xmm tmm zmm orig}
    # Set values
    switch -- $mode {
	"small" {
	    set val(plot) "150 100 0.4"
	    set val(axis) "130 36 36 10"
	}
	"medium" {
	    set val(plot) "190 130 0.5"
	    set val(axis) "170 50 50 12.5"
	}
	"large" {
	    set val(plot) "270 195 1"
	    set val(axis) "250 80 80 12.5"
	}
	"A4" {
	    set val(plot) "250 180 2"
	    set val(axis) "230 70 70 16.0"
	}
    }
    # Set related values.
    foreach i {plot axis} {
	for {set j 0} {$j < [llength $ls($i)]} {incr j} {
	    set gbl(param,$i,[lindex $ls($i) $j]) [lindex $val($i) $j]
	}
    }
}
# --------------------------------------------------------------------
# Generate new velocity structure.
# --------------------------------------------------------------------
proc new_model {} {
   # Global variables --------------------------------------------------
   global gbl wk

   # Main --------------------------------------------------
   bar_message "Cleaning the workspace..."
   # Delete old structure.
   clear_all
   set wk(undo,n) 0
   # Generate!!
   foreach {i j} "n_interface 2 n_node,1 2 n_node,2 2 n_vnode,1 2\
	    x,1 {$gbl(xmin) $gbl(xmax)} z,1 {$gbl(zmin) $gbl(zmin)}\
	    x,2 {$gbl(xmin) $gbl(xmax)} z,2 {$gbl(zmax) $gbl(zmax)}\
            z,SD,1 {0.0 0.0} z,SD,2 {0.0 0.0}\
	    vx,1 {$gbl(xmin) $gbl(xmax)} vtop,1 {1.5 1.5} vbot,1 {1.5 1.5}\
	    vtop,SD,1 {0.0 0.0} vbot,SD,1 {0.0 0.0}\
            poi,1,top 0.0 poi,SD,1,top 0.0\
            poi,1,bot 0.0 poi,SD,1,bot 0.0\
	    gradient_max 3.0 ERI,poisson 0 ERI,poisson_orig 0" {
      set gbl($i) $j
   }
   # Set selected interface.
   set wk(selected) [expr int($gbl(n_interface)/2)]
   # Set ray group parameter
   for {set i 1} {$i <$gbl(n_interface)} {incr i} {
      foreach j {refract  reflect  head} {
	    set gbl(ray,$j,$i) 0
      }
   }
   
   # Message
   bar_message "New Model !!"
   # Redraw ------------------------------
   draw_frame
   draw_model
   update_bindings
}
# --------------------------------------------------------------------
# Delete velocity structure model
# --------------------------------------------------------------------
proc clear_all {} {
    # Global variables --------------------------------------------------
    global gbl sys 

    # Main --------------------------------------------------
    # Delete structure from graph widget --------------------
    # Delete interfaces (element)
    catch "$sys(graph) element delete [$sys(graph) element names]"
    # Delete velocity grid (Marker)
    catch "$sys(graph) marker delete [$sys(graph) marker names]"

    # Clear variables --------------------
    for {set i 1} {$i <= $gbl(n_interface)} {incr i} {
	foreach mode {x z vx vtop vbot} {
	    catch "unset gbl($mode,$i)"
	    catch "vec_$mode$i clear"
	}
    }
    set gbl(n_interface) 0
}
# --------------------------------------------------------------------
# Update title bar
# --------------------------------------------------------------------
proc update_titlebar {} {
   global wk
   wm title . [format "Modeling and RayTracing : %s" \
		   [file tail $wk(file,project)] ]
}
# --------------------------------------------------------------------
# Change velocity font
# --------------------------------------------------------------------
proc update_velocity_font {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # Main --------------------------------------------------
   set new_font [lindex $sys(VELOCITY_FONTS) \
         [expr [lsearch $sys(VELOCITY_FONTS) $gbl(option,label_font)] + 1]]
   option add *Graph.TextMarker.Font $new_font; 
   draw_grid
}
# --------------------------------------------------------------------
# Make ON source list.
# --------------------------------------------------------------------
proc make_source_list {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   set l {}
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      if {$gbl(src,$i,onoff)} {
	 lappend l [format "%.3f" $gbl(src,$i,x)]
      }
   }
   return $l
}
# --------------------------------------------------------------------
# make ON raycode list
# --------------------------------------------------------------------
proc make_ray_list {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   set l {}
   
   if { $gbl(option,backend) == "hamen" } {
      set j 1
      if { $gbl(hamen,first) } { 
	 lappend l $j 
	 incr j
      }
      for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	 if {$gbl(ray,reflect,$i)} {
	    lappend l $j
	    incr j
	 }
      }
   } else {
      for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	 foreach {type n} {refract 0 reflect 1 head 2} {
	    if {$gbl(ray,$type,$i)} {
	       lappend l [expr $i*3 + $n + 1]
	    }
	 }
      }
   }
   return $l
}
# --------------------------------------------------------------------
# Remove verbose values in a list
# --------------------------------------------------------------------
#   l  :  list, sorted by -real option
# --------------------------------------------------------------------
proc unique_list {l} {
   # make dummy value
   set prev [expr [lindex $l 0] + 1.0]
   # remove verbose values
   set result {}
   for {set i 0} {$i < [llength $l]} {incr i} {
      set cur [lindex $l $i]
      if {$prev != $cur} {
	 lappend result $cur
	 set prev  $cur
      }
   }
   return $result
}
# --------------------------------------------------------------------
# Read a text file and send it to STDOUT
# --------------------------------------------------------------------
# f   :  text file name
# --------------------------------------------------------------------
proc cat_file {f} {
   # check file existence
   if { ![file exists $f] || ![file readable $f] } {
      return
   }
   set fp [open $f "r"]
   while {1} {
      if { [gets $fp line] == -1 } {
	 break
      }
      puts stdout $line
   }
   close $fp
}
# --------------------------------------------------------------------
# Return absolute path of a given file
# --------------------------------------------------------------------
# fn : filename
# --------------------------------------------------------------------
proc absolute_path {fn} {
   global env
   if { [llength $fn] < 1 } { return $fn }
   set plist [file split $fn]
   if { [llength $plist] == 1 } {
      set fn [format "./%s" $fn]
      set plist [file split $fn]
   }
   set top [lindex $plist 0]

   # absolute path
   if { $top == [file volumes] } {
      return $fn
   }
   # special paths
   if { [lsearch [list "." ".." "~"] $top] >= 0} {
      # current directory
      if { $top == "." } {
	 set plist [lreplace $plist 0 0 [pwd]]
      }
      # my home
      if { $top == "~" } {
	 set plist [lreplace $plist 0 0 $env(HOME)]
      }
      # count upward directories
      set up 0
      while { $top == ".." } {
	 incr up
	 set top [lindex $plist $up]
      }
      if { $up > 0 } {
	 set cwd [file split [pwd]]
	 set n [llength $cwd]
	 set parents [lrange $cwd 0 [expr $n - $up -1]]
	 set plist [eval list $parents [lrange $plist $up end]]
      }
   } else {
      # current direcotry without "./"
      set plist [concat [pwd] $plist]
   }
   return [eval file join $plist]
}
# --------------------------------------------------------------------
# Return relative path of a given file
# --------------------------------------------------------------------
# fn : filename
# --------------------------------------------------------------------
proc relative_path {fn} {
   if { [llength $fn] < 1 } { return $fn }
   if { [file pathtype $fn] != "absolute" } { return $fn }
   set afn [file split [absolute_path $fn]]
   set cwd [file split [pwd]]
   set n [llength $cwd]
   set commons 0
   for {set i 0} {$i < $n} {incr i} {
      set c [lindex $cwd $i]
      set a [lindex $afn $i]
      if { $a == $c } {
	 incr commons
      }
   }
   set afn [lreplace $afn 0 [expr $commons -1]]
   set parents [expr $n - $commons]
   if { $parents == 0 } {
      set relative "./"
   } else {
      set relative ""
      while { $parents > 0 } {
	 incr parents -1
	 set relative [format "../%s" $relative]
      }
   }
   return [format "%s%s" $relative [eval file join $afn]]
}
# --------------------------------------------------------------------
# Convert all the filenames into relative path
# --------------------------------------------------------------------
proc convert_filenames_relative {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # pick files
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      set gbl(src,$i,pick) [relative_path $gbl(src,$i,pick)]
   }
   # other files
   foreach {member} [array names gbl] {
      if { [string match file,* $member] } {
	 set gbl($member) [relative_path $gbl($member)]
      }
   }
}
# --------------------------------------------------------------------
# Convert all the filenames into absolute path
# --------------------------------------------------------------------
proc convert_filenames_absolute {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # pick files
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      set gbl(src,$i,pick) [absolute_path $gbl(src,$i,pick)]
   }
   # other files
   foreach {member} [array names gbl] {
      if { [string match file,* $member] } {
	 set gbl($member) [absolute_path $gbl($member)]
      }
   }
}

# --------------------------------------------------------------------
# register backend ray-tracer to menu and add key bindings
# --------------------------------------------------------------------
# nam   :  backend name (explanation)
# id    :  backend ID inside the source code
# key   :  a key stroke to invoke parameter window for this backend
# com   :  a command to invoke parameter window for this backend
# --------------------------------------------------------------------
proc register_backend {nam id key com} {
   # Global variables --------------------------------------------------
   global wk
   
   # Main --------------------------------------------------
   $wk(w,backend_menu) add radiobutton -label $nam \
       -variable gbl(option,backend) -value $id
   $wk(w,parameter_menu) add command -label "$nam parameter..." \
       -command $com -accelerator " $key"
   bind . $key $com
}