# --------------------------------------------------------------------
# export_data --------------------------------------------------------
#            Export velocity structure and others in simple formats.
# --------------------------------------------------------------------
proc export_data {} {
    # Global variables --------------------------------------------------
    global sys gbl

    # Main --------------------------------------------------
    set w .export_data
    catch {destroy $w}
    toplevel $w
    wm title $w "Export data"
    wm iconname $w "Export data"
    
    # Title frame ------------------------------
    title_frame $w.title "Export Data"

    # Export frmae ------------------------------
    set ef $w.export_frame
    frame $ef -relief ridge -bd 2
    label $ef.lboundary -text "Interface information \"./boundary.dat\""\
	-relief groove -anchor w
    button $ef.bboundary -text "Export" -command {GMT_interface ./boundary.dat}
    grid $ef.lboundary $ef.bboundary -sticky news

    label $ef.lvgrid -text "Velocity grid information \"./velocity_grid.dat\""\
	-relief groove -anchor w
    button $ef.bvgrid -text "Export"\
	-command {GMT_vel_grid ./velocity_grid.dat}
    grid $ef.lvgrid $ef.bvgrid -sticky news

    label $ef.lobs -text "Shot(OBS) position information \"./obs.dat\""\
	-relief groove -anchor w
    button $ef.bobs -text "Export" -command {GMT_obs_data ./obs.dat}
    grid $ef.lobs $ef.bobs -sticky news
    
    # Button frame ------------------------------
    set bf $w.button_frame
    frame $bf
    button $bf.ok -text "Dismiss" -padx 20 -command "destroy $w" -width 15
    pack $bf.ok -side left -expand 1
   
    # Packing ------------------------------
    pack $w.title $ef $bf -side top -fill x
}

# --------------------------------------------------------------------
# GMT_interface ------------------------------------------------------
#            Generate Interface data file for GMT scripts
# --------------------------------------------------------------------
# filename : Output filename
# --------------------------------------------------------------------
proc GMT_interface {filename} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    set fp [open $filename "w"]
    for {set i 1} {$i <= $gbl(n_interface)} {incr i} {
	for {set j 0} {$j < [llength $gbl(x,$i)]} {incr j} {
	    puts $fp [format "%f %f"\
		    [lindex $gbl(x,$i) $j] [lindex $gbl(z,$i) $j] ]
	}
	puts $fp ">"
    }
    close $fp
}

# --------------------------------------------------------------------
# GMT_vel_grid -------------------------------------------------------
#            Generate Velocity grid data for GMT scripts.
# --------------------------------------------------------------------
# filename : Output filename.
# --------------------------------------------------------------------
proc GMT_vel_grid {filename} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    set fp [open $filename "w"]
    for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	for {set j 0} {$j < [llength $gbl(vx,$i)]} {incr j} {
	    set x [lindex $gbl(vx,$i) $j]
	    set top [interpolate_depth $i $x]
	    set bottom [interpolate_depth [expr $i + 1] $x]
	    puts $fp [format "%f %f\n%f %f\n>\n" $x $top $x $bottom]
	}
    }
    close $fp
}
# --------------------------------------------------------------------
# GMT_obs_data -------------------------------------------------------
#            Generate data file of Source position for GMT scripts
# --------------------------------------------------------------------
# filename : filename for output
# --------------------------------------------------------------------
proc GMT_obs_data {filename} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    set fp [open $filename "w"]
    for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
	if {$gbl(src,$i,onoff)} {
	    puts $fp [format "%f %f"\
		    $gbl(src,$i,x) $gbl(src,$i,z)]
	}
    }
    close $fp
}
