# --------------------------------------------------------------------
# move_start ---------------------------------------------------------
#           Start to move a node
# --------------------------------------------------------------------
proc move_start {x z} {
   # Global variables --------------------------------------------------
   global gbl sys wk
   global tmp_x tmp_z
   
   # Main --------------------------------------------------
   if ![$sys(graph) element closest $x $z point\
	    -halo $wk(mark,select_size) interface($wk(selected))] {
	       set gbl(move,index) ""
	       bar_message "No node around the mouse pointer."
	       return
	    }
   set gbl(move,index) $point(index)
   set xz [$sys(graph) invtransform $x $z]
   # New bindigs to move and finish moving
   bind $sys(graph) <B3-Motion> {
      move_motion %x %y
   }
   bind $sys(graph) <ButtonRelease-3> {
      move_end %x %y
   }
   show_position $x $z
   # create temporary vector
   vec_x$wk(selected) dup tmp_x
   vec_z$wk(selected) dup tmp_z
   $sys(graph) element create tmp_interface\
       -xdata tmp_x -ydata tmp_z -pixe $wk(mark,size)\
       -outline $wk(col,tmp) -fil $wk(col,tmp) -color $wk(col,tmp) -label ""
}
# --------------------------------------------------------------------
# move_motion --------------------------------------------------------
#           Move a point (During move)
# --------------------------------------------------------------------
proc move_motion {x z} {
    # Global variables --------------------------------------------------
    global gbl sys
    global tmp_x tmp_z
    
    # Main --------------------------------------------------
    bind $sys(graph) <B3-Motion> {}
    if {[string compare $gbl(move,index) ""] == 0} {
	return;
    }
    show_position $x $z
    set xz [$sys(graph) invtransform $x $z]
    set x [lindex $xz 0]
    set z [lindex $xz 1]
    set tmp_x($gbl(move,index)) $x
    set tmp_z($gbl(move,index)) $z
    bind $sys(graph) <B3-Motion> {
	move_motion %x %y
    }
}
# --------------------------------------------------------------------
# move_end -----------------------------------------------------------
#           Finish moving a node
# --------------------------------------------------------------------
proc move_end {x z} {
    # Global variables --------------------------------------------------
    global gbl sys wk
    global vec_x$wk(selected) vec_z$wk(selected)
    
    # Main --------------------------------------------------
    if {[string compare $gbl(move,index) ""] == 0} {
	return
    }
    # delete temporary interface
    $sys(graph) element delete tmp_interface

    set xz [$sys(graph) invtransform $x $z]
    set x [format "%.3f" [lindex $xz 0]]
    set z [format "%.3f" [lindex $xz 1]]
    # undo information update
    set_node_undo_info point.move $wk(selected) $gbl(move,index)

    # Check the new position ------------------------------
    # Left end check
    if {$gbl(move,index) > 0} {
	set left [lindex $gbl(x,$wk(selected)) [expr $gbl(move,index) - 1]]
	if {$x < $left} {
	    set x $left
	}
    } else {
	set x $gbl(xmin)
    }
    # Right end check
    if {$gbl(move,index) < [expr $gbl(n_node,$wk(selected)) - 1]} {
	set right [lindex $gbl(x,$wk(selected)) [expr $gbl(move,index) + 1]]
	if {$x > $right} {
	    set x $right
	}
    } else {
	set x $gbl(xmax)
    }
    # Upper side check
    if {$wk(selected) > 1} {
	set top [interpolate_depth [expr $wk(selected) - 1] $x]
	if {$z < $top} {
	    set z $top
	}
    }
    # Lower side check
    if {$wk(selected) < $gbl(n_interface)} {
	set bottom [interpolate_depth [expr $wk(selected) + 1] $x]
	if {$z > $bottom} {
	    set z $bottom
	}
    }
    # Move to a new node
    set vec_x$wk(selected)\($gbl(move,index)) $x
    set vec_z$wk(selected)\($gbl(move,index)) $z
    # Finish!
    set gbl(x,$wk(selected)) [lreplace $gbl(x,$wk(selected))\
	    $gbl(move,index) $gbl(move,index) $x]
    set gbl(z,$wk(selected)) [lreplace $gbl(z,$wk(selected))\
	    $gbl(move,index) $gbl(move,index) $z]
    bar_message [format "A node was moved to (%.3fkm, %.3fkm)" $x $z]
    update_layers
    draw_model
    set gbl(move,index) ""
    bind $sys(graph) <B3-Motion> {}
    bind $sys(graph) <ButtonRelease-3> {}
}
# --------------------------------------------------------------------
# delete_point -------------------------------------------------------
#            Delete a node
# --------------------------------------------------------------------
# [arguments]
#   x   : X coordinate of mouse cursor
#   y   : Y coordinate of mouse cursor
# --------------------------------------------------------------------
proc delete_point {x y} {
    # Global variables --------------------------------------------------
    global gbl sys wk
    
    # Main --------------------------------------------------
    if ![$sys(graph) element closest $x $y point\
	    interface($wk(selected))] {
	set gbl(move,index) ""
	bar_message "No Selected node around the Mouse cursor."
	return
    }
    # edge node?
    if { $point(index) == 0 ||\
	     $point(index) == [expr $gbl(n_node,$wk(selected)) -1 ]} {
       bar_message "Cannot delete node on the edge."
       return
    }
    # Delete after blinking
    set markerName "bltClosest_$point(name)"
    flash_point delete $wk(selected) $point(index) 4
}
# --------------------------------------------------------------------
# create_point -------------------------------------------------------
#            Create a new node
# --------------------------------------------------------------------
# [arguments]
#   x   : X coordinate of mouse cursor
#   y   : Y coordinate of mouse cursor
# --------------------------------------------------------------------
proc create_point {x y} {
    # Global variables --------------------------------------------------
    global gbl sys wk
    
    # Main --------------------------------------------------
    # Get the position of mouse cursor
    set xz [$sys(graph) invtransform $x $y]
    set x [format "%.3f" [lindex $xz 0]]
    set z [format "%.3f" [lindex $xz 1]]
    # Upper side
    if {$wk(selected) > 1} {
	set top [interpolate_depth [expr $wk(selected) - 1] $x]
    } else {
	set top -1.0e10
    }
    # Lower side
    if {$wk(selected) < $gbl(n_interface)} {
	set bottom [interpolate_depth [expr $wk(selected) + 1] $x]
    } else {
	set bottom 1.0e10
    }
    # Mouse cursor is valid range? ----------
    if {[expr ($bottom - $z)*($z - $top)] < 0} {
	return
    }
    # Left end check
    if {$x < $gbl(xmin)} {
	set x $gbl(xmin)
    }
    # Right end check
    if {$x > $gbl(xmax)} {
	set x $gbl(xmax)
    }
    # Insert a new node into an appropriate position ----------
    vec_x$wk(selected) append $x
    vec_z$wk(selected) append $z
    vec_x$wk(selected) sort vec_z$wk(selected)
    vector create workx workz
    vec_x$wk(selected) dup workx
    vec_z$wk(selected) dup workz
    workx variable workx
    workz variable workz
    set gbl(x,$wk(selected)) $workx(:)
    set gbl(z,$wk(selected)) $workz(:)
    set index [workx search $x]
    incr gbl(n_node,$wk(selected))
    # SD-value for z
    if {$index == 0} {
       set sd [lindex $gbl(z,SD,$wk(selected)) 1]
    } elseif {$index == [expr $gbl(n_node,$wk(selected)) - 1] } {
       set sd [lindex $gbl(z,SD,$wk(selected)) end]
    } else {
       set sd [expr ( [lindex $gbl(z,SD,$wk(selected)) [expr $index - 1]] +\
		   [lindex $gbl(z,SD,$wk(selected)) $index] ) /2.0 ]
    }
    set gbl(z,SD,$wk(selected))\
	[linsert $gbl(z,SD,$wk(selected)) $index $sd]

    # undo inforamtion update
    set_node_undo_info point.create $wk(selected) $index

    # Blinking the new node
    bar_message "A New node is added"
    update_layers
    flash_point insert $wk(selected) $index 4
}
# --------------------------------------------------------------------
# flash_point -------------------------------------------------------
#           Blinking the node
# --------------------------------------------------------------------
# [arguments]
# mode  : Process after blinking, "delete" or "create"
# i     : Interface number in which the node is.
# index : Node number in the interface.
# count : The rest of number of times to blink.  
# --------------------------------------------------------------------
proc flash_point { mode i index count } {
    # Global variables --------------------------------------------------
    global sys 
    
    # Main --------------------------------------------------
    set name interface($i)
    # Blink until the "count" become 0.
    if { $count & 1 } {
       $sys(graph) element deactivate $name 
    } else {
       $sys(graph) element activate $name $index
    }
    incr count -1
    if { $count > 0 } {
	after 200 flash_point $mode $i $index $count
	update
    } else {
	# After finishing blink
	eval $sys(graph) marker delete\
		[$sys(graph) marker names "bltClosest_*"]
	if ![string compare $mode "delete"] {
	    exec_delete $i $index
	}
    }
}
# --------------------------------------------------------------------
# delete_point -------------------------------------------------------
#           Delete a node
# --------------------------------------------------------------------
# [arguments]
# i     : Interface number in which the node is.
# index : Node number in the interface.
# --------------------------------------------------------------------
proc exec_delete {i index} {
    # Global variables --------------------------------------------------
    global gbl
    
    # Main --------------------------------------------------
    # undo information update
    set_node_undo_info point.delete $i $index
    # Exec delete
    set gbl(x,$i) [lreplace $gbl(x,$i) $index $index]
    set gbl(z,$i) [lreplace $gbl(z,$i) $index $index]
    set gbl(z,SD,$i) [lreplace $gbl(z,SD,$i) $index $index]
    incr gbl(n_node,$i) -1
    vec_x$i delete $index
    vec_z$i delete $index
    bar_message "Deleted a node."
    update_layers
}
# --------------------------------------------------------------------
# select_point -------------------------------------------------------
#            Invoke node edit window
# --------------------------------------------------------------------
# [arguments]
#   x   : x coordinate of mouse cursor. (Graph widget coordiantes.
#   y   : y coordinate of mouse cursor. (Graph widget coordiantes.
#   XX  : x coordinate of mouse cursor. (root window coordiantes.)
#   YY  : y coordinate of mouse cursor. (root window coordiantes.)
# --------------------------------------------------------------------
proc select_point {x y XX YY} {
    # Global variables --------------------------------------------------
    global sys gbl wk
    
    # Main --------------------------------------------------
    # Search closest point from mouse cursor ------------------------------
    if ![$sys(graph) element closest $x $y point\
	    interface($wk(selected))] {
	bar_message "No Selected node around the Mouse cursor."
	return
    }
    # Highlight the selected node ------------------------------
    set markerName "bltClosest_$point(name)"
    $sys(graph) element activate $point(name) $point(index)

    # Invoke the edit window ------------------------------
    set pw .point_window
    catch {destroy $pw}
    toplevel $pw
    wm geometry $pw [format "+%d+%d" $XX $YY]
    
    # Title frame ------------------------------
    title_frame $pw.title "Node Edit : Interface $wk(selected)"

    # Parameter frame ------------------------------
    set pwl $pw.location
    frame $pwl -relief ridge -bd 2
    label $pwl.xlabel -text "X = "
    entry $pwl.xentry -width 12
    $pwl.xentry insert 0 $point(x)
    label $pwl.xunit -text "Km.   "
    label $pwl.zlabel -text "Depth = "
    entry $pwl.zentry -width 12
    $pwl.zentry insert 0 $point(y)
    label $pwl.zunit -text "Km.   "
    label $pwl.sdlabel -text "SD = "
    entry $pwl.sdentry -width 12
    label $pwl.sdunit -text "Km."
    $pwl.sdentry insert 0 [lindex $gbl(z,SD,$wk(selected)) $point(index)]
    if {$gbl(ERI,inv)} {
       # ERI-Inversion mode
       pack $pwl.xlabel $pwl.xentry $pwl.xunit\
	   $pwl.zlabel $pwl.zentry $pwl.zunit\
	   $pwl.sdlabel $pwl.sdentry $pwl.sdunit\
	   -in $pwl -fill x -side left -pady 5
    } else {
       # Non-inversion mode
       pack $pwl.xlabel $pwl.xentry $pwl.xunit\
	   $pwl.zlabel $pwl.zentry $pwl.zunit\
	   -in $pwl -fill x -side left -pady 5
    }

    # Key bindigs 
    bind Entry <Return> "finish_point $point(index) ok $pw; destroy $pw"

    # Button frame ------------------------------
    frame $pw.button
    # OK button
    button $pw.button.ok -text "OK" -width 10\
	-command "finish_point $point(index) ok $pw; destroy $pw"
    # Revert button
    button $pw.button.revert -text "Revert" -width 10\
	-command "select_point $x $y $XX $YY"
    # Cancel button
    button $pw.button.cancel -text "Cancel" -width 10\
	-command "finish_point $point(index) cancel $pw; destroy $pw"
    
    # Set SD all button
    button $pw.button.setall -text "Set all SD" -width 10\
	-command "set_all_node_sd $pwl.sdentry $wk(selected)"

    # Packing in button frame
    if {$gbl(ERI,inv)} {
       # ERI-Inversion mode
       pack $pw.button.ok $pw.button.revert $pw.button.cancel\
	   $pw.button.setall\
	   -in $pw.button -side left -expand 1
    } else {
       # Non-Inversion mode
       pack $pw.button.ok $pw.button.revert $pw.button.cancel\
	   -in $pw.button -side left -expand 1
    }

    # Packing ------------------------------
    pack $pw.title $pwl $pw.button -side top -fill x
    busy hold .
}
# --------------------------------------------------------------------
# finish_point -------------------------------------------------------
#            Quit process of node edit window
# --------------------------------------------------------------------
# [arguments] 
#   node  :  Node number of the point
#   mode  :  Quit mode, "cancel" or "ok"
#   w     :  Widget name of the edit window
# --------------------------------------------------------------------
proc finish_point {node mode w} {
    # Global variables --------------------------------------------------
    global gbl sys wk
    
    # Main --------------------------------------------------
    # Release grab --------------------
    busy release .

    # Unhighlight the selected point --------------------
    $sys(graph) element deactivate "interface($wk(selected))"

    # "cancel" mode --------------------
    if ![string compare $mode "cancel"] {
	bar_message ""
	return
    }
    # Update a node position ----------
    set new(x) [format "%.3f" [$w.location.xentry get]]
    set new(z) [format "%.3f" [$w.location.zentry get]]
    set new(sd) [format "%g" [$w.location.sdentry get]]
    # Left and right check
    if {$node < 1 || $node == [expr $gbl(n_node,$wk(selected)) - 1]} {
	# If the both-end of model, can't move.
	set new(x) [lindex $gbl(x,$wk(selected)) $node]
    } else {
	set left [lindex $gbl(x,$wk(selected)) [expr $node-1]]
	set right [lindex $gbl(x,$wk(selected)) [expr $node+1]]
	if {$new(x) < $left} {
	    set new(x) $left
	} elseif {$new(x) > $right} {
	    set new(x) $right
	}
    }
    # Upper and lower side.
    # Upper side check
    if {$wk(selected) > 1} {
	set top [interpolate_depth [expr $wk(selected) - 1] $new(x)]
    } else {
	set top -1.0e10
    }
    # Lower side check
    if {$wk(selected) < $gbl(n_interface)} {
	set bottom [interpolate_depth [expr $wk(selected) + 1] $new(x)]
    } else {
	set bottom 1.0e10
    }
    # Mouse cursor is corrected position?? ----------
    if {$new(z) > $bottom} {
	set new(z) $bottom
    } elseif {$new(z) < $top} {
	set new(z) $top
    }
    # Has the node been edited?
    if { $new(x) != [lindex $gbl(x,$wk(selected)) $node] || \
	     $new(z) != [lindex $gbl(z,$wk(selected)) $node] || \
	     $new(sd) != [lindex $gbl(z,SD,$wk(selected)) $node] } {
	# undo information update
	set_node_undo_info point.move $wk(selected) $node
	# New position
	set gbl(x,$wk(selected))\
	    [lreplace $gbl(x,$wk(selected)) $node $node $new(x)]
	set gbl(z,$wk(selected))\
	    [lreplace $gbl(z,$wk(selected)) $node $node $new(z)]
	set gbl(z,SD,$wk(selected))\
	    [lreplace $gbl(z,SD,$wk(selected)) $node $node $new(sd)]
	bar_message "A node was edited"
	draw_model
    }
    bind Entry <Return> ""
}
# --------------------------------------------------------------------
# update_layers ------------------------------------------------------
#            Update both upper and lower layers of selected interface
# --------------------------------------------------------------------
proc update_layers {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   if {$wk(selected) > 1} {
      draw_grid_layer [expr $wk(selected) - 1]
   }
   if {$wk(selected) < $gbl(n_interface)} {
      draw_grid_layer $wk(selected)
   }
}
# --------------------------------------------------------------------
# set_node_undo_info -------------------------------------------------
#        Update undo information, when you edit a node.
# --------------------------------------------------------------------
# [arguments]
#   mode  : edit mode (point.move, point.delete, point.create)
#   inter : Interface number in which the node is.
#   i     : Index of the node in this interface.
# --------------------------------------------------------------------
proc set_node_undo_info {mode inter i} {
    # Global variables --------------------------------------------------
    global gbl wk

    # Main --------------------------------------------------
    # Common undo information
    set n $wk(undo,n)
    set wk(undo,x,$n) [lindex $gbl(x,$inter) $i]
    set wk(undo,z,$n) [lindex $gbl(z,$inter) $i]
    set wk(undo,vtop,$n) dum
    set wk(undo,vbot,$n) dum
    set wk(undo,vtop,SD,$n) dum
    set wk(undo,vbot,SD,$n) dum
    set wk(undo,interface,$n) $inter
    set wk(undo,index,$n) $i
    set wk(undo,mode,$n) $mode
    if {$mode == "point.all_sd"} {
       set wk(undo,z,SD,$n) $gbl(z,SD,$inter)
    } else {
       set wk(undo,z,SD,$n) [lindex $gbl(z,SD,$inter) $i]
    }
    set wk(undo,poi,top,$n) dum
    set wk(undo,poi,bot,$n) dum
    set wk(undo,poi,SD,top,$n) dum
    set wk(undo,poi,SD,bot,$n) dum
    # Update undo-list.
    incr wk(undo,n)
    undo_check
}

# --------------------------------------------------------------------
# set_all_node_sd ----------------------------------------------------
#        Set all the SD value of node depth in a interface
# --------------------------------------------------------------------
# [arguments]
#  w  : entry widget name of SD value
#  i  : Interface number  
# --------------------------------------------------------------------
proc set_all_node_sd {w i} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # Get SD value
   set v [$w get]
   if [catch "expr 1*$v"] {
      # Invalid value and return
      tk_messageBox -icon error -message "Invalid Standard Deviation value."\
	  -type ok
      return
   }
   # update undo information
   set_node_undo_info point.all_sd $i -1
   # Set all!
   set gbl(z,SD,$i) {}
   for {set j 0} {$j < $gbl(n_node,$i)} {incr j} {
      lappend gbl(z,SD,$i) $v
   }
}