# --------------------------------------------------------------------
# source_parameter ---------------------------------------------------
#            Source parameter set window
# --------------------------------------------------------------------
proc source_parameter {} {
   # Global variables --------------------------------------------------
   global sys gbl wk
   
   # Main --------------------------------------------------
   set w $wk(w,src_param)
   if { [winfo exists $w] } {
      wm deiconify $w
      raise $w
      return
   }
   toplevel $w
   wm title $w "Source Parameter"
   wm resizable $w 0 1
   
   # Title frame ------------------------------
   title_frame $w.title "Source  Parameter"
   
   # Parameter frame ------------------------------
   # Legend
   set wf $w.legend
   frame $wf -relief groove
   label $wf.check -text "    " -relief flat -font a14
   label $wf.label -text "Source No." -width 15 -anchor w -relief raised
   label $wf.name -text "Name" -width 20 -anchor w -border 4 -relief raised
   label $wf.x -text "X (Km)"     -anchor w -width 20 -border 4 -relief raised
   label $wf.z -text "Depth (Km)" -anchor w -width 20 -border 4 -relief raised
   pack $wf.check $wf.label $wf.name $wf.x $wf.z -side left
   
   frame $w.frame
   text $w.frame.text -yscrollcommand "$w.frame.scroll set"\
       -width 93 -height 20 -state disabled
   scrollbar $w.frame.scroll -command "$w.frame.text yview"
   set wp $w.frame
   pack $wp.scroll -side right -fill y
   pack $wp.text -side left -expand yes -fill both
   display_source $wp
   
   # scroll by wheel mouse
   bind $w <Button-4> "$w.frame.text yview scroll -2 unit"
   bind $w <Button-5> "$w.frame.text yview scroll 2 unit"
   
   # Button frame ------------------------------
   set bf [button_frame $w $w.button_frame]
   
   # menu ------------------------------
   set m [menu $w.menubar -tearoff 0]
   set m2 [menu $m.src -tearoff 0]
   $m add cascade -label "Source" -menu $m2 -underline 0
   $m2 add command -label "Add..." -command "add_source $wp" -accel Ctl-a
   $m2 add command -label "Del..." -command "delete_source $wp"
   $m2 add command -label "Sort by X..." -command "sort_source $wp"
   $m2 add separator
   $m2 add command -label "ON all sources..." -command "check_all_source on"
   $m2 add command -label "OFF all sources..." -command "check_all_source off"
   $m2 add separator
   $m2 add checkbutton -label "single source mode" -variable gbl(src,single) \
       -command "check_all_source off"

   $w conf -menu $m
   bind $w <Control-a> "add_source $wp"

   # Packing
   pack $w.title $w.legend -side top -fill x
   pack $w.frame -side top -fill both -expand yes
   pack $bf -side top -fill x
}
# --------------------------------------------------------------------
# display_source -----------------------------------------------------
#            Display a source parameter widget
# --------------------------------------------------------------------
# [arguments]
#   wp   :  Parent widget name
# --------------------------------------------------------------------
proc display_source {wp} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   $wp.text conf -state normal
   update
   $wp.text delete 0.0 end
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      set wf $wp.text.frame$i
      catch {destroy $wf}
      frame $wf -relief groove -bd 1
      checkbutton $wf.check$i -variable gbl(src,$i,onoff)\
	  -command "source_checkbutton $i"
      label $wf.label$i -text [format "No. %d" $i] -anchor w -width 15
      entry $wf.name$i -wid 20 -textva gbl(src,$i,name) -ju left
      entry $wf.xentry$i -width 20 -textvariable gbl(src,$i,x)
      entry $wf.zentry$i -width 20 -textvariable gbl(src,$i,z)
      grid $wf.check$i $wf.label$i $wf.name$i $wf.xentry$i $wf.zentry$i\
	  -sticky news
      label $wf.plabel$i -text "pick file" -anchor e
      entry $wf.filename$i -wid 60 -textva gbl(src,$i,pick) -ju left
      button $wf.ref$i -text "Ref..."\
	  -command "select_source_file $i"
      grid  x $wf.plabel$i $wf.filename$i - - $wf.ref$i -sticky news
      $wp.text window create end -window $wf
   }
   $wp.text conf -state disabled
}
# --------------------------------------------------------------------
# source_checkbutton
#            command binding to the checkbutton of each sources
# --------------------------------------------------------------------
# [arguments]
#   n   :  source number
# --------------------------------------------------------------------
proc source_checkbutton {n} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   if { $gbl(src,single) } {
      for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
	 set gbl(src,$i,onoff) 0
      }
      set gbl(src,$n,onoff) 1
   }
   draw_source
}
# --------------------------------------------------------------------
# select_source_file -------------------------------------------------
#            Select a source file
# --------------------------------------------------------------------
proc select_source_file {ind} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # file selection
   set types {
      {"Pick files"	   *.pick}
      {"MacRay files"	   *.macray}
      {"tx.in files"         *.in}
      {"dat files"            *.dat}
      {"All files"		*}
   }
   file_selection gbl(src,$ind,pick) load $types
   set fn $gbl(src,$ind,pick)
   set dat [read_stn_x_z_from_pick $fn]
   if {$dat == ""} { return }
   # update parameters
   if { $gbl(src,$ind,x) == "" } {
      set gbl(src,$ind,x) [lindex $dat 0]
   }
   if { $gbl(src,$ind,z) == "" } {
      set gbl(src,$ind,z) [lindex $dat 1]
   }
   if { $gbl(src,$ind,name) == "" } {
      set gbl(src,$ind,name) [lrange $dat 2 end]
   }
}
# --------------------------------------------------------------------
# add_source ---------------------------------------------------------
#            Add a source parameter entry
# --------------------------------------------------------------------
proc add_source {w} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   incr gbl(src,n_source)
   display_source $w
   $w.text conf -state normal
   update
   $w.text yview 1
   $w.text conf -state disabled
}

# --------------------------------------------------------------------
# delete_source ------------------------------------------------------
#            Delete a source parameter entry
# --------------------------------------------------------------------
proc delete_source {wp} {
   # Global variables --------------------------------------------------
   global gbl

   # Main --------------------------------------------------
   # Invoke window
   set w .delete_source
   catch {destroy $w}
   toplevel $w
   set geom [split [wm geometry .[lindex [split $wp .] 1]] "+x"]
   set x [expr [lindex $geom 2] + 30]
   set y [expr [lindex $geom 3] + 30]
   wm geometry $w [format "+%d+%d" $x $y]
   grab $w

   # Entry frame to input source number
   set wf $w.entry_frame
   frame $wf
   label $wf.title -text "Which source do you want to delete?  No. "
   entry $wf.entry -width 8
   pack $wf.title $wf.entry -side left
   
   # Button frame
   set wb $w.button_frame
   frame $wb
   button $wb.delete -text "Exec Delete" -width 15\
       -command "exec_del_source $wf.entry $wp; destroy $w"
   button $wb.cancel -text "Cencel" -width 15\
       -command "destroy $w;"
   pack $wb.delete $wb.cancel -side left -expand 1

   # Packing
   pack $wf $wb -side top -fill x
}
# --------------------------------------------------------------------
# exec_del_source ----------------------------------------------------
#            Execute delete a source
# --------------------------------------------------------------------
# [arguments]
#    we   :  Entry widget name in which source number is written.
#    wp   :  Widget name in which source list is displayed.
# --------------------------------------------------------------------
proc exec_del_source {we wp} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    # Read a source number from entry widget 
    set n [$we get]
    
    # Check the number
    if {[string length $n] < 1} {
	return
    }
    if {$n > 0 && $n <= $gbl(src,n_source)} {
	for {set i $n} {$i < $gbl(src,n_source)} {incr i} {
	    foreach j {x z name onoff pick} {
		set gbl(src,$i,$j) $gbl(src,[expr $i+1],$j)
	    }
	}
	incr gbl(src,n_source) -1
    }
    # Redisplay
    display_source $wp
}
# --------------------------------------------------------------------
# check_all_source ---------------------------------------------------
#            Check all the sources or Uncheck all the sources.
# --------------------------------------------------------------------
# [arguments]
#   of_off :  "on" or "off"
# --------------------------------------------------------------------
proc check_all_source {on_off} {
   # Global variables --------------------------------------------------
   global gbl

   # Main --------------------------------------------------
   # Distinguish mode --------------------
   if ![string compare $on_off "on"] {
      set mode 1
      if { $gbl(src,single) } {
	 bar_message "To select plural sources, trun off the \"single source mode\"."
	 return
      }
   } else {
      set mode 0
   }
   # Set all the source status --------------------
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      set gbl(src,$i,onoff) $mode
   }

   # Redisplay --------------------
   draw_source
}

# --------------------------------------------------------------------
# sort_source ---------------------------------------------------------
#            Sort all sources
# --------------------------------------------------------------------
proc sort_source {w} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    for {set i $gbl(src,n_source)} {$i > 1} {incr i -1} {
	for {set j 1} {$j < $i} {incr j} {
	    set i1 $j
	    set i2 [expr $j + 1]
	    if { $gbl(src,$i1,x) > $gbl(src,$i2,x) } {
		# swap
		foreach k {x z name onoff pick} {
		    set tmp $gbl(src,$i1,$k)
		    set gbl(src,$i1,$k) $gbl(src,$i2,$k)
		    set gbl(src,$i2,$k) $tmp
		}
	    }
	}
    }
}

# --------------------------------------------------------------------
# ray_parameter ---------------------------------------------------
#            Ray group set window
# --------------------------------------------------------------------
proc ray_parameter {} {
   # Global variables --------------------------------------------------
   global gbl sys
   
   # Main --------------------------------------------------
   set w .rays
   set geometry ""
   if [winfo exist $w] {
      regsub \\d*x\\d* [winfo geometry $w] "" geometry
      destroy $w
   }
   toplevel $w
   wm title $w "RAY Group"
   wm resizable $w 0 1
   
   # Title frame ------------------------------
   title_frame $w.title "Set  RAY Group  Parameter"
   
   # Simple specification mode ------------------------------
   # Title
   label $w.simple_title -text "Simple specification" -bg gray60 -fg white

   set wf $w.parameter_frame
   frame $wf -relief groove -bd 2

   # Ray code ----------
   # Layer number
   label $wf.l_label -text "Layer" -relief raised
   set winlist $wf.l_label
   for {set i 1} {$i < $gbl(n_interface)} {incr i} {
      label $wf.l_layer$i -text $i -relief raised
      lappend winlist $wf.l_layer$i
   }
   eval {grid} $winlist -sticky news
   # Refraction
   label $wf.ra_label -text "Refract in" -relief ridge
   set winlist $wf.ra_label
   for {set i 1} {$i < $gbl(n_interface)} {incr i} {
      checkbutton $wf.ra_layer$i -variable gbl(ray,refract,$i)
      lappend winlist $wf.ra_layer$i
   }
   eval {grid} $winlist -sticky news
   # Reflection
   label $wf.re_label -text "Reflect at the Bottom of" -relief ridge
   set winlist $wf.re_label
   for {set i 1} {$i < $gbl(n_interface)} {incr i} {
      checkbutton $wf.re_layer$i -variable gbl(ray,reflect,$i)
      lappend winlist $wf.re_layer$i
   }
   eval {grid} $winlist -sticky news
   # Headwave
   label $wf.h_label -text "Headwave at the Bottom of" -relief ridge
   set winlist $wf.h_label
   for {set i 1} {$i < $gbl(n_interface)} {incr i} {
      checkbutton $wf.h_layer$i -variable gbl(ray,head,$i)
      lappend winlist $wf.h_layer$i
   }
   eval {grid} $winlist -sticky news

   # Complex specification mode ------------------------------
   label $w.complex_title -text "Complex specification" -bg gray60 -fg white
   set cf [frame $w.complex_frame -relief groove -bd 2]
   for {set i 1} {$i <= $sys(MAX_COMPLEX_WAVE)} {incr i} {
      checkbutton $cf.check$i -variable gbl(ray,convert,$i,onoff)
      tk_optionMenu $cf.mode$i gbl(ray,convert,$i,mode)\
	  "     Refract in          "\
	  "Reflect at the Bottom of "\
	  "Headwave at the Bottom of"\
	  "     Take-off angle      "
      entry $cf.layer$i -width 2 -justify right\
	  -textvariable gbl(ray,convert,$i,layer)
      label $cf.label$i -text "layer,  cbnd:"
      entry $cf.inter$i -width 8 -justify left\
	  -textvariable gbl(ray,convert,$i,interface)
      label $cf.mlabel$i -text "  rbnd:"
      entry $cf.rbnd$i -width 8 -justify left\
	  -textvariable gbl(ray,convert,$i,rbnd)
      grid $cf.check$i $cf.mode$i $cf.layer$i $cf.label$i $cf.inter$i \
	  $cf.mlabel$i $cf.rbnd$i -sticky news
   }
   # Take-off angle  ----------
   set tf [frame $w.takeoff_frame -relief groove -bd 2]
   label $tf.label -text "take-off angle  from:"
   entry $tf.takeoff_min -width 7 -justify right -textvar gbl(zelt,trace,amin)
   label $tf.to -text "   to:"
   entry $tf.takeoff_max -width 7 -justify right -textvar gbl(zelt,trace,amax)
   label $tf.unit -text "degree"
   grid $tf.label $tf.takeoff_min $tf.to $tf.takeoff_max $tf.unit -sticky news

   # Button frame ------------------------------
   set bf [button_frame $w $w.button_frame]

   # Packing
   pack $w.title $w.simple_title $wf\
       $w.complex_title $cf $tf $bf -side top -fill x -expand 1

   update
   wm geometry $w "$geometry"
}

# --------------------------------------------------------------------
# update_model_size --------------------------------------------------
#            update model size
# --------------------------------------------------------------------
proc update_model_size {} {
   # Global variables --------------------------------------------------
   global gbl sys wk
   
   # Main --------------------------------------------------
   # Model existence check --------------------
   if { ![$sys(graph) element exist graph_frame] } { return }
   
   # Horizontal direction --------------------
   # Set the accuracy of xmin, xmax to 8.3f
   set wk(xmin) [format "%8.3f" $wk(xmin)]
   set wk(xmax) [format "%8.3f" $wk(xmax)]
   if { $wk(xmin) >= $wk(xmax) } {
      tk_messageBox -message "invalid (xmin, xmax)." -type ok -icon error
      return
   }
   
   set enlarge 0   ; # no enlargement
   set shrink  0   ; # no shrink 
   if {$gbl(xmax) > $wk(xmax) || $gbl(xmin) < $wk(xmin) } {
      set ans [tk_messageBox -message "The model will shrink. Are you sure?"\
		   -type yesno -icon question]
      if { $ans == "yes" } {
	 set shrink 1
	 set gbl(xmax) $wk(xmax)
	 set gbl(xmin) $wk(xmin)
      } else {
	 return
      }
   }
   if {$gbl(xmax) < $wk(xmax)} {
      set gbl(xmax) $wk(xmax)
      set enlarge 1
   }
   if {$gbl(xmin) > $wk(xmin)} {
      set gbl(xmin) $wk(xmin)
      set enlarge 1
   }
   
   if { $enlarge } {
      # Enlarge all over the structure. ---------------
      for {set i 1} {$i <= $gbl(n_interface)} {incr i} {
	 set gbl(x,$i) [lreplace $gbl(x,$i) 0 0 $gbl(xmin)]
	 set gbl(x,$i) [lreplace $gbl(x,$i) end end $gbl(xmax)]
      }
      for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	 set gbl(vx,$i) [lreplace $gbl(vx,$i) 0 0 $gbl(xmin)]
	 set gbl(vx,$i) [lreplace $gbl(vx,$i) end end $gbl(xmax)]
      }
   }
   if { $shrink } {
      # Shrink all over the structure. ---------------
      # shrink interfaces -----
      for {set i 1} {$i <= $gbl(n_interface)} {incr i} {
	 set newx {}
	 set newz {}
	 set newsd {}
	 for {set j 0} {$j < $gbl(n_node,$i)} {incr j} {
	    set x [lindex $gbl(x,$i) $j]
	    if { $x >= $gbl(xmin) && $x <= $gbl(xmax) } {
	       lappend newx $x
	       lappend newz [lindex $gbl(z,$i) $j]
	       lappend newsd [lindex $gbl(z,SD,$i) $j]
	    }
	 }
	 if { [llength $newx] < 1 || [lindex $newx 0] != $gbl(xmin) } {
	    # make left edge node
	    set newx [concat $gbl(xmin) $newx]
	    set newz [concat [interpolate_depth $i $gbl(xmin)] $newz]
	    set newsd [concat [lindex $newsd 0] $newsd]
	 }
	 if {  [llength $newx] < 2 || [lindex $newx end] != $gbl(xmax) } {
	    # make right edge node
	    lappend newx $gbl(xmax)
	    lappend newz [interpolate_depth $i $gbl(xmax)]
	    lappend newsd [interpolate_zSD $i $gbl(xmax)]
	 }
	 set gbl(x,$i) $newx
	 set gbl(z,$i) $newz
	 set gbl(z,SD,$i) $newsd
	 set gbl(n_node,$i) [llength $newx]
      }
      # shrink layers -----
      for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	 set newx {}
	 set newvtop {}
	 set newvbot {}
	 set newsdtop {}
	 set newsdbot {}
	 for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
	    set x [lindex $gbl(vx,$i) $j]
	    if { $x >= $gbl(xmin) && $x <= $gbl(xmax) } {
	       lappend newx $x
	       lappend newvtop [lindex $gbl(vtop,$i) $j]
	       lappend newvbot [lindex $gbl(vbot,$i) $j]
	       lappend newsdtop [lindex $gbl(vtop,SD,$i) $j]
	       lappend newsdbot [lindex $gbl(vbot,SD,$i) $j]
	    }
	 }
	 if { [llength $newx] < 1 || [lindex $newx 0] != $gbl(xmin) } {
	    # make left edge v-grid
	    set newx [concat $gbl(xmin) $newx]
	    set v [calc_v_layer $i $gbl(xmin)]
	    set newvtop [concat [lindex $v 0] $newvtop]
	    set newvbot [concat [lindex $v 1] $newvbot]
	    set v [calc_SD_layer $i $gbl(xmin)]
	    set newsdtop [concat [lindex $v 0] $newsdtop]
	    set newsdbot [concat [lindex $v 1] $newsdbot]
	 }
	 if { [llength $newx] < 2 || [lindex $newx end] != $gbl(xmin) } {
	    # make right edge v-grid
	    lappend newx $gbl(xmax)
	    set v [calc_v_layer $i $gbl(xmax)]
	    lappend newvtop [lindex $v 0]
	    lappend newvbot [lindex $v 1]
	    set v [calc_SD_layer $i $gbl(xmax)]
	    lappend newsdtop [lindex $v 0]
	    lappend newsdbot [lindex $v 1]
	 }
	 set gbl(vx,$i) $newx
	 set gbl(vtop,$i) $newvtop
	 set gbl(vbot,$i) $newvbot
	 set gbl(vtop,SD,$i) $newsdtop
	 set gbl(vbot,SD,$i) $newsdbot
	 set gbl(n_vnode,$i) [llength $newx]
      }
   }
   # Vertical direction --------------------
   set gbl(zmax) $wk(zmax)
   set gbl(zmin) $wk(zmin)

   draw_frame
   draw_model
}

# --------------------------------------------------------------------
# Background image related parameter tab
# --------------------------------------------------------------------
proc bgimage_parameter {ts} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   set w [frame $ts.vfield]
   $ts tab conf "BGimage" -window $w -fill x

   # parameters
   set f1 [radio_frame $w nobg "No background image" gbl(hamen,velimage) 0]
   set f2 [radio_labelframe $w vmodel "Velocity model" gbl(hamen,velimage) 1]
   make_menu_com $f2 mformat "Model format" gbl(hamen,mformat) ""\
       "Ray86" 1 "Ray86+gm" 2 "netCDF" 3
   filename_hscroll_frame $f2 cpt "CPT file" gbl(file,cptfile) 40 load\
       { {"cpt"  *.cpt} {"All"   *} }
   make_entry $f2 cntr "contour interval" gbl(bg,contour) "km/sec"
   make_entry $f2 density "image density" gbl(bg,density) "dpi"

   set f3 [radio_labelframe $w image "Image file(jpeg/png)"\
	       gbl(hamen,velimage) 2]
   filename_hscroll_frame $f3 image "Image file(jpeg/png)" gbl(file,bgimage)\
       40 load { {"image"  {*.jpg *.png}}  {"All"   *} }
   # netCDF file ----------
   set f4 [radio_labelframe $w cdf "netCDF file" gbl(hamen,velimage) 3]
   filename_hscroll_frame $f4 cdf "netCDF file" gbl(file,bgcdf)\
       40 load { {"netCDF"  {*.grd}}  {"All"   *} }
   # cpt file
   set f4f1 [radio_labelframe $f4 auto "Use automatic CPT" gbl(bg,cpt,mode) 0]
   make_menu_com $f4f1 cpttype "CPT master" gbl(bg,cpt,master) ""\
       "gray" "gray" "rainbow" "rainbow" "hot" "hot" "cool" "cool"
   make_checkbutton $f4f1 "Inverse CPT" cptinv gbl(bg,cpt,inverse)
   make_entry $f4f1 density "image density" gbl(bg,density) "dpi"
   set f4f2 [radio_labelframe $f4 specified "use CPT file" gbl(bg,cpt,mode) 1]
   filename_hscroll_frame $f4f2 cpt "CPT file" gbl(file,bgcdfcpt)\
       40 load { {"cpt"  {*.cpt}}  {"All"   *} }
   update_radio_frames $f4 gbl(bg,cpt,mode)
   grid $f4f1 - - - - - -sticky news
   grid $f4f2 - - - - - -sticky news

   # update
   update_radio_frames $w gbl(hamen,velimage)

   button_frame "" $w.button "update" "update_bgimage"
   # packing ------------------------------
   pack $f1 $f2 $f3 $f4 $w.button -side top -expand 1 -fill x
}
