# --------------------------------------------------------------------
# whow_pois_label ----------------------------------------------------
#            show poisson's ratio on the widget of $w
# --------------------------------------------------------------------
#  vpvs  : Vp/Vs
#  w     : a label widget
# --------------------------------------------------------------------
proc show_pois_label {vpvs w} {
   regexp {[\d\.]*} $vpvs val
   if { [string length $val] < 1 || [expr $val *1.0] <= 1.4 } { 
      $w conf -text "(-.----)"
      return 1 
   }
   $w conf -text [format "(%.4f)" [vpvs2poisson $val]]
   return 1
}

# --------------------------------------------------------------------
# make_vpvs_frame ----------------------------------------------------
#            vp/vs editing frame
# --------------------------------------------------------------------
# [arguments]
#  w     : parent widget name
#  layer : layer number
#  vnode : velocity grid number
# --------------------------------------------------------------------
proc make_vpvs_frame {w layer vnode} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   frame $w -relief ridge -bd 2
   # common widgets
   entry $w.e1 -width 7
   entry $w.e2 -width 7
   entry $w.e3 -width 7
   entry $w.e4 -width 7
   label $w.poi1 -text ""
   label $w.poi2 -text ""
   label $w.l2 -text "SD ratio"
   # each widgets
   switch $gbl(ERI,poisson) {
      0 {
	 # none
	 return
      }
      1 {
	 # 1 value per 1 layer
	 label $w.l1 -text "Vp/Vs of this layer"
	 set wl {$w.l1 $w.e1 $w.poi1}
	 set vpvs [poisson2vpvs $gbl(poi,$layer,top)]
	 $w.e1 insert 0 $vpvs
	 $w.e1 conf -validate key -validatecommand "show_pois_label %P $w.poi1"
	 show_pois_label $vpvs $w.poi1
	 if {$gbl(ERI,inv)} {
	    set wl [concat $wl {$w.l2 $w.e2}]
	    $w.e2 insert 0 [poisson2vpvs $gbl(poi,SD,$layer,top)]
	 }
	 eval grid $wl -sticky news
      }
      2 {
	 # 1 value per 1 grid
	 label $w.l1 -text "Vp/Vs of this grid"
	 button $w.a1 -text "Set All" \
	     -command "set_all_poisson $w.e1 $layer top"
	 set wl {$w.l1 $w.e1 $w.poi1 $w.a1}
	 set vpvs [poisson2vpvs [lindex $gbl(poi,$layer,top) $vnode]]
	 $w.e1 insert 0 $vpvs
	 $w.e1 conf -validate key -vcmd "show_pois_label %P $w.poi1"
	 show_pois_label $vpvs $w.poi1
	 if {$gbl(ERI,inv)} {
	    button $w.a2 -text "Set All"\
		-command "set_all_poisson_SD $w.e2 $layer top"
	    set wl [concat $wl {$w.l2 $w.e2 $w.a2}]
	    $w.e2 insert 0 [lindex $gbl(poi,SD,$layer,top) $vnode]
	 }
	 eval grid $wl -sticky news
      }
      3 {
	 # 2 values per 1 grid
	 foreach {i j k m n} {"Top" top e1 e2 poi1 "Bottom" bot e3 e4 poi2} {
	    label $w.l1($i) -text "$i Vp/Vs"
	    button $w.a1($i) -text "Set All"\
		-command "set_all_poisson $w.$k $layer $j"
	    set wl {$w.l1($i)   $w.$k    $w.$n   $w.a1($i)}
	    set vpvs [poisson2vpvs [lindex $gbl(poi,$layer,$j) $vnode]]
	    $w.$k insert 0 $vpvs
	    $w.$k conf -validate key -vcmd "show_pois_label %P $w.$n"
	    show_pois_label $vpvs $w.$n
	    if {$gbl(ERI,inv)} {
	       label $w.l2($i) -text "SD ratio"
	       button $w.a2($i) -text "Set All"\
		   -command "set_all_poisson_SD $w.$m $layer $j"
	       set wl [concat $wl {$w.l2($i)   $w.$m   $w.a2($i)}]
	       $w.$m insert 0 [lindex $gbl(poi,SD,$layer,$j) $vnode]
	    }
	    eval grid $wl -sticky news
	 }
      }
   }
}

# --------------------------------------------------------------------
# change_poisson_settings --------------------------------------------
#            Change Poisson's ratio settings
# --------------------------------------------------------------------
# [arguments]
#    n     : Poisson's ratio edit mode.
# --------------------------------------------------------------------
proc change_poisson_settings {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # Confirmation ------------------------------
   if { $gbl(ERI,poisson) == $gbl(ERI,poisson_orig) } { return }
   if {$gbl(ERI,poisson) < $gbl(ERI,poisson_orig) } {
      set ans [tk_messageBox -type yesno -icon warning -default no -message\
		   "You will lose some Poisson's ratio information.  Do you chage truly?"]
      if { $ans == "no" } {
	 set gbl(ERI,poisson) $gbl(ERI,poisson_orig)
	 return
      }
   }
   if {$wk(undo,n) > 0} {
      set ans [tk_messageBox -type yesno -icon warning -default no -message\
		   "You will lose all the undo information. OK?"]
      if {$ans == "no"} {
	 set gbl(ERI,poisson) $gbl(ERI,poisson_orig)
	 return
      }
   }
   # Clear the undo information
   set wk(undo,n) 0
   
   # update poisson's ratio information ------------------------------
   switch $gbl(ERI,poisson) {
      1 {
	 # to 1 value per 1 layer ----------
	 switch $gbl(ERI,poisson_orig) {
	    0 {
	       # from none
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set gbl(poi,$i,top) $wk(def,poi)
		  set gbl(poi,SD,$i,top) $wk(def,SDps)
		  set gbl(poi,$i,bot) $wk(def,poi)
		  set gbl(poi,SD,$i,bot) $wk(def,SDps)
	       }
	    }
	    2 {
	       # from 1 value per 1 grid
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set gbl(poi,$i,top) [lindex $gbl(poi,$i,top) 0]
		  set gbl(poi,SD,$i,top) [lindex $gbl(poi,SD,$i,top) 0]
		  set gbl(poi,$i,bot) [lindex $gbl(poi,$i,top) 0]
		  set gbl(poi,SD,$i,bot) [lindex $gbl(poi,SD,$i,top) 0]
	       }
	    }
	    3 {
	       # from 2 values per 1 grid
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set gbl(poi,$i,top) [lindex $gbl(poi,$i,top) 0]
		  set gbl(poi,SD,$i,top) [lindex $gbl(poi,SD,$i,top) 0]
		  set gbl(poi,$i,bot) [lindex $gbl(poi,$i,top) 0]
		  set gbl(poi,SD,$i,bot) [lindex $gbl(poi,SD,$i,top) 0]
	       }
	    }
	 }
      }
      2 {
	 # to 1 value per 1 grid ----------
	 switch $gbl(ERI,poisson_orig) {
	    0 {
	       # from none
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set gbl(poi,$i,top) ""
		  set gbl(poi,SD,$i,top) ""
		  set gbl(poi,$i,bot) ""
		  set gbl(poi,SD,$i,bot) ""
		  for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
		     lappend gbl(poi,$i,top) $wk(def,poi)
		     lappend gbl(poi,SD,$i,top) $wk(def,SDps)
		     lappend gbl(poi,$i,bot) $wk(def,poi)
		     lappend gbl(poi,SD,$i,bot) $wk(def,SDps)
		  }
	       }
	    }
	    1 {
	       # from 1 value per 1 layer
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set tmp1 $gbl(poi,$i,top)
		  set tmp2 $gbl(poi,SD,$i,top)
		  set gbl(poi,$i,top) ""
		  set gbl(poi,SD,$i,top) ""
		  set gbl(poi,$i,bot) ""
		  set gbl(poi,SD,$i,bot) ""
		  for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
		     lappend gbl(poi,$i,top) $tmp1
		     lappend gbl(poi,SD,$i,top) $tmp2
		     lappend gbl(poi,$i,bot) $wk(def,poi)
		     lappend gbl(poi,SD,$i,bot) $wk(def,SDps)
		  }
	       }
	    }
	    3 {
	       # from 2 values per 1 grid
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set tmp1 $gbl(poi,$i,top)
		  set tmp2 $gbl(poi,SD,$i,top)
		  set gbl(poi,$i,top) ""
		  set gbl(poi,SD,$i,top) ""
		  set gbl(poi,$i,bot) ""
		  set gbl(poi,SD,$i,bot) ""
		  for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
		     lappend gbl(poi,$i,top) [lindex $tmp1 $j]
		     lappend gbl(poi,SD,$i,top) [lindex $tmp2 $j]
		     lappend gbl(poi,$i,bot) $wk(def,poi)
		     lappend gbl(poi,SD,$i,bot) $wk(def,SDps)
		  }
	       }
	    }
	 }
      }
      3 {
	 # to 2 values per 1 grid ----------
	 switch $gbl(ERI,poisson_orig) {
	    0 {
	       # from none
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set gbl(poi,$i,top) ""
		  set gbl(poi,$i,bot) ""
		  set gbl(poi,SD,$i,top) ""
		  set gbl(poi,SD,$i,bot) ""
		  for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
		     lappend gbl(poi,$i,top) $wk(def,poi)
		     lappend gbl(poi,$i,bot) $wk(def,poi)
		     lappend gbl(poi,SD,$i,top) $wk(def,SDps)
		     lappend gbl(poi,SD,$i,bot) $wk(def,SDps)
		  }
	       }	       
	    }
	    1 {
	       # from 1 value per 1 layer
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set tmp1 $gbl(poi,$i,top)
		  set tmp2 $gbl(poi,SD,$i,top)
		  set gbl(poi,$i,top) ""
		  set gbl(poi,$i,bot) ""
		  set gbl(poi,SD,$i,top) ""
		  set gbl(poi,SD,$i,bot) ""
		  for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
		     lappend gbl(poi,$i,top) $tmp1
		     lappend gbl(poi,$i,bot) $tmp1
		     lappend gbl(poi,SD,$i,top) $tmp2
		     lappend gbl(poi,SD,$i,bot) $tmp2
		  }
	       }
	    }
	    2 {
	       # from 1 value per 1 grid
	       for {set i 1} {$i < $gbl(n_interface)} {incr i} {
		  set tmp1 $gbl(poi,$i,top)
		  set tmp2 $gbl(poi,SD,$i,top)
		  set gbl(poi,$i,top) ""
		  set gbl(poi,$i,bot) ""
		  set gbl(poi,SD,$i,top) ""
		  set gbl(poi,SD,$i,bot) ""
		  for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
		     set tmp [lindex $tmp1 $j]
		     lappend gbl(poi,$i,top) $tmp
		     lappend gbl(poi,$i,bot) $tmp
		     set tmp [lindex $tmp2 $j]
		     lappend gbl(poi,SD,$i,top) $tmp
		     lappend gbl(poi,SD,$i,bot) $tmp
		  }
	       }
	    }
	 }
      }
   }
   set gbl(ERI,poisson_orig) $gbl(ERI,poisson)
}

# --------------------------------------------------------------------
# initialize_poisson -------------------------------------------------
#            Initialize parameters of poisson's ratio relation
# --------------------------------------------------------------------
proc initialize_poisson {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # Poisson's ratio
   switch $gbl(ERI,poisson) {
      0 {
	 # none
	 for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	    set gbl(poi,$i,top) ""
	    set gbl(poi,$i,bot) ""
	    set gbl(poi,SD,$i,top) ""
	    set gbl(poi,SD,$i,bot) ""
	 }
	 return
      }
      1 {
	 # 1 value per 1 layer
	 for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	    set gbl(poi,$i,top) $wk(def,poi)
	    set gbl(poi,$i,bot) ""
	    set gbl(poi,SD,$i,top) $wk(def,SDps)
	    set gbl(poi,SD,$i,bot) ""
	 }
      }
      2 -
      3 {
	 # "1 value per 1 grid" and "2 values per 1 layer"
	 for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	    set gbl(poi,$i,top) ""
	    set gbl(poi,$i,bot) ""
	    set gbl(poi,SD,$i,top) ""
	    set gbl(poi,SD,$i,bot) ""
	    for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
	       lappend gbl(poi,$i,top) $wk(def,poi)
	       lappend gbl(poi,$i,bot) $wk(def,poi)
	       lappend gbl(poi,SD,$i,top) $wk(def,SDps)
	       lappend gbl(poi,SD,$i,bot) $wk(def,SDps)
	    }
	 }
      }
   }
}
# --------------------------------------------------------------------
# adjust_poisson -----------------------------------------------------
#            Adjust parameters of poisson's ratio relation 
#            after reading ray86
# --------------------------------------------------------------------
proc adjust_poisson {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # variable existence check
   for {set i 1} {$i < $gbl(n_interface)} {incr i} {
      foreach j "poi,$i,top poi,$i,bot poi,SD,$i,top poi,SD,$i,bot" {
	 if ![info exists gbl($j)] {
	    set gbl($j) ""
	 }
      }
   }

   # Adjust
   switch $gbl(ERI,poisson) {
      0 {
	 # none
	 for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	    set gbl(poi,$i,top) ""
	    set gbl(poi,$i,bot) ""
	    set gbl(poi,SD,$i,top) ""
	    set gbl(poi,SD,$i,bot) ""
	 }
	 return
      }
      1 {
	 # 1 value per 1 layer
	 for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	    foreach {j k} "poi,$i,top  $wk(def,poi)\
			   poi,SD,$i,top $wk(def,SDps)" {
	       if {[llength $gbl($j)] >= 1} {
		  set gbl($j) [lindex $gbl($j) 0]
	       } else {
		  set gbl($j) $k
	       }
	    }
	    set gbl(poi,$i,bot) ""
	    set gbl(poi,SD,$i,bot) ""
	 }
      }
      2 -
      3 {
	 # "1 value per 1 grid" and "2 values per 1 grid"
	 for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	    foreach {j k} "poi,$i,top $wk(def,poi)\
			   poi,SD,$i,top $wk(def,SDps)\
			   poi,$i,bot $wk(def,poi)\
			   poi,SD,$i,bot $wk(def,SDps)" {
	       if {[llength $gbl($j)] != $gbl(n_vnode,$i)} {
		  set gbl($j) {}
		  for {set m 0} {$m < $gbl(n_vnode,$i)} {incr m} {
		     lappend gbl($j) $k
		  }
	       }
	    }
	 }
      }
   }
}
# --------------------------------------------------------------------
# set_all_poisson ----------------------------------------------------
#            Set poisson's ratio of all the v-grid in the layer
# --------------------------------------------------------------------
# [arguments]
#  w     : entry widget path
#  layer : layer number
#  side  : "top" or "bot".
# --------------------------------------------------------------------
proc set_all_poisson {w layer side} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # undo info
   set_vgrid_undo_info vgrid.all_velocity $layer -1

   # set all poisson's ratio
   set val [vpvs2poisson [format "%.4f" [$w get]]]
   set gbl(poi,$layer,$side) ""
   for {set i 0} {$i < $gbl(n_vnode,$layer)} {incr i} {
      lappend gbl(poi,$layer,$side) $val
   }
}

# --------------------------------------------------------------------
# set_all_poisson_SD -------------------------------------------------
#            Set SD of poisson's ratio of all the v-grid in the layer
# --------------------------------------------------------------------
# [arguments]
#  w     : entry widget path
#  layer : layer number
#  side  : "top" or "bot".
# --------------------------------------------------------------------
proc set_all_poisson_SD {w layer side} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # undo info
   set_vgrid_undo_info vgrid.all_velocity $layer -1

   # set SD of all poisson's ratio
   set val [format "%.4f" [$w get]]
   set gbl(poi,SD,$layer,$side) ""
   for {set i 0} {$i < $gbl(n_vnode,$layer)} {incr i} {
      lappend gbl(poi,SD,$layer,$side) $val
   }
}

# --------------------------------------------------------------------
# set_multiple_poisson -----------------------------------------------
#            Set poisson's ratio between wk(vxmin) and wk(vxmax)
# --------------------------------------------------------------------
# [arguments]
#  w     : entry widget path
#  layer : layer number
#  side  : "top" or "bot".
# --------------------------------------------------------------------
proc set_multiple_poisson {w layer side} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # undo info
   set_vgrid_undo_info vgrid.all_velocity $layer -1

   # set all poisson's ratio
   set val [vpvs2poisson [format "%.4f" [$w get]]]
   set prev $gbl(poi,$layer,$side)
   set gbl(poi,$layer,$side) ""
   for {set i 0} {$i < $gbl(n_vnode,$layer)} {incr i} {
      set x [lindex $gbl(vx,$layer) $i]
      if { $x >= $wk(vxmin) && $x <= $wk(vxmax) } {
	 lappend gbl(poi,$layer,$side) $val
      } else {
	 lappend gbl(poi,$layer,$side) [lindex $prev $i]
      }
   }
}

# --------------------------------------------------------------------
# set_all_poisson_SD -------------------------------------------------
#            Set SD of poisson's ratio between wk(vxmin) and wk(vxmax)
# --------------------------------------------------------------------
# [arguments]
#  w     : entry widget path
#  layer : layer number
#  side  : "top" or "bot".
# --------------------------------------------------------------------
proc set_multiple_poisson_SD {w layer side} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # undo info
   set_vgrid_undo_info vgrid.all_velocity $layer -1

   # set SD of all poisson's ratio
   set val [format "%.4f" [$w get]]
   set prev $gbl(poi,SD,$layer,$side)
   set gbl(poi,SD,$layer,$side) ""
   for {set i 0} {$i < $gbl(n_vnode,$layer)} {incr i} {
      set x [lindex $gbl(vx,$layer) $i]
      if { $x >= $wk(vxmin) && $x <= $wk(vxmax) } {
	 lappend gbl(poi,SD,$layer,$side) $val
      } else {
	 lappend gbl(poi,SD,$layer,$side) [lindex $prev $i]
      }
   }
}

# --------------------------------------------------------------------
# s_vel --------------------------------------------------------------
#            Compute S-velocity using poisson's ratio and p-velocity
# --------------------------------------------------------------------
# [arguments]
#  p  : p velocity
#  n  : poisson's ratio
# --------------------------------------------------------------------
proc s_vel {p n} {
   return [expr sqrt( ( 1.0 - 2.0 *$n) / ( 2.0 * ( 1.0 - $n) ) ) * $p]
}

# --------------------------------------------------------------------
# poisson2vpvs -------------------------------------------------------
#            Compute Vp/Vs ratio using Poisson's ratio
# --------------------------------------------------------------------
#  n  : poisson's ratio
# --------------------------------------------------------------------
proc poisson2vpvs {n} {
   return [format "%.3f" \
	       [expr sqrt( ( 2.0 * ( 1.0 - $n) / ( 1.0 - 2.0 *$n) ) )]]
}

# --------------------------------------------------------------------
# vpvs2poisson -------------------------------------------------------
#            Compute Poisson's ratio from Vp/Vs
# --------------------------------------------------------------------
#  r  : Vp/Vs
# --------------------------------------------------------------------
proc vpvs2poisson {r} {
   return [format "%.4f" [expr ($r*$r - 2.0) / ( 2.0*($r*$r - 1.0) )]]
}

# --------------------------------------------------------------------
# calc_poisson -------------------------------------------------------
#            Compute poisson's ratio at point P.
# --------------------------------------------------------------------
# [arguments]
#  x  : x-coordinate of P
#  z  : y-coordinate of P
# --------------------------------------------------------------------
proc calc_poisson {x z} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   switch $gbl(ERI,poisson) {
      0 {
	 # none
	 return ""
      }
      1 {
	 # 1 value per 1 layer
	 set layer [find_layer $x $z]
	 return $gbl(poi,$layer,top)
      }
      2 {
	 # 1 value per 1 grid
	 set layer [find_layer $x $z]
	 set right [search_vgrid $layer $x]
	 set left [expr $right -1]
	 set x1 [lindex $gbl(vx,$layer) $left]
	 set x2 [lindex $gbl(vx,$layer) $right]
	 set p1 [lindex $gbl(poi,$layer,top) $left]
	 set p2 [lindex $gbl(poi,$layer,top) $right]
	 return [expr ( ($x2 - $x)*$p1 + ($x - $x1)*$p2 ) / ($x2 - $x1) ]
      }
      3 {
	 # 2 values per 1 layer
	 set info [find_trape $x $z]
	 if {$info == ""} {
	    return ""
	 } else {
	    set i [lindex $info 0]
	    set x1 [lindex $info 1]
	    set x2 [lindex $info 2]
	 }
	 
	 # Compute poisson's ratio at the vertexes
	 set v1 [calc_p_layer $i $x1]
	 set v2 [calc_p_layer $i $x2]
	 set p1top [lindex $v1 0]
	 set p2top [lindex $v2 0]
	 set p1bot [lindex $v1 1]
	 set p2bot [lindex $v2 1]
	 # Retrun
	 return [zelt_velocity $i $x1 $p1top $p1bot $x2 $p2top $p2bot $x $z]
      }
   }
   return ""
}

# --------------------------------------------------------------------
# calc_p_layer -------------------------------------------------------
#            Compute poisson's ratio at the interface of $i-th layer at $x.
# --------------------------------------------------------------------
# [arguments]
#  i  :  layer number.
#  x  :  x-coordinate.
# --------------------------------------------------------------------
proc calc_p_layer {i x} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   set j2 [search_vgrid $i $x]
   set j1 [expr $j2 - 1]
   set x1 [lindex $gbl(vx,$i) $j1]
   set x2 [lindex $gbl(vx,$i) $j2]
   foreach mode {top bot} {
      set v1 [lindex $gbl(poi,$i,$mode) $j1]
      set v2 [lindex $gbl(poi,$i,$mode) $j2]
      set $mode [expr ($v1*($x2 - $x) + $v2*($x - $x1)) / ($x2 - $x1)]
   }
   # return
   return "$top $bot"
}
