# --------------------------------------------------------------------
# open_project -------------------------------------------------------
#           Open project file
# --------------------------------------------------------------------
proc open_project {} {
   # Global variables --------------------------------------------------
   global gbl wk

   # Main --------------------------------------------------
   # If a model already exists, ask save or not.
   if {$gbl(n_interface) > 0} {
      # Confirm to save or not.
      set ans [tk_messageBox -title "Save Current Project?"\
		   -message "Do you want to save current Project?"\
		   -type yesnocancel -icon question]
      if {$ans=="yes"} {
	 save_project overwrite
      } elseif {$ans=="cancel"} {
	 bar_message "Opening project file was canceled."
	 return
      }
   }
   # Select porject file ----------
   set types {
      {"project files" {.pro} } 
      {"Dat files"	 {.dat}	}
      {"All files"		*}
   }
   set data_file [tk_getOpenFile -filetypes $types -parent .]
   if [string compare $data_file ""] {
      # When a file is selected.
      bar_message "Load Project file \"$data_file\" ..."
      update
      set wk(file,project) $data_file
      read_project
   } else {
      # When canceled
      bar_message "No file is selected."
      return
   }
}
# --------------------------------------------------------------------
# read_project
# --------------------------------------------------------------------
proc read_project {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # Evaluate project file(project file is written in Tcl.) ----------
   source $wk(file,project)
   update_titlebar
   bar_message "Project file was read."
   
   # Check file paths (avoid /export and /mnt problems)
   for {set i 1} { $i <= $gbl(src,n_source) } {incr i} {
      if { [set new_fn [read_file_check $gbl(src,$i,pick)\
			    "pick file for No.$i source"]] != 0} {
	 set gbl(src,$i,pick) $new_fn
      } else {
	 set gbl(src,$i,pick) ""
      }
   }
   # compatibility with V1.3 and former
   if [info exists gbl(file,ray86_model)] {
      if [file readable $gbl(file,ray86_model)] {
	 read_ray86 $gbl(file,ray86_model)
	 if $wk(read_status) {
	    bar_message "Reading model file error."
	 }
      }
      unset gbl(file,ray86_model)
   }
   if [info exists gbl(file,project)] {
      unset gbl(file,project)
   }
   
   # Change directory to where project file is
   cd [file dirname $wk(file,project)]
   
   # update of bindings, font size, hiding-axis and undo info
   after_read_model
   update_ALT_bindings
   update_velocity_font
   update_hide_axis x2
   update_hide_axis y2
   set wk(undo,n) 0
   show_bgimage
}
# --------------------------------------------------------------------
# save_project -------------------------------------------------------
#           Save project file
# --------------------------------------------------------------------
# mode : "new" or "overwrite"
# --------------------------------------------------------------------
proc save_project {mode} {
   # Global variables --------------------------------------------------
   global gbl sys wk

   # Main --------------------------------------------------
   if ![file writable $wk(file,project)] {
      set mode "new"
   }

   # Select output file ----------
   if {[string compare $mode "new"] == 0} {
      set types {
	 {"project files" {.pro} } 
	 {"Dat files"	{.dat}	}
	 {"All files"		*}
      }
      set data_file [tk_getSaveFile -filetypes $types -parent .\
			 -initialfile $wk(file,project) -defaultextension .pro]
      if ![string compare $data_file ""] {
	 bar_message "Save canceled..."
	 update
	 return
      }
      set wk(file,project) $data_file
   }
   # Convert filenames into relative path? ----------
   if { $gbl(option,use_relative_path) } {
      convert_filenames_relative
   } else {
      convert_filenames_absolute
   }

   # Write project file ----------
   bar_message "Writting to Project file..."
   set fp [open $wk(file,project) w]

   # output all the variables in the gbl array
   foreach { member } [lsort [array names gbl]] {
      puts $fp [format "set %s \"%s\"" gbl($member) $gbl($member)]
   }

   # Close!
   close $fp
   bar_message "Project was written in $wk(file,project)"
   update_titlebar
}
# --------------------------------------------------------------------
# read_file_check ----------------------------------------------------
#            Read file existence check
#            /export and /mnt manipulation.
# --------------------------------------------------------------------
# [arguments]
# fn    :  filename
# m     :  file type description (message in the error window)
# --------------------------------------------------------------------
proc read_file_check {fn m} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    # Null(empty) filename is valid
    if { $fn == ""} {
	return $fn
    }
    # exist and readable ?
    if [file exists $fn] {
	if { [file readable $fn] && [file isfile $fn] } {
	    return $fn
	}
	tk_messageBox -icon error\
	    -message "$m file \"$fn\" is not readable file for you."
	return 0
    }
    # /export/ , /mnt/ prefix problems
    set prefix {"/export/" "/mnt/" "/rmnt/"}
    foreach p $prefix {
	if [string match [format "%s*" $p] $fn] {
	    set new_fn [format "/%s" [string range $fn [string length $p] end]]
	    if { [file exists $new_fn] && [file readable $new_fn]\
		     && [file isfile $new_fn]} {
		return $new_fn
	    }
	}
    }
    tk_messageBox -icon error -message "$m file \"$fn\" not found."
    return 0
}
