namespace eval RAYINVR {
   namespace export *

   # --------------------------------------------------------------------
   # init : initialize parameters for rayinvr
   # --------------------------------------------------------------------
   proc init {} {
      global gbl sys

      # register to menu bar, and add key binding
      register_backend RAYINVR rayinvr z "RAYINVR::parameter_window"

      # declare constants
      set sys(rayinvr,TRAMP)      "$sys(RAYINVR_DIR)/tramp"
      set sys(rayinvr,XTRAMP)     "$sys(RAYINVR_DIR)/xtramp"
      set sys(rayinvr,RAYTRACE)   "$sys(RAYINVR_DIR)/rayinvr"
      set sys(rayinvr,XRAYTRACE)  "$sys(RAYINVR_DIR)/xrayinvr"
      set sys(rayinvr,RAYCOORDS)  "$sys(RAYINVR_DIR)/raycoords"
      set sys(rayinvr,SYNTHETIC)  "$sys(RAYINVR_DIR)/pltsyn"
      set sys(rayinvr,XSYNTHETIC) "$sys(RAYINVR_DIR)/xpltsyn"

      # declare variables
      set gbl(zelt,plot,iroute) 5
      set gbl(zelt,plot,iray)   1
      set gbl(zelt,plot,xwndow) 200
      set gbl(zelt,plot,ywndow) 135
      set gbl(zelt,plot,imod)   1
      set gbl(zelt,plot,ibnd)   0
      set gbl(zelt,plot,ivel)   0
      set gbl(zelt,plot,idot)   0
      set gbl(zelt,plot,symht)  0.5
      set gbl(zelt,plot,nrskip) 1
      set gbl(zelt,plot,npskip) 1
      set gbl(zelt,plot,vred)   0.0
      set gbl(zelt,plot,isep)   0
      set gbl(zelt,plot,itx)    1
      set gbl(zelt,plot,isum)   0
      set gbl(zelt,plot,itxout) 1
      set gbl(zelt,plot,idata)  1
      set gbl(zelt,plot,ircol)  2
      set gbl(zelt,plot,itcol)  0
      set gbl(zelt,plot,colour) 2,3,6,4,5,8,9,10,12,1
      set gbl(zelt,plot,iplot)  1

      set gbl(zelt,axis,xmax)   0
      set gbl(zelt,axis,xmin)   0
      set gbl(zelt,axis,zmax)   0
      set gbl(zelt,axis,zmin)   0
      set gbl(zelt,axis,xmm)    170
      set gbl(zelt,axis,zmm)    50
      set gbl(zelt,axis,tmm)    50
      set gbl(zelt,axis,albht)  2.5
      set gbl(zelt,axis,iaxlab) 1
      set gbl(zelt,axis,itrev)  0
      set gbl(zelt,axis,orig)   12.5

      set gbl(zelt,trace,imodf)  1
      set gbl(zelt,trace,ibsmth) 1
      set gbl(zelt,trace,ishot)  2
      set gbl(zelt,trace,nsmax)  10
      set gbl(zelt,trace,ifast)  0
      set gbl(zelt,trace,nray)   10
      set gbl(zelt,trace,step)   0.05
      set gbl(zelt,trace,stol)   0.05
      set gbl(zelt,trace,amin)   0
      set gbl(zelt,trace,amax)   90

      set gbl(zelt,invr,invr)   0
      set gbl(zelt,invr,ivray)  1

      set gbl(zelt,amp,iamp)   0
      set gbl(zelt,amp,icomp)  1
      set gbl(zelt,amp,icmp)   0
      set gbl(zelt,amp,surcon) 0
      set gbl(zelt,amp,amprev) 1
      set gbl(zelt,amp,spamp)  1

      set gbl(zelt,section,isect)  0
      set gbl(zelt,section,xincs)  5
      set gbl(zelt,section,iplot)  1
      set gbl(zelt,section,iconv)  1
      set gbl(zelt,section,iwavlt) 0
      set gbl(zelt,section,ipol)   0
      set gbl(zelt,section,iscale) 0
      set gbl(zelt,section,ishade) 1
      set gbl(zelt,section,ifill)  1
      set gbl(zelt,section,vred)   0.0
      set gbl(zelt,section,sps)    100.0
      set gbl(zelt,section,scalef) 100.0
      set gbl(zelt,section,rcor)   1.0
      set gbl(zelt,section,xnorm)  100.0
      set gbl(zelt,section,clip)   0
      set gbl(zelt,section,nptsw)  19
      set gbl(zelt,section,nskip)  1
      set gbl(zelt,section,inmo)   0
      set gbl(zelt,section,vrms)   2.5
      set gbl(zelt,section,iamp)   0
      set gbl(zelt,section,idump)  0
      set gbl(zelt,section,amp)    1.0
      set gbl(zelt,section,itrev)  0
      set gbl(zelt,section,xmax)   10
      set gbl(zelt,section,xmin)   1
      set gbl(zelt,section,tmax)   10
      set gbl(zelt,section,tmin)   0
      set gbl(zelt,section,xmm)    170
      set gbl(zelt,section,tmm)    100
      set gbl(zelt,section,albht)  3
      set gbl(zelt,section,orig)   20
      set gbl(zelt,section,nsmth)  0
   }
   


   # --------------------------------------------------------------------
   # Execute raytracing
   # --------------------------------------------------------------------
   proc raytracing {} {
      # Global variables --------------------------------------------------
      global gbl sys
   
      # Main --------------------------------------------------
   
      # Generate "v.in" ------------------------------
      bar_message "Making structure file 'v.in' ...."
      if { [write_vin "v.in"] != 0 } { return }

      # Generate "tx.in" ------------------------------
      bar_message "Making parameter file 'tx.in' ...."
      make_txin

      # Generate "r.in" ------------------------------
      bar_message "Making parameter file 'r.in' ...."
      if {[make_rin "raytrace"] != 0} { return }

      # Unlink old results -----
      foreach fn {"rp.out" "tx.out"} {
	 if [file exists $fn] { file delete $fn }
      }

      # Invoke subprocess ----------
      bar_message "Executing ray-trace..."
      if { $gbl(zelt,plot,iroute) < 5 } {
	 # XRAYINVR
	 set com [open "| $sys(rayinvr,XRAYTRACE)" r+]
      } else {
	 # inside modeling
	 set com [open "| $sys(rayinvr,RAYCOORDS)" r+]
      }
      exec_rayinvr 1 $com

      # Plot raypaths and tx graph
      if { $gbl(zelt,plot,iroute) == 5} { plot_rays }
      TX::plot "./tx.out"

      # Generate synthetic traveltime data file in MacRay Format.
      txout2macray

      # Fin ------------------------------
      if { $gbl(zelt,plot,iroute) == 2 || $gbl(zelt,plot,iroute) == 4 } {
	 bar_message "END: ray-tracing (Output to PostScript File, p.out)"
      } else {
	 bar_message "END: ray-tracing"
      }
   }

   # --------------------------------------------------------------------
   # calculating amplitude
   # --------------------------------------------------------------------
   proc amplitude {} {
      # Global variables --------------------------------------------------
      global gbl sys
   
      # Main --------------------------------------------------

      # Generate "v.in" ------------------------------
      bar_message "Making structure file 'v.in' ...."
      if { [write_vin "v.in"] != 0 } { return }

      # Generate "tx.in" ------------------------------
      bar_message "Making parameter file 'tx.in' ...."
      make_txin

      # Generate "r.in" ------------------------------
      bar_message "Making parameter file 'r.in' ...."
      if {[make_rin "amp"] != 0} { return }

      # Execute! ------------------------------
      bar_message "Calculating amplitude..."

      # Unlink old results -----
      foreach fn {"rp.out" "tx.out"} {
	 if [file exists $fn] { file delete $fn }
      }
      
      # Invoke subprocess ----------
      if { $gbl(zelt,plot,iroute) == 1} {
	 # xtramp (with X)
	 set com [open "| $sys(rayinvr,XTRAMP)" r+]
	 exec_rayinvr 1 $com
      } else {
	 # tramp (without X)
	 set com [open "| $sys(rayinvr,TRAMP)" r+]
	 exec_rayinvr 0 $com
      }

      # Plot raypaths and tx graph
      if { $gbl(zelt,plot,iroute) == 5} { plot_rays }
      TX::plot "./tx.out"

      # Generate synthetic traveltime data file in MacRay Format.
      txout2macray

      # Display Synsetic Section (isect = on)
      if { $gbl(zelt,section,isect) > 0 } {
	 display_synthetic_section x
      }

      # Fin ------------------------------
      if { $gbl(zelt,plot,iroute) == 0 } {
	 bar_message "END: ray-tracing (Output to PostScript File, p.out)"
      } else {
	 bar_message "END: ray-tracing"
      }
   }

   # --------------------------------------------------------------------
   # exec_rayinvr -------------------------------------------------------
   #            Exec rayinvr as a external process
   # --------------------------------------------------------------------
   # [arguments]
   #   wait :  wait(1) or not(0)
   #   fp   :  PIPE pointer for external process
   # --------------------------------------------------------------------
   proc exec_rayinvr {wait fp} {
      # Global variables --------------------------------------------------
      global sys gbl

      # Main --------------------------------------------------
      while {1} {
	 # If the subprocess is over, quit loop.
	 if {[gets $fp line] == -1} {
	    break;
	 }
	 # Next window?
	 if {[string match "*<CR>*" $line] == 1} {
	    if { $wait } {
	       # wait for click
	       set tw .tramp
	       catch {destroy $tw}
	       toplevel $tw
	       wm title $tw "Continue"
	       button $tw.button -text "Click to Continue"\
		   -font bold_font -command "destroy $tw"
	       pack $tw.button
	       update
	       tkwait window $tw
	    }
	    puts -nonewline $fp "\n"
	    flush $fp
	    update
	 }
	 # If waiting to quit, display "quit" button.
	 if {[string match "*quit*" $line] == 1} {
	    if { $wait } {
	       # wait for click
	       set tw .tramp
	       catch {destroy $tw}
	       toplevel $tw
	       wm title $tw "Close"
	       button $tw.button -text "  Close  "\
		   -font bold_font -command "destroy $tw"
	       pack $tw.button
	       update
	       tkwait window $tw
	    }
	    puts $fp q
	    flush $fp
	    break;
	 }
	 # Display output of subprocess to STDOUT.
	 if {$gbl(option,exec_condition)} {
	    puts stdout $line
	 }
      }
      # Kill subprocess, if necessary.
      catch {close $fp}
   }

   # --------------------------------------------------------------------
   # generate tx.in for rayinvr
   # --------------------------------------------------------------------
   proc make_txin {} {
      # Global variables
      global gbl sys
      
      # Main
      set shot_range 0.0
      set fn "./tx.in"
      catch {file delete $fn}
      set out_fp [open $fn "a"]
      for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
	 if {$gbl(src,$i,onoff)} {
	    set shot_range $gbl(src,$i,x)
	    switch -- [discriminate_pickfile $gbl(src,$i,pick)] {
	       "zelt" {
		  tx_in_append_zelt $out_fp $gbl(src,$i,pick)
	       }
	       "macray" {
		  tx_in_append_macray $out_fp $gbl(src,$i,pick)\
		      $gbl(plot,pick_acc)
	       }
	       "none" {
		  # not appropriate pick file
		  if { $gbl(trace,2pt) > 0} {
		     tx_in_append_auto_2pt $out_fp $shot_range
		  }
	       }
	    }
	 }
      }
      # output the end of data.
      puts $out_fp [format "%10.3f%10.3f%10.3f%10d" 0.0 0.0 0.0 -1]
      close $out_fp
   }

   # --------------------------------------------------------------------
   # make_rin -----------------------------------------------------------
   #           Generate "r.in" file
   # --------------------------------------------------------------------
   # [arguments]
   #   mode :  raytrace or amp
   # --------------------------------------------------------------------
   proc make_rin {mode} {
      # Global variables --------------------------------------------------
      global gbl sys
      
      # Main --------------------------------------------------
      # Open output file
      if [catch {set fp [open "r.in" w]}] {
	 bar_message "ERROR: Can't make r.in file !!"
	 return -1
      }
      
      # plot parameter --------------------
      switch -- $gbl(zelt,plot,iroute) {
	 1 {
	    # Plot to Zelt Screen
	    set gbl(zelt,plot,iplot) 1
	 }
	 5 {
	    if { $mode == "raytrace" } {
	       # Plot inside modeling
	       set gbl(zelt,plot,iplot) -1
	    } else {
	       # Plot to Zelt screen if amplitude
	       set gbl(zelt,plot,iplot) 1
	    }
	 }
	 default {
	    # PostScript output
	    set gbl(zelt,plot,iplot) 0
	 }
      }
      
      # two-point ?
      if { $gbl(trace,2pt) == 1 } {
	 set gbl(zelt,plot,itxout) 2
      } else {
	 set gbl(zelt,plot,itxout) 1
      }
      
      # update gbl(zelt,*) by common parameters
      set gbl(zelt,plot,vred) $gbl(plot,vred)
      set gbl(zelt,axis,xmax) $gbl(xmax)
      set gbl(zelt,axis,xmin) $gbl(xmin)
      set gbl(zelt,axis,zmax) $gbl(zmax)
      set gbl(zelt,axis,zmin) $gbl(zmin)
      set gbl(zelt,plot,itx)  $gbl(plot,tx)
      set gbl(zelt,plot,itcol) $gbl(plot,pick_color)
      set gbl(zelt,plot,ircol) $gbl(plot,ray_color)
      set gbl(zelt,axis,itrev) $gbl(plot,taxis_dir)
      set gbl(zelt,trace,i2pt) $gbl(trace,2pt)
      set gbl(zelt,axis,tmin) $gbl(tmin)
      set gbl(zelt,axis,tmax) $gbl(tmax)

      # output plot parameters
      puts $fp "  &pltpar"
      foreach i [array names gbl zelt,plot,* ] {
	 if {[string length $gbl($i)] > 0} {
	    set key [lindex [split $i ,] end]
	    puts $fp "\t $key=$gbl($i)"
	 }
      }
      puts $fp "  &end"
      # axis parameter --------------------
      puts $fp "  &axepar"
      foreach i [array names gbl zelt,axis,*] {
	 if {[string length $gbl($i)] > 0} {
	    set key [lindex [split $i ,] end]
	    puts $fp "\t $key=$gbl($i)"
	 }
      }
      puts $fp "  &end"
      # trace parameter --------------------
      puts $fp "  &trapar"
      foreach i [array names gbl zelt,trace,*] {
	 set key [lindex [split $i ,] end]
	 if {[string length $gbl($i)] > 0 && [string compare $key spec]} {
	    puts $fp "\t $key=$gbl($i)"
	 }
      }
      # Source parameter (redifine ishot) --------------------
      set xshot ""
      set zshot ""
      set ishot ""
      for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
	 if {$gbl(src,$i,onoff)} {
	    set xshot "$xshot$gbl(src,$i,x),"
	    set zshot "$zshot$gbl(src,$i,z),"
	    set ishot "$ishot$gbl(zelt,trace,ishot),"
	 }
      }
      puts $fp "\t xshot=$xshot"
      puts $fp "\t zshot=$zshot"
      puts $fp "\t ishot=$ishot"
      if {$xshot == ""} {
	 bar_message "No source is specified!!"
	 return -1
      }
      # Ray group parameter --------------------
      set raygroup {}
      set convert {}
      set c_interface {}
      set multiple {}
      set m_interface {}
      
      # Refraction, reflection, headwave
      for {set i 1} {$i <$gbl(n_interface)} {incr i} {
	 foreach {j k} {refract 1 reflect 2 head 3} {
	    if {$gbl(ray,$j,$i) == 1} {
	       lappend raygroup $i.$k
		  lappend convert 0
	       lappend multiple 0
	    }
	 }
      }
      # Converted wave + take off angle specification
      for {set i 1} {$i <= $sys(MAX_COMPLEX_WAVE)} {incr i} {
	 if {$gbl(ray,convert,$i,onoff)} {
	    lappend raygroup [format "%d.%d" $gbl(ray,convert,$i,layer)\
				  [ray_type $gbl(ray,convert,$i,mode)]]
	    set n [count_reflector $gbl(ray,convert,$i,interface)]
	    lappend convert $n
	    if {$n > 0} {
	       lappend c_interface \
		   [format "%s"\
			[make_reflector $gbl(ray,convert,$i,interface)] ]
	    }
	    set n [count_reflector $gbl(ray,convert,$i,rbnd)]
	    lappend multiple $n
	    if {$n > 0} {
	       lappend m_interface \
		   [format "%s"\
			[make_reflector $gbl(ray,convert,$i,rbnd)] ]
	    }
	 }
      }
      puts $fp [format "\t ray=%s" [join $raygroup ","]]
      puts $fp [format "\t ncbnd=%s" [join $convert ","]]
      puts $fp [format "\t nrbnd=%s" [join $multiple ","]]
      if {[llength $c_interface] > 0} {
	 puts $fp [format "\t cbnd=%s" [join $c_interface ","]]
      }
      if {[llength $m_interface] > 0} {
	 puts $fp [format "\t rbnd=%s" [join $m_interface ","]]
      }
      if {$raygroup == ""} {
	 bar_message "No Ray Group is specified!!"
	 close $fp
	 return -1
      }
      # Poisson's ratio
      switch -- $gbl(ERI,poisson) {
	 0 {
	    # None
	 }
	 1 {
	    # 1 value per 1 layer
	    set pois {}
	    for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	       lappend pois $gbl(poi,$i,top)
	    }
	    puts $fp [format "\t pois=%s" [join $pois ","]]
	 }
	 3 -
	 2 {
	    # 1 value per 1 grid
	    set plist [make_pois_1per1grid]
	    puts $fp [format "\t pois=%s" [join [lindex $plist 0] ","]]
	    if {[llength [lindex $plist 1]] > 0} {
	       puts $fp [format "\t poisl=%s" \
			     [long_list2multiple_lines [lindex $plist 1]]]
	       puts $fp [format "\t poisb=%s" \
			     [long_list2multiple_lines [lindex $plist 2]]]
	       puts $fp [format "\t poisbl=%s" \
			     [long_list2multiple_lines [lindex $plist 3]]]
#	       puts $fp [format "\t poisl=%s" [join [lindex $plist 1] ","]]
#	       puts $fp [format "\t poisb=%s" [join [lindex $plist 2] ","]]
#	       puts $fp [format "\t poisbl=%s" [join [lindex $plist 3] ","]]
	    }
	 }
      }

      puts $fp "  &end"
      # invr parameter --------------------
      puts $fp "  &invpar"
      foreach i [array names gbl zelt,invr,*] {
	 if {[string length $gbl($i)] > 0} {
	    set key [lindex [split $i ,] end]
	    puts $fp "\t $key=$gbl($i)"
	 }
      }
      puts $fp "  &end"
      # amp parameter --------------------
      puts $fp "  &amppar"
      foreach i [array names gbl zelt,amp,*] {
	 if {[string length $gbl($i)] > 0} {
	    set key [lindex [split $i ,] end]
	    puts $fp "\t $key=$gbl($i)"
	 }
      }
      # isect (Whether computes "syn.out" or not)
      set isect [expr $gbl(zelt,section,isect)/2]
      puts $fp "\t isect=$isect"
      # In case of using xmins, xmaxs and xincs.
      if {$gbl(zelt,section,isect) == 2 || $gbl(zelt,section,isect) == 3} {
	 if {[string length $gbl(zelt,section,xincs)] > 0} {
	    puts $fp "\t xincs=$gbl(zelt,section,xincs)"
	 }
	 puts $fp "\t xmins=$gbl(zelt,section,xmin)"
	 puts $fp "\t xmaxs=$gbl(zelt,section,xmax)"
      }
      puts $fp "  &end"
      close $fp
      return 0
   }

   # --------------------------------------------------------------------
   # format "long list" in multiple line text for r.in
   # --------------------------------------------------------------------
   proc long_list2multiple_lines {ls} {
      set out ""
      set counter 11
      foreach item $ls {
	 incr counter -1
	 if {$counter <= 0} {
	    set out [format "%s\n\t     " $out]
	    set counter 10
	 }
	 set out [format "%s%s," $out $item]
      }
      return $out
   }

   # --------------------------------------------------------------------
   # ray_type -----------------------------------------------------------
   #           Identify ray type.
   # --------------------------------------------------------------------
   # [arguments]
   # type : "     Refract in          " and so on.
   # --------------------------------------------------------------------
   proc ray_type {type} {
      # Global variables --------------------------------------------------
      global gbl

      # Main --------------------------------------------------
      switch -regexp $type {
	 Refract {
	    return 1
	 }
	 Reflect {
	    return 2
	 }
	 Head {
	    return 3
	 }
	 Take {
	    return 0
	 }
	 default {
	    puts stderr "Invalid type"
	    exit
	 }
      }
   }

   # --------------------------------------------------------------------
   # count_reflector ----------------------------------------------------
   #           Count number of reflector
   # --------------------------------------------------------------------
   # [arguments]
   # s   : List of interface number, such as "1,3,5".
   # --------------------------------------------------------------------
   proc count_reflector {s} {
      # Global variables --------------------------------------------------
      global gbl

      # Main --------------------------------------------------
      # "Comma" spliting array.
      set a [split $s ","]
      set n 0
      foreach i $a {
	 regexp {[\-\d]+} $i number
	 if {[string length $number] > 0} {
	    incr n
	 }
      }
      return $n
   }

   # --------------------------------------------------------------------
   # make_reflector ----------------------------------------------------
   #           Make reflector list expression.
   # --------------------------------------------------------------------
   # [arguments]
   # s   : List of interface number, such as "1,3,5".
   # --------------------------------------------------------------------
   proc make_reflector {s} {
      # Global variables --------------------------------------------------
      global gbl

      # Main --------------------------------------------------
      # Comma spliting array
      set a [split $s ","]
      if { [llength $a] < 1 } { return }
      set new {}
      foreach i $a {
	 regexp {[\-\d]+} $i number
	 if {[string length $number] > 0} {
	    lappend new $number
	 }
      }
      return [join $new ","]
   }
   # --------------------------------------------------------------------
   # make_pois_1per1grid ------------------------------------------------
   #           Make pois parameter for "1 poisson's ratio value per 1 grid"
   # --------------------------------------------------------------------
   proc make_pois_1per1grid {} {
      # Global variables --------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      # Initialize results --------------------
      set pois {}
      set poisl {}
      set poisb {}
      set poisbl {}


      # Make poisson's ratio parameters --------------------
      for {set i 1} {$i < $gbl(n_interface)} {incr i} {
	 # Make trapezoid information  ----------
	 set xlist [lsort -real "$gbl(x,$i) $gbl(x,[expr $i+1]) $gbl(vx,$i)"]
	 set xlist [unique_list $xlist]
	 set xmlist {}
	 set plist {}
	 for {set j 0} {$j < [expr [llength $xlist] - 1]} {incr j} {
	    set xm [expr ([lindex $xlist $j] + [lindex $xlist [expr $j+1]]) / 2.0]
	    lappend xmlist $xm
	    lappend plist [trape_poisson $i $xm]
	 }
	 
	 # Search most frequent value ----------
	 set plist_sorted [lsort -real $plist]
	 if { [lindex $plist_sorted 0] == [lindex $plist_sorted end] } {
	    lappend pois [lindex $plist_sorted 0]
	 } else {
	    # make value distribution
	    array unset dist
	    foreach p $plist_sorted {
	       if { [lsearch -exact [array name dist] $p] < 0 } {
		  set dist($p) 1
	       } else {
		  incr dist($p)
	       }
	    }
	    # which is most frequent value?
	    set max -1
	    set max_val -1
	    foreach {val n} [array get dist] {
	       if {$n > $max} {
		  set max $n
		  set max_val $val
	       }
	    }
	    lappend pois $max_val
	    # search exceptional values
	    for {set j 0} {$j < [llength $plist]} {incr j} {
	       set val [lindex $plist $j]
	       if {$max_val != $val} {
		  lappend poisl $i
		  lappend poisb [expr $j + 1]
		  lappend poisbl $val
	       }
	    }
	 }
      }
      # return
      return [list $pois $poisl $poisb $poisbl]
   }
   # --------------------------------------------------------------------
   # trape_poisson ------------------------------------------------------
   #           Compute poisson's ratio of specified trapezoid
   # --------------------------------------------------------------------
   # [arguments]
   # l   : layer
   # x   : x-coordinates.
   # --------------------------------------------------------------------
   proc trape_poisson {l x} {
      # Global variables --------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      set j 1
      while {1} {
	 set x1 [lindex $gbl(vx,$l) [expr $j - 1]]
	 set x2 [lindex $gbl(vx,$l) $j]
	 if {$x1 <= $x && $x2 >= $x && [expr $x1 - $x2] != 0.0} {
	    break;
	 }
	 incr j
	 if {$j >= [llength $gbl(vx,$l)]} {
	    bar_message "ERROR: I'm afraid something wrong with the model."
	    return
	 }
      }
      set p1 [lindex $gbl(poi,$l,top) [expr $j-1]]
      set p2 [lindex $gbl(poi,$l,top) $j]
      if {$p1 == $p2} { 
	 return $p1 
      } else {
	 return [expr  ($p1 + $p2)/2.0]
      }
   }

   # --------------------------------------------------------------------
   # make_sin -----------------------------------------------------------
   #           Generate "s.in"
   # --------------------------------------------------------------------
   proc make_sin {} {
      # Global variables --------------------------------------------------
      global gbl

      # Main --------------------------------------------------
      # Open output file
      if [catch {set fp [open "s.in" w]}] {
	 bar_message "ERROR: Can't make s.in file !!"
	 return -1
      }

      # Output parameters --------------------
      # plot parameter (Synthetic Section)
      puts $fp "  &pltpar"
      foreach i {iplot iconv iwavlt ipol iscale ishade ifill vred sps scalef\
		     rcor xnorm clip nptsw nskip inmo vrms iamp idump amp nsmth} {
	 if { [string length $gbl(zelt,section,$i)] > 0} {
	    puts $fp "\t $i=$gbl(zelt,section,$i)"
	 }
      }
      # plot parameter (plot)
      foreach i {xwndow ywndow} {
	 if { [string length $gbl(zelt,plot,$i)] > 0} {
	    puts $fp "\t $i=$gbl(zelt,plot,$i)"
	 }
      }
      puts $fp "  &end"
      # axis parameter --------------------
      puts $fp "  &axepar"
      foreach i {xmax xmin tmax tmin xmm tmm albht itrev orig} {
	 if {[string length $gbl(zelt,section,$i)] > 0} {
	    puts $fp "\t $i=$gbl(zelt,section,$i)"
	 }
      }
      puts $fp "  &end"    

      # Close
      close $fp
      return 0
   }

   # --------------------------------------------------------------------
   # Convert tx.out into macray
   # --------------------------------------------------------------------
   proc txout2macray {} {
      # Global variables --------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      bar_message "Making synthetic traveltime file...."
      
      # Open txout ------------------------------
      set fn "./tx.out"
      if { ![file readable $fn] } {
	 tk_messageBox -message "Cannot find \"$fn\" file." -icon info -type ok
	 return
      }
      set fp [open "tx.out"]
      
      # determine shot range by source data
      for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
	 if {$gbl(src,$i,onoff)} {
	    set shot_x $gbl(src,$i,x)
	    set shot_z $gbl(src,$i,z)
	 }
      }
      # Make output file
      set out_fp [open $gbl(file,calculated) "w"]
      puts $out_fp "$shot_x $shot_z 0.0 1.0"
      set n 1
      while { [gets $fp line] != -1 } {
	 set dist [lindex $line 0]
	 set time [lindex $line 1]
	 set accuracy [lindex $line 2]
	 set type [lindex $line 3]
	 if { $dist == 0.0 && $time == 0.0 && $accuracy == 0.0 && $type == 1 } {
	    break
	 }
	 if {$type <= 0} {
	    continue
	 }
	 puts $out_fp [format " %4d%10.3f%10.3f%10.3f%10d"\
			   $n [expr $dist - $shot_x] $time $accuracy $type]
	 incr n
      }
      close $out_fp
      close $fp
   }
   
   # --------------------------------------------------------------------
   # plotting rays
   # --------------------------------------------------------------------
   proc plot_rays {} {
      # Global variables
      global sys gbl
      
      # Main --------------------------------------------------
      RAYPATH::clear

      # Check "rp.out" file
      if ![file exists "rp.out"] { return }

      # color
      RAYPATH::define_color

      # Read "rp.out" and plot ------------------------------
      set fp [open "rp.out" "r"]
      while {1} {
	 if { [gets $fp line] == -1} { break }
	 set src [lindex $line 0]  ; # source (OBS) number
	 set type [lindex $line 1] ; # ray type
	 gets $fp n                ; # number of points
	 # read points
	 set coord {}
	 for {set i 0} {$i < $n} {incr i} {
	    lappend coord [gets $fp]
	 }
	 # Plot
	 RAYPATH::plot1 $type $src $coord
      }
      # lower raypaths
      RAYPATH::lower
      close $fp
   }


}
