namespace eval RAYINVR {
   namespace export *

   # --------------------------------------------------------------------
   # parameters window for RAYINVR (by C. A. Zelt)
   # --------------------------------------------------------------------
   proc parameter_window {} {
      # Global variables --------------------------------------------------
      global sys gbl
      
      # Main --------------------------------------------------
      set w .zelt_window
      
      # check backend program
      if { $gbl(option,backend) != "rayinvr" } {
	 tk_messageBox -icon warning -type ok\
	     -message "Now the backend program is the \"WaveFrontMethod\".  If you want to use \"RAYINVR\" for a backend program, change the backend program before ray-trace."
      }

      if { [winfo exists $w] } {
	 wm deiconify $w
	 raise $w
	 return
      }
      toplevel $w
      wm title $w "Parameter Window"
      
      # Title frame ------------------------------
      title_frame $w.title "Parameters for RAYINVR"
      
      # tabset frame ------------------------------
      set ts [tabset $w.tabset -relief flat]
      $ts conf -borderwidth 1 -highlightthickness 0 -side top
      foreach i {"Plot" "Axis" "Trace" "Amplitude"\
		     "Section" "BGimage"} {
	 $ts insert end $i -text $i -background gray70 -anchor nw
      }
      
      # plot parameter tab ------------------------------
      plot_parameter $ts
      axis_parameter $ts
      trace_parameter $ts
      amp_parameter $ts
      section_parameter $ts
      bgimage_parameter $ts

      # Button frame ------------------------------
      set bf [button_frame $w $w.button_frame]
      
      # Packing ------------------------------
      pack $w.title $ts $bf -side top -fill x
   }

   # --------------------------------------------------------------------
   # plot parameter tab
   # --------------------------------------------------------------------
   proc plot_parameter {ts} {
      # Global ------------------------------
      global gbl
      
      # Main ------------------------------
      set wf [frame $ts.plot]
      $ts tab conf "Plot" -window $wf -fill x

      set ge [labelframe $wf.general -text "General"]
      make_menu_com $ge iroute "Plot to" gbl(zelt,plot,iroute) "" \
	  "Zelt window" 1 "Inside modeling" 5 "PostScript" 2 "ColorPS" 4
      make_menu_com $ge itx "T-X plot" gbl(plot,tx) "" \
	  "Don't plot" 0 "With line" 1 "With symbols" 2
      make_menu_com $ge iray "Display rays" gbl(zelt,plot,iray) ""\
	  "All rays" 1 "Only reach surface" 2
      make_menu_com $ge ircol "Ray color" gbl(plot,ray_color) ""\
	  "None(black)" 0 "Ray group" 3 "Shot(OBS)" 2
      make_menu_com $ge itcol "Observed color" gbl(plot,pick_color) ""\
	  "None(black)" 0 "Pick code" 1 "Shot(OBS)" 2
      make_entry $ge observe_thinout "Plot observed picks every" \
	  gbl(plot,observe_thinout) "picks"
      make_entry $ge vred "Reduction velocity" gbl(plot,vred) "km/sec"
      make_entry $ge pacc "Synthetic traveltime uncertainty" gbl(plot,pick_acc)\
	  "sec. (Default value)"

      set ri [labelframe $wf.rayinvr -text "RAYINVR (external) window"]
      foreach {i j k}\
	  {xwndow "Size of Window(width)" mm\
	       ywndow "Size of Window(height)" mm\
	       symht "Height of the symbols" mm\
	       nrskip "Plot every" "ray traced"\
	       npskip "Plot every" "point of each ray"\
	       colour "Color code array" "(specify like '2,3,4')"} {
		  make_entry $ri $i $j gbl(zelt,plot,$i) $k
	       }
      make_menu_com $ri isep "Plot model and rays" gbl(zelt,plot,isep) ""\
	  "Same frame" 0 "Separate frame" 1 "Separate each shot" 2
      foreach {i j}\
	  {"Plot model boundary" imod "Plot velocity grid" ibnd\
	       "Plot P-wave velocity" ivel "Plot a symbol at each point" idot\
	       "Show Observed Data" idata} {
		  make_checkbutton $ri $i $j gbl(zelt,plot,$j)
	       }
      pack $ge $ri -fill x -side top -pady 5 -padx 10 -ipady 5
   }

   # --------------------------------------------------------------------
   # AXIS parameter set tab
   # --------------------------------------------------------------------
   proc axis_parameter {ts} {
      # Global variables --------------------------------------------------
      global gbl wk

      # Main --------------------------------------------------
      set w [frame $ts.axis]
      $ts tab conf "Axis" -window $w -fill x

      # initialize
      set wk(xmin) $gbl(xmin)
      set wk(xmax) $gbl(xmax)
      set wk(zmin) $gbl(zmin)
      set wk(zmax) $gbl(zmax)

      # parameter frame ------------------------------
      set wf [frame $w.parameter]

      set ms [labelframe $wf.modelsize -text "Model size"]
      foreach {i j} {xmin "X min"    xmax "X max" \
			 zmin "Depth min"  zmax "Depth max" } {
	 make_entry $ms $i $j wk($i) "Km"
      }
      make_command_button $ms update "" "update" update_model_size
      
      set ta [labelframe $wf.txgraph -text "T-X graph"]
      make_entry $ta tmin "Time min" gbl(tmin) sec
      make_entry $ta tmax "Time max" gbl(tmax) sec
      make_entry $ta vred "Reduction velocity" gbl(plot,vred) "km/sec."
      make_menu_com $ta itrev "Time axis direction" gbl(plot,taxis_dir) ""\
	  Upward 0 Downward 1
      
      set ps [labelframe $wf.axisplot -text "axis plot (for RAYINVR external window)"]
      foreach {i j k}\
	  {xmm "X-length" mm zmm "Z-length" mm\
	       tmm "T-length" mm albht "Height of label font" mm\
	       orig "Bottom and Left margin of plot" mm} {
		  make_entry $ps $i $j gbl(zelt,axis,$i) $k
	       }
      make_checkbutton $ps "Show axis label" iaxlab gbl(zelt,axis,iaxlab)

      pack $ms $ta $ps -fill x -side top -pady 5 -padx 10 -ipady 5
      
      # packing ------------------------------
      pack $wf -side top -fill x -expand 1
   }

   # --------------------------------------------------------------------
   # RayTrace parameter set tab
   # --------------------------------------------------------------------
   proc trace_parameter {ts} {
      # Global variables --------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------

      # Initialize parameter ------------------------------
      set gbl(zelt,trace,imodf) 1
      
      # Parameter frame ------------------------------
      set w [frame $ts.trace]
      $ts tab conf "Trace" -window $w -fill x
      
      set wf [frame $w.parameters]
      make_checkbutton $wf "Layer boundary smoothing" ibsmth\
	  gbl(zelt,trace,ibsmth)
      make_menu_com $wf ishot "Ray direction" gbl(zelt,trace,ishot) ""\
	  "Right & Left" 2 "Right only" 1 "Left only" -1 "No rays" 0
      make_menu_com $wf ifast "Calculation" gbl(zelt,trace,ifast) ""\
	  "Fast" 1 "Slow(but accurate)" 0
      make_checkbutton $wf "Two-point raytracing" i2pt gbl(trace,2pt)
      foreach {i j k} {
	 nray "Number of rays" "(per a ray group)"\
	     nsmax "Max number of rays" "(while serching take-off angle, )"\
	     stol "Ray search tolerance" "Km. (Distinguishable ray end-points)"\
	     step "Ray step length factor" "Km. (step=factor*v/(|vx|+|vz|))"} {
	 make_entry $wf $i $j gbl(zelt,trace,$i) $k
      }
      make_entry $wf ivray "Two-point Ray-group list"\
	  gbl(zelt,invr,ivray) "(pick code array)"
      make_entry $wf dx2pt "Receiver(Shot) spacing" gbl(trace,dx2pt)\
	  "Km. (Only for auto 2pt-raytracing)"

      # Filename frame for computation results.
      frame $w.calc -relief flat
      filename_hscroll_frame $w.calc txout "Calculated T-X OUTPUT File"\
	  gbl(file,calculated) 40 save  {
	     {"MacRay files"	   *.macray}
	     {"Pick files"	   *.pick}
	     {"dat files"            *.dat}
	     {"All files"		*}
	  }
      
      # packing
      pack $wf $w.calc -side top -fill x -expand 1
   }

   # --------------------------------------------------------------------
   # Amp parameter set tab
   # --------------------------------------------------------------------
   proc amp_parameter_old {ts} {
      # Global variables --------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      set wf [frame $ts.amp]
      $ts tab conf "Amplitude" -window $wf -fill x
      make_menu_com $wf iamp "Amp-calculation mode" gbl(zelt,amp,iamp) ""\
	  "No calculation" 0 "Calculate" 1 "plot Amp-Dist curve" 2
      make_menu_com $wf amprev "Amp. Calcurated from" gbl(zelt,amp,amprev) ""\
	  "Source" 0 "Receiver(OBS)" 1
      make_menu_com $wf icomp "Seismometer Type" gbl(zelt,amp,icomp) ""\
	  "Vertical" 1 "Horizontal" 0
      foreach {i j} {"Measure Rec. loc. from shot" icmp \
			 "Calc Surface Conversion" surcon} {
	 make_checkbutton $wf $i $j gbl(zelt,amp,$j)
      }
      foreach {i j k} {spamp "ratio of S to P displacement" "(at the source)"} {
	 make_entry $wf $i $j gbl(zelt,amp,$i) $k
      }
   }

   # --------------------------------------------------------------------
   # Amp parameter set tab
   # --------------------------------------------------------------------
   proc amp_parameter {ts} {
      # Global variables --------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      set wf [frame $ts.amp]
      $ts tab conf "Amplitude" -window $wf -fill x

      # for amplitude
      set wa [labelframe $wf.amp -text "Amplitude calculation"]
      make_menu_com $wa iamp "Amp-calculation mode" gbl(zelt,amp,iamp) ""\
	  "No calculation" 0 "Calculate" 1 "plot Amp-Dist curve" 2
      make_menu_com $wa amprev "Amp. Calcurated from" gbl(zelt,amp,amprev) ""\
	  "Source" 0 "Receiver(OBS)" 1
      make_menu_com $wa icomp "Seismometer Type" gbl(zelt,amp,icomp) ""\
	  "Vertical" 1 "Horizontal" 0
      foreach {i j} {"Calc amp at traveltime calc. point" iamp \
			 "Measure Rec. loc. from shot" icmp \
			 "Calc Surface Conversion" surcon} {
	 make_checkbutton $wa $i $j gbl(zelt,amp,$j)
      }
      foreach {i j k} {spamp "ratio of S to P displacement" "(at the source)"} {
	 make_entry $wa $i $j gbl(zelt,amp,$i) $k
      }

      # for section
      set wx [labelframe $wf.sect -text "Section"]
      make_menu_com $wx isect "Make Synthetic Section 'sect.out'" \
	  gbl(zelt,section,isect) ""\
	  "No" 0 "Automatic" 2 "Use Specified values" 3\
	  "Use 'rec.in'" 4
      foreach {i j} {"Plot also to 'syn.out' binary file" idump\
			 "Convolve Impulse Responce" iconv} {
	 make_checkbutton $wx $i $j gbl(zelt,section,$j)
      }
      make_entry $wx sps "Digitization Rate" gbl(zelt,section,sps) "Hz"
      make_checkbutton $wx "RMS-velocity NMO correction" inmo\
	  gbl(zelt,section,inmo)
      make_entry $wx vrms "RMS-velocity (for NMO correction)" gbl(zelt,section,vrms)\
	  "Km/sec"
      
      # Sources
      set ws [labelframe $wf.src -text "Source time function"]
      make_menu_com $ws iwavlt "Source wavelet" gbl(zelt,section,iwavlt) ""\
	  "Ricker Wavelet" 0 "Sine wave" 1 "Defined in file" 2
      make_entry $ws nsmth "# of application of Smoothing-Filter"\
	  gbl(zelt,section,nsmth) "(to Impulse Response)"
      make_entry $ws nptsw "# of Points of sine wave wavelet"\
	  gbl(zelt,section,nptsw) ""
      make_checkbutton $ws "Reverse polarity of wavelet" ipol\
	  gbl(zelt,section,ipol)
      
      # Filename of Source time function
      filename_hscroll_frame $ws wavelet "Wavelet File" \
	  gbl(file,wavelet) 20 load \
	  {
	     {"Wavelet files"    *.wav}
	     {"dat files"        *.dat}
	     {"All files"	    *}
	  }

      pack $wa $wx $ws -fill x -side top -pady 5 -padx 10 -ipady 2
      
   }

   # --------------------------------------------------------------------
   # Seismic Section parameter set tab
   # --------------------------------------------------------------------
   proc section_parameter {ts} {
      # Global variables --------------------------------------------------
      global gbl
      
      # Main --------------------------------------------------
      set wf [frame $ts.section]
      $ts tab conf "Section" -window $wf -fill x

      # amplitude
      set wa [labelframe $wf.amp -text "Amplitude relation"]
      make_menu_com $wa isect "Make Synthetic Section 'sect.out'" \
	  gbl(zelt,section,isect) ""\
	  "No" 0 "Automatic" 2 "Use Specified values" 3\
	  "Use 'rec.in'" 4
      make_menu_com $wa iscale "Amplitude Scaling with distance"\
	  gbl(zelt,section,iscale) ""\
	  "None" 0  "Auto correction" 2 "by Specified values" 1
      make_entry $wa amp "Maximum amplitude of traces" gbl(zelt,section,amp) \
	  "(for 'Auto' and 'None' mode)"
      foreach {i j k}\
	  {scalef "Global Scale factor" "(for 'Specified' mode)"\
	       clip "Clipping Amplitude(0 means 'No clip')" "(for 'Specified' mode)"\
	       rcor "Power of distant correction factor" ""\
	       xnorm "Normalization factor of Src-Rec dist." "Km"\
	    } {
	       make_entry $wa $i $j gbl(zelt,section,$i) $k
	    }

      # plot
      set wp [labelframe $wf.plot -text "Plot relation"]
      make_menu_com $wp iplot "Plot to" gbl(zelt,section,iplot) ""\
	  "X-Window" 1 "PS-file 'p.out'" 0 "Both X and PS-file" 2

      foreach {i j k} {vred "Reduction Velocity" "Km/sec"} {
	 make_entry $wp $i $j gbl(zelt,section,$i) $k
      }
      make_menu_com $wp itrev "Time axis direction" gbl(zelt,section,itrev) ""\
	  Upward 0 Downward 1
      foreach {i j k}\
	  {xmin "X min" Km xmax "X max" Km xincs "Trace spacing" Km\
	       tmin "Time min" sec tmax "Time max" sec\
	       xmm "X-length" mm tmm "T-length" mm\
	       albht "Height of Axis label font" mm\
	       orig "Bottom and Left margin of plot" mm} {
		  make_entry $wp $i $j gbl(zelt,section,$i) $k
	       }
      make_entry $wp nskip "Plot every" gbl(zelt,section,nskip) "seismogram"
      make_checkbutton $wp "Trace Shading" ishade gbl(zelt,section,ishade)
      make_menu_com $wp ifill "Shade polarity" gbl(zelt,section,ifill) ""\
	  "Positive" 1 "Negative" -1
      pack $wa $wp -fill x -side top -pady 5 -padx 10 -ipady 2
   }
}