# --------------------------------------------------------------------
# raytrace
# --------------------------------------------------------------------
proc raytrace {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------

   # If there is no velocity model, return.
   if {$gbl(n_interface) < 1} {
      bar_message "ERROR: No model defined."
      return
   }

   switch -- $gbl(option,backend) {
      rayinvr {
	 RAYINVR::raytracing
      }
      hamen {
	 HAMEN::raytracing
      }
      default {
	 tk_messageBox -type ok -icon error \
	     -message "Specify backend program before exectute ray-tracer."

      }
   }
}

# --------------------------------------------------------------------
# calculate amplitude 
# --------------------------------------------------------------------
proc calc_amplitude {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------

   # If there is no velocity model, return.
   if {$gbl(n_interface) < 1} {
      bar_message "ERROR: No model defined."
      return
   }

   switch -- $gbl(option,backend) {
      rayinvr {
	 RAYINVR::amplitude
      }
      hamen {
	 bar_message "Cannot compute amplitude using wavefront method."
      }
   }
}

# --------------------------------------------------------------------
# exec_external_command ----------------------------------------------
#            Execute external command
# --------------------------------------------------------------------
proc exec_external_command {com} {
   global gbl

   set stderrfn [format "./modeling.tmp.%d.stderr" [pid]]
   if { [string first ">" $com] > 0 } {
      eval exec $com 2> $stderrfn
   } else {
      if $gbl(option,exec_condition) {
	 eval exec $com >& $stderrfn
      } else {
	 eval exec $com > /dev/null 2> $stderrfn
      }
   }
   cat_file $stderrfn
   file delete -force $stderrfn
}