# --------------------------------------------------------------------
# make_active_source_list --------------------------------------------
#           Make active source list
# --------------------------------------------------------------------
proc make_active_source_list {} {
   # Global variables --------------------------------------------------
   global gbl

   # Main --------------------------------------------------
   set l {}
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      if { $gbl(src,$i,x) != "" && $gbl(src,$i,z) != "" } {
	 lappend l $i
      }
   }
   return $l
}

# --------------------------------------------------------------------
# check_source_range -------------------------------------------------
#           Check source range and turn off the outside source
# --------------------------------------------------------------------
proc check_source_range {} {
   # Global variables --------------------------------------------------
   global gbl

   # Main --------------------------------------------------
   set sl [make_active_source_list]
   foreach i $sl {
      if {$gbl(src,$i,onoff)} {
	 if {$gbl(src,$i,x) < $gbl(xmin) \
		 || $gbl(src,$i,x) > $gbl(xmax) } {
	    # turn off the outside source
	    set gbl(src,$i,onoff) 0
	 }
      }
   }
}

# --------------------------------------------------------------------
# write_source_param -------------------------------------------------
#           Write source related information to the file
# --------------------------------------------------------------------
proc write_source_param {} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # check number of sources ----------
   set al [make_active_source_list]
   if { [llength $al] < 1} {
      # no source
      tk_messageBox -type ok -icon warning\
	  -message "There is no source to write out."
      return
   }
   # output filename ----------
   set fn [tk_getSaveFile -title "Write source paramters"]
   if {$fn == ""} {
      # canceled
      return
   }
   # write source parameters ----------
   set fp [open $fn w]
   foreach i $al {
      # name
      if { [regexp {\w} $gbl(src,$i,name)] } {
	 set name $gbl(src,$i,name)
      } else {
	 set name [format "noname(%d)" $i]
      }
      # pick file
      if { [regexp {\w} $gbl(src,$i,pick)] } {
	 set pick $gbl(src,$i,pick)
      } else {
	 set pick [format "" $i]
      }
      # output
      puts $fp [format "%s %s %s %s" $name\
		    $gbl(src,$i,x) $gbl(src,$i,z) $pick]
   }
   close $fp
   # Done !
   bar_message [format "Done. Wrote %s source(s) parameters to a file."\
		    [llength $al]]
}

# --------------------------------------------------------------------
# read_source_param --------------------------------------------------
#           Read source related information from the file
# --------------------------------------------------------------------
proc read_source_param {} {
   # Global variables --------------------------------------------------
   global gbl wk
   
   # Main --------------------------------------------------
   # check number of sources ----------
   set al [make_active_source_list]
   if { [llength $al] >= 1} {
      # you have source
      set ans [tk_messageBox -type yesno -icon question\
		   -message "You've already have some sources.  If you read source information from a file, all the current source parameters will be lost.  Are you sure to load source parameters from a file?"]
      if { $ans != "yes" } {
	 bar_message "load source parameters canceled."
	 return
      }
   }
   # Clear all the source parameters from the memory
   for {set i 1} {$i <= $gbl(src,n_source)} {incr i} {
      foreach {j k} {x "" z "" name "" onoff 0 pick ""} {
	 set gbl(src,$i,$j) $k
      }
   }
   # dismiss source window
   catch { destroy $wk(w,src_param) }

   # filename ----------
   set fn [tk_getOpenFile -title "Read source paramters"]
   
   # Read it! ----------
   set fp [open $fn]
   set gbl(src,n_source) 0
   set i 0
   while {1} {
      # read a line
      if { [gets $fp dat] == -1 } {
	 break
      }
      # comment ?
      if { [string match "\#*" $dat] } {
	 continue
      }
      # check data format
      if { [llength $dat] == 1 } {
	 incr i
	 set tmp [read_stn_x_z_from_pick $dat]
	 set gbl(src,$i,name) [lrange $tmp 2 end]
	 set gbl(src,$i,x) [lindex $tmp 0]
	 set gbl(src,$i,z) [lindex $tmp 1]
	 set gbl(src,$i,pick) $dat
      } elseif { [llength $dat] == 4 } {
	 incr i
	 set gbl(src,$i,name) [lindex $dat 0]
	 set gbl(src,$i,x) [lindex $dat 1]
	 set gbl(src,$i,z) [lindex $dat 2]
	 set gbl(src,$i,pick) [lindex $dat 3]
      }
   }
   close $fp
   bar_message [format "%d sources are read from file." $i]
   if {$i > 5} {
      set gbl(src,n_source) $i
   } else {
      set gbl(src,n_source) 5
   }
   # Pop up or raise source parameter window ----------
   source_parameter
}
# --------------------------------------------------------------------
# read_stn_x_z_from_pick ---------------------------------------------
#            Read station name, x and z from a pick file
# --------------------------------------------------------------------
proc read_stn_x_z_from_pick {fn} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   if {$fn == ""} { return "" }
   # read parameters from the file
   set ftype [discriminate_pickfile $fn] 
   if { $ftype == "macray" } {
      # macray format
      set fp [open $fn]
      set stn [file tail [file rootname $fn]]
      if { [gets $fp line] > 0} {
	 set x [lindex $line 0]
	 set z [lindex $line 1]
      }
      close $fp
   } elseif { $ftype == "zelt" } {
      # zelt format
      set stn [file tail [file rootname $fn]]
      if { [gets $fp line] > 0} {
	 set x [lindex $line 0]
	 set z ""
      }
   } else {
      return ""
   }
   return "$x $z $stn"
}