# --------------------------------------------------------------------
# undo ---------------------------------------------------------------
#           Undo
# --------------------------------------------------------------------
proc undo {} {
   # Global variables --------------------------------------------------
   global gbl sys wk
   
   # Main --------------------------------------------------
   if {$wk(undo,n) < 1} {
      bar_message "No further Undo!!"
      return
   }
   # Undo (one step) ------------------------------
   incr wk(undo,n) -1
   set i $wk(undo,interface,$wk(undo,n))
   set index $wk(undo,index,$wk(undo,n))
   # Highlight the related interface.
   set wk(selected) $i
   # undo ----------
   switch $wk(undo,mode,$wk(undo,n)) {
      point.create {
	 # Delete a newly inserted point ----------
	 set gbl(x,$i) [lreplace $gbl(x,$i) $index $index]
	 set gbl(z,$i) [lreplace $gbl(z,$i) $index $index]
	 set gbl(z,SD,$i) [lreplace $gbl(z,SD,$i) $index $index]
	 incr gbl(n_node,$i) -1
      }
      point.delete {
	 # Recover a deleted point ----------
	 set gbl(x,$i) [linsert $gbl(x,$i) $index $wk(undo,x,$wk(undo,n))]
	 set gbl(z,$i) [linsert $gbl(z,$i) $index $wk(undo,z,$wk(undo,n))]
	 set gbl(z,SD,$i) [linsert $gbl(z,SD,$i) $index\
			       $wk(undo,z,SD,$wk(undo,n))]
	 incr gbl(n_node,$i)
	 draw_model
	 flash_point undo $i $index 4
      }
      point.move {
	 # Relocated a moved point ----------
	 set gbl(x,$i) [lreplace $gbl(x,$i) $index $index\
			    $wk(undo,x,$wk(undo,n))]
	 set gbl(z,$i) [lreplace $gbl(z,$i) $index $index\
			    $wk(undo,z,$wk(undo,n))]
	  flash_point undo $i $index 4
      }
      point.all_sd {
	 # Recover all the SD value in a interface ----------
	 set gbl(z,SD,$i) $wk(undo,z,SD,$wk(undo,n))
	 $sys(graph) element activate interface($i)
	 after 1000 $sys(graph) element deactivate interface($i)
	 after 2000 $sys(graph) element deactivate interface($i)
      }
      vgrid.edit {
	 # Recover a velocity value ----------
	 foreach {m n} {vx x   vtop vtop   vbot vbot} {
	    set gbl($m,$i) [lreplace $gbl($m,$i) $index $index\
				$wk(undo,$n,$wk(undo,n))]
	 }
	 foreach {m} {vtop vbot} {
	    set gbl($m,SD,$i) [lreplace $gbl($m,SD,$i) $index $index\
				   $wk(undo,$m,SD,$wk(undo,n))]
	 }
	 # Recover poisson's ratio ----------
	 if {$gbl(ERI,poisson) <= 1} { set j 0 } else { set j $index}
	 foreach {m n} {poi top   poi bot   poi,SD top   poi,SD bot} {
	    set gbl($m,$i,$n) [lreplace $gbl($m,$i,$n) $j $j\
				     $wk(undo,$m,$n,$wk(undo,n))]
	 }
	 # Highlight the velocity grid
	 highlight_grid $i $index
	 draw_grid_layer $i
      }
      vgrid.all_velocity {
	 # Recover all the velocity grid in the same layer ----------
	 set gbl(vtop,$i) $wk(undo,vtop,$wk(undo,n))
	 set gbl(vbot,$i) $wk(undo,vbot,$wk(undo,n))
	 set gbl(vtop,SD,$i) $wk(undo,vtop,SD,$wk(undo,n))
	 set gbl(vbot,SD,$i) $wk(undo,vbot,SD,$wk(undo,n))
	 set gbl(poi,$i,top) $wk(undo,poi,top,$wk(undo,n))
	 set gbl(poi,$i,bot) $wk(undo,poi,bot,$wk(undo,n))
	 set gbl(poi,SD,$i,top) $wk(undo,poi,SD,top,$wk(undo,n))
	 set gbl(poi,SD,$i,bot) $wk(undo,poi,SD,bot,$wk(undo,n))
	 for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
	    highlight_grid $i $j
	 }
	 draw_grid_layer $i
      }
      vgrid.all_layer {
	 # Recover all the velocity grid in the same layer ----------
	 set gbl(vx,$i) $wk(undo,x,$wk(undo,n))
	 set gbl(vtop,$i) $wk(undo,vtop,$wk(undo,n))
	 set gbl(vbot,$i) $wk(undo,vbot,$wk(undo,n))
	 set gbl(vtop,SD,$i) $wk(undo,vtop,SD,$wk(undo,n))
	 set gbl(vbot,SD,$i) $wk(undo,vbot,SD,$wk(undo,n))
	 set gbl(poi,$i,top) $wk(undo,poi,top,$wk(undo,n))
	 set gbl(poi,$i,bot) $wk(undo,poi,bot,$wk(undo,n))
	 set gbl(poi,SD,$i,top) $wk(undo,poi,SD,top,$wk(undo,n))
	 set gbl(poi,SD,$i,bot) $wk(undo,poi,SD,bot,$wk(undo,n))
	 set gbl(n_vnode,$i) [llength $gbl(vx,$i)]
	 for {set j 0} {$j < $gbl(n_vnode,$i)} {incr j} {
	    highlight_grid $i $j
	 }
	 draw_grid_layer $i
      }
      vgrid.delete {
	 # Recover the deleted velocity grid ----------
	 foreach {m n} {vx x   vtop vtop   vbot vbot} {
	    set gbl($m,$i) [linsert $gbl($m,$i) $index\
				$wk(undo,$n,$wk(undo,n))]
	 }
	 foreach {j} {vtop vbot} {
	    set gbl($j,SD,$i) [linsert $gbl($j,SD,$i) $index \
				   $wk(undo,$j,SD,$wk(undo,n))]
	 }
	 # poisson's ratio
	 if {$gbl(ERI,poisson) >= 2} {
	    foreach {j} {top bot} {
	       set gbl(poi,$i,$j) [linsert $gbl(poi,$i,$j) $index \
				       $wk(undo,poi,$j,$wk(undo,n))]
	       set gbl(poi,SD,$i,$j) [linsert $gbl(poi,SD,$i,$j) $index \
					  $wk(undo,poi,SD,$j,$wk(undo,n))]
	    }
	 }
	 incr gbl(n_vnode,$i)
	 highlight_grid $i $index
	 draw_grid_layer $i
      }
      vgrid.create {
	 # Delete a newly inserted velocity grid ----------
	 foreach j {vx vtop vbot} {
	    set gbl($j,$i)\
		[lreplace $gbl($j,$i) $index $index]
	 }
	 foreach j {vtop vbot} {
	    set gbl($j,SD,$i)\
		[lreplace $gbl($j,SD,$i) $index $index]
	 }
	 if { $gbl(ERI,poisson) >= 2 } {
	    foreach j {top bot} {
	       set gbl(poi,$i,$j) [lreplace $gbl(poi,$i,$j) $index $index]
	       set gbl(poi,SD,$i,$j) [lreplace $gbl(poi,SD,$i,$j) $index $index]
	    }
	 }
	 incr gbl(n_vnode,$i) -1
	 draw_grid_layer $i
      }
      interface.move {
	 # Relocated the moved interface
	 set gbl(z,$i) $wk(undo,z,$wk(undo,n))
      }
      interface.delete {
	 # Recover deleted interface
	 shift_interface $i down
	 set gbl(x,$i) $wk(undo,x,$wk(undo,n))
	 set gbl(z,$i) $wk(undo,z,$wk(undo,n))
	 set gbl(z,SD,$i) $wk(undo,z,SD,$wk(undo,n))
	 set gbl(vbot,[expr $i - 1]) $wk(undo,vtop,$wk(undo,n))
	 set gbl(vbot,SD,[expr $i - 1]) $wk(undo,vtop,SD,$wk(undo,n))
	 set gbl(vx,$i) [lindex $wk(undo,vbot,$wk(undo,n)) 0]
	 set gbl(vtop,$i) [lindex $wk(undo,vbot,$wk(undo,n)) 1]
	 set gbl(vbot,$i) [lindex $wk(undo,vbot,$wk(undo,n)) 2]
	 set gbl(vtop,SD,$i) [lindex $wk(undo,vbot,SD,$wk(undo,n)) 0]
	 set gbl(vbot,SD,$i) [lindex $wk(undo,vbot,SD,$wk(undo,n)) 1]
	 
	 set gbl(poi,[expr $i-1],bot) $wk(undo,poi,top,$wk(undo,n))
	 set gbl(poi,SD,[expr $i-1],bot) $wk(undo,poi,SD,top,$wk(undo,n))
	 set gbl(poi,$i,top) [lindex $wk(undo,poi,bot,$wk(undo,n)) 0]
	 set gbl(poi,$i,bot) [lindex $wk(undo,poi,bot,$wk(undo,n)) 1]
	 set gbl(poi,SD,$i,top) [lindex $wk(undo,poi,SD,bot,$wk(undo,n)) 0]
	 set gbl(poi,SD,$i,bot) [lindex $wk(undo,poi,SD,bot,$wk(undo,n)) 1]
	 set gbl(n_vnode,$i) [llength $gbl(vx,$i)]
	 set gbl(n_node,$i) [llength $gbl(x,$i)]
	 incr gbl(n_interface)
      }
      interface.create {
	 # Delete inserted interface
	 # make velocity
	 set upper [expr $i - 1]
	 set gbl(vbot,$upper) $gbl(vbot,$i)
	 set gbl(vbot,SD,$upper) $gbl(vbot,$i)
	 set gbl(poi,$upper,bot) $gbl(poi,$i,bot)
	 set gbl(poi,SD,$upper,bot) $gbl(poi,SD,$i,bot)
	 $sys(graph) element delete interface($gbl(n_interface))
	 # shift
	 shift_interface $i up
	 incr gbl(n_interface) -1
      }
      interface.pinchout {
	 set gbl(x,$i) $wk(undo,x,$wk(undo,n))
	 set gbl(z,$i) $wk(undo,z,$wk(undo,n))
	 set gbl(z,SD,$i) $wk(undo,z,SD,$wk(undo,n))
	 set gbl(n_node,$i) [llength $gbl(x,$i)]
      }
   }
   bar_message "Undo!! ($wk(undo,n) times left)"
   draw_model
}
# --------------------------------------------------------------------
# undo_check ---------------------------------------------------------
#           undo buffer check. Only "MAX_UNDO" times undo can be saved.
# --------------------------------------------------------------------
proc undo_check {} {
   # Global variables --------------------------------------------------
   global sys wk
   
   # Main --------------------------------------------------
   if {$wk(undo,n) > $sys(MAX_UNDO)} {
      for {set i 1} {$i <= $sys(MAX_UNDO)} {incr i} {
	 set j [expr $i - 1]
	 set wk(undo,x,$j) $wk(undo,x,$i) 
	 set wk(undo,z,$j) $wk(undo,z,$i)
	 set wk(undo,z,SD,$j] $wk(undo,z,SD,$i)
	 set wk(undo,vtop,$j) $wk(undo,vtop,$i)
	 set wk(undo,vbot,$j) $wk(undo,vbot,$i)
	 set wk(undo,vtop,SD,$j) $wk(undo,vtop,SD,$i)
	 set wk(undo,vbot,SD,$j) $wk(undo,vbot,SD,$i)
	 set wk(undo,interface,$j) $wk(undo,interface,$i)
	 set wk(undo,index,$j) $wk(undo,index,$i)
	 set wk(undo,mode,$j) $wk(undo,mode,$i) 
	 set wk(undo,poi,top,$j) $wk(undo,poi,top,$i)
	 set wk(undo,poi,bot,$j) $wk(undo,poi,bot,$i)
	 set wk(undo,poi,SD,top,$j) $wk(undo,poi,SD,top,$i)
	 set wk(undo,poi,SD,bot,$j) $wk(undo,poi,SD,bot,$i)
      }
      incr wk(undo,n) -1
   }
}