# --------------------------------------------------------------------
# write model data in v.in format file
# --------------------------------------------------------------------
proc write_vin {fn} {
   # Global variables --------------------------------------------------
   global gbl
   
   # Main --------------------------------------------------
   # Open output file
   if [catch {set fp [open $fn w]}] {
      bar_message "ERROR: Can't make v.in file !!"
      return -1
   }
   
   for {set i 1} {$i < $gbl(n_interface)} {incr i} {
      # i-th interface ----------
      put_vin_line $i $gbl(n_node,$i) 0 $gbl(x,$i) $gbl(z,$i) $fp
      # i-th layer (upper) ----------
      put_vin_line $i $gbl(n_vnode,$i) 0 $gbl(vx,$i) $gbl(vtop,$i) $fp
      # i-th layer (lower) ----------
      put_vin_line $i $gbl(n_vnode,$i) 0 $gbl(vx,$i) $gbl(vbot,$i) $fp
   }
   # Buttom interface ----------
   set i $gbl(n_interface)
   put_vin_line $i $gbl(n_node,$i) 1 $gbl(x,$i) $gbl(z,$i) $fp
   
   close $fp
   return 0
}
# --------------------------------------------------------------------
# 3 rows output procedure for "v.in"
# --------------------------------------------------------------------
#  i    : Interface number.
#  n    : Number of data.
#  last : Is this a last line? (1=>true, otherwise=>false)
#  l1   : List of X-coordinates.
#  l2   : List of Y-coordinates.
#  fp   : File point of output file.
# --------------------------------------------------------------------
proc put_vin_line {i n last l1 l2 fp} {
    # Global variables --------------------------------------------------
    global gbl

    # Main --------------------------------------------------
    set count 0
    while {$count < $n} {
	# Head part of each line
	if { [expr ($n - $count)] > 10 } {
	    set rest 1
	} else {
	    set rest 0
	}
	set line1 [format "%2d " $i]
	set line2 [format "%2d " $rest]
	set line3 "   "
	# Data part of each line
	for {set k 0} {$k < 10} {incr k} {
	    set line1 [format "%s%7.2f" $line1 [lindex $l1 $count]]
	    set line2 [format "%s%7.2f" $line2 [lindex $l2 $count]]
	    set line3 [format "%s%7d"   $line3 0]
	    incr count
	    if {$count >= $n} {
		# All written?
		set k 10
	    }
	}
	# Output to file.
	puts $fp $line1
	puts $fp $line2
	if {$last == 0 || $count < $n} {
            # If this is not the last, and have data
	    puts $fp $line3
        }
    }
}
