# --------------------------------------------------------------------
# make_entry ---------------------------------------------------------
#            Make entry frame to set parameter.
# --------------------------------------------------------------------
# [arguments]
#    w     : paraent widget name.
#    name  : Name of parameters (Identifier in this script)
#    label : Labels to display.
#    val   : Variable name in this script.
#    unit  : Unit.
# --------------------------------------------------------------------
proc make_entry {w name label val unit} {
    # Main --------------------------------------------------
    label $w.label($name) -text $label -anchor w
    entry $w.entry($name) -width 12 -textvariable $val -justify right
    label $w.unit($name) -text $unit -anchor w
    grid $w.label($name) $w.entry($name) $w.unit($name) -in $w -sticky news
}
# --------------------------------------------------------------------
# make_many_radio ----------------------------------------------------
#            Make many(more than 3) radio button widget.
# --------------------------------------------------------------------
# [arguments]
#    w     : Parent widget name
#    label : Labels to display
#    name  : Name of parameters (Identifier in this script)
#    val   : Variable name in this script
#   (Variable lengths args)
#    l1    : Label1
#    v1    : Value1
# --------------------------------------------------------------------
proc make_many_radio {w label name val args} {
    # Variable length arguments --------------------
    set i 0
    foreach {j k} $args {
	set l($i) $j
	set v($i) $k	
	incr i
    }
    label $w.label($name) -text $label -anchor e
    set winlist $w.label($name)
    for {set j 0} {$j < $i} {incr j} {
	radiobutton $w.$name\($j) -text $l($j) -anchor w\
		-variable $val -value $v($j) 
	lappend winlist $w.$name\($j)
    }
    eval {grid} $winlist -sticky news
}
# --------------------------------------------------------------------
# File selection dialog
# --------------------------------------------------------------------
proc file_selection {filename operation types} {
    # Global variables and "address" type arguments
    upvar $filename tmp_file
    global gbl

    # Main
    if { [string match "load" $operation] == 1} {
	set file [tk_getOpenFile -filetypes $types -initialfile $tmp_file]
    } else {
	set file [tk_getSaveFile -filetypes $types -initialfile $tmp_file]
    }
    if {$file != ""} {
	set $filename $file
    }
}

# --------------------------------------------------------------------
# file name entry with ref button (use pack)
# --------------------------------------------------------------------
# arguments:
#  w     : parent widget name
#  name  : nickname inside the script (identifier)
#  label : explanation to show
#  mode  : "save" or "load"
#  type  : file type list
# --------------------------------------------------------------------
proc filename_entry {w name var label mode type} {
   label $w.l($name) -text $label -anchor e
   entry $w.e($name) -width 65 -textva $var -ju left
   button $w.b($name) -text "Ref..."\
       -command "file_selection $var $mode {$type}"
   grid $w.l($name) $w.e($name) $w.b($name) -sticky news
}
# --------------------------------------------------------------------
# make_spinbox -------------------------------------------------------
#            SpinBox parameter frame
# --------------------------------------------------------------------
# [arguments]
#    w     : parent widget name
#    name  : id inside program (for grid manager)
#    label : explanation
#    var   : variable name
#    min   : minimum value
#    max   : maximum value
#    inc   : increment
#    unit  : unit
# --------------------------------------------------------------------
proc make_spinbox {w name label var min max inc unit} {
   # main --------------------------------------------------
   label $w.label($name) -text $label -anchor w
   set s [spinbox $w.spinbox($name) -from $min -to $max -width 12\
	      -increment $inc -textvariable $var -justify right]
   label $w.unit($name) -text $unit -anchor w
   grid $w.label($name) $s $w.unit($name) -in $w -sticky news
   bind $s <Button-4> "$s invoke buttonup"
   bind $s <Button-5> "$s invoke buttondown"
}
# --------------------------------------------------------------------
# make_onoff_plus_com ------------------------------------------------
#            on/off button with command button
# --------------------------------------------------------------------
#    w     : parent widget name
#    label : explanations to show
#    name  : id inside program (for grid manager)
#    val   : variable name
#    blabel: button label
#    com   : command
# --------------------------------------------------------------------
proc make_onoff_plus_com {w label name val blabel com} {
    # main --------------------------------------------------
    label $w.label($name) -text $label -anchor e
    radiobutton $w.on($name) -text "on" -variable $val -value 1 -anchor w
    radiobutton $w.off($name) -text "off" -variable $val -value 0 -anchor w
    button $w.com($name) -text $blabel -command $com
    grid $w.label($name) $w.on($name) $w.off($name) x $w.com($name)\
	-in $w -sticky news
}

# --------------------------------------------------------------------
# make_command_button ------------------------------------------------
#            Command button
# --------------------------------------------------------------------
#    w     : parent widget name
#    name  : id inside program
#    label : explanation to show
#    blabel: button label, "" can be accepted.
#    com   : command
# --------------------------------------------------------------------
proc make_command_button {w name label blabel com} {
   # main --------------------------------------------------
   label $w.label($name) -text $label -anchor e
   button $w.button($name) -text $blabel -command $com
   grid $w.label($name) $w.button($name) -in $w -sticky news
}
# --------------------------------------------------------------------
# make_many_radio_com ------------------------------------------------
#            Many radio button with command action
# --------------------------------------------------------------------
#    w     : parent widget name
#    label : explanation to show
#    name  : id
#    var   : variable name
#    com   : command
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_many_radio_com {w label name var com args} {
   # Variable length arguments --------------------
   set i 0
   foreach {j k} $args {
      set l($i) $j
      set v($i) $k	
      incr i
   }
   label $w.label($name) -text $label -anchor e
   set winlist $w.label($name)
   for {set j 0} {$j < $i} {incr j} {
      radiobutton $w.radio($name,$j) -text $l($j) -anchor w\
	  -variable $var -value $v($j) -command $com
      lappend winlist $w.radio($name,$j)
   }
   eval {grid} $winlist -sticky news
}
# --------------------------------------------------------------------
# make_menu_com ------------------------------------------------------
#            Make menu with command binding
# --------------------------------------------------------------------
#    w     : parent widget name
#    id    : id
#    label : explanation to show
#    var   : variable name
#    com   : command
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_menu_com {w id label var com args} {
   label $w.label($id) -text $label -anchor w
   eval {gf_optionMenu $w.mb($id) $var $com} $args
   grid $w.label($id) $w.mb($id) -sticky news
}
# --------------------------------------------------------------------
# make_menu_plus_button ----------------------------------------------
#            Make menu + command button
# --------------------------------------------------------------------
#    w     : parent widget name
#    id    : id
#    label : explanation to show
#    var   : variable name
#    com   : command for menu button
#    blabel: label for command button
#    bcom  : command for button
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc make_menu_plus_button {w id label var com blabel bcom args} {
   label $w.label($id) -text $label -anchor w
   eval {gf_optionMenu $w.mb($id) $var $com} $args
   button $w.button($id) -text $blabel -command $bcom
   grid $w.label($id) $w.mb($id) $w.button($id) -sticky news
}
# --------------------------------------------------------------------
# gf_optionMenu ------------------------------------------------------
#            Make an option Menu widget set
# --------------------------------------------------------------------
#    w     : widget path for this widget set
#    var   : variable name
#    com   : command
#   (variable length arguments)
#    l1    : label
#    v1    : value
# --------------------------------------------------------------------
proc gf_optionMenu {w var com args} {
   upvar #0 $var value
   proc gf_optionMenu_local {w com args} {
      eval $com
      $w conf -text $args
   }
   # determine length 
   set len 8
   foreach {i j} $args {
      set tmp [string length $i]
      set len [expr {$tmp > $len} ? $tmp : $len ]
   }
   # create menu
   menubutton $w -indicatoron 1 -menu $w.menu -direction flush\
       -relief raised -bd 2 -highlightthickness 2 -anchor c -width $len
   menu $w.menu -tearoff 0
   foreach {i j} $args {
      $w.menu add radiobutton -label $i -variable $var -value $j\
	  -command "gf_optionMenu_local $w \"$com\" $i"
      if {$j == $value} {
	 $w conf -text $i
      }
   }
   return $w.menu
}
# --------------------------------------------------------------------
# Make checkbutton widget set
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    name  : identifier
#    var   : variable name
# --------------------------------------------------------------------
proc make_checkbutton {w label name var} {
   checkbutton $w.check($name) -var $var -text $label
   grid $w.check($name) -sticky nws
   return $w.check($name)
}
# --------------------------------------------------------------------
# Make checkbutton plus label widget
# --------------------------------------------------------------------
#    w     : parent widget
#    label : text to show
#    id    : identifier
#    var   : variable name
#    com   : command to bind the checkbutton
# --------------------------------------------------------------------
proc make_check_label {w label id var com} {
   checkbutton $w.check($id) -var $var -command $com
   label $w.label($id) -text $label
   grid $w.check($id) $w.label($id) - - -sticky nws
   grid conf $w.check($id) -sticky nes
   return $w.check($id)
}
# --------------------------------------------------------------------
# Filename combo widgets : with h-scrollbar
# --------------------------------------------------------------------
# w           : parent widget name
# id          : id name in this application
# text        : label to show
# width       : width of entry widget
# var         : variable name for file
# operation   : "load", "save" or "dir"
# type        : default extension indicator
# --------------------------------------------------------------------
proc filename_hscroll_frame {w id text var width operation type} {
   # label, filename entry and "ref" button
   label $w.label($id) -text $text -anchor e
   entry $w.entry($id) -textvariable $var -width $width\
       -xscrollcommand "$w.hscroll($id) set" -justify left
   button $w.button($id) -text "Ref..."\
       -command "file_selection $var $operation {$type}"
   grid $w.label($id) $w.entry($id) - - - $w.button($id) -sticky news
   bind $w.entry($id) <Button-4> "$w.entry($id) xview scroll -3 unit"
   bind $w.entry($id) <Button-5> "$w.entry($id) xview scroll +3 unit"
   # scroll bar
   scrollbar $w.hscroll($id) -width 8 -orient horiz -relief flat -bd 2\
       -command "$w.entry($id) xview"
   grid x $w.hscroll($id) - - - x -sticky news
   # view
   $w.entry($id) xview end
}
# --------------------------------------------------------------------
# button_frame -------------------------------------------------------
#    Create button frame for several window
# --------------------------------------------------------------------
# w     : toplevel widget name (to dismiss), if $w=="", then no dismiss button
# f     : button frame widget name
# args  : "label" and "command"
# --------------------------------------------------------------------
proc button_frame {w f args} {
   # global --------------------------------------------------
   global custom
   
   # main --------------------------------------------------
   frame $f

   # command button(s) (left side) ----------
   set i 1
   set coms {}
   foreach {l c} $args {
      button $f.$i -text $l -command $c -width 12 -bd 2
      lappend coms $f.$i
      incr i
   }
   if { $coms != {} } {
      eval pack $coms -side left -padx 10 -pady 5
   }

   # dismiss button (right side) ----------
   if { $w != "" } {
      button $f.dismiss -text "dismiss" -command "destroy $w" -width 12 -bd 2\
	  -bg $custom(DismissButtonBackground)
      pack $f.dismiss -side right -padx 10 -pady 5
   }
   return $f
}
# --------------------------------------------------------------------
# enable/disable labelframe and its descendant
# --------------------------------------------------------------------
proc labelframe_status {w newstatus} {
   foreach cw [winfo children $w] {
      # current status
      catch {$cw cget -state} state
      if {$state == "disabled" || $state == "normal" } {
	 $cw conf -state $newstatus
      }
      # frame?
      if [string match -nocase frame [winfo class $cw] ] {
	 labelframe_status $cw $newstatus
      }
      # labelframe?
      if [string match -nocase labelframe [winfo class $cw] ] {
	 # disabled
	 if { $newstatus == "disabled" } {
	    labelframe_status $cw $newstatus
	    continue
	 }
	 # normal
	 set lw [$cw cget -labelwidget]
	 if { $lw == "" } {
	    labelframe_status $cw "normal"
	    continue
	 }
	 set lwclass [winfo class $lw]
	 if { "Radiobutton" == $lwclass } {
	    set varname [$lw cget -variable]
	    set onvalue [$lw cget -value]
	    upvar #0 $varname value
	    if { $value == $onvalue } {
	       labelframe_status $cw "normal"
	    } else {
	       labelframe_status $cw "disabled"
	    }
	 } elseif { "Checkbutton" == $lwclass } {
	    set varname [$lw cget -variable]
	    set onvalue [$lw cget -onvalue]
	    upvar #0 $varname value
	    if { $value == $onvalue } {
	       labelframe_status $cw "normal"
	    } else {
	       labelframe_status $cw "disabled"
	    }
	 }
      }
   }
}
# --------------------------------------------------------------------
# Update radio_frame and radio_labelframes
# --------------------------------------------------------------------
proc update_radio_frames {w varName} {
   upvar #0 $varName value
   foreach {cw} [winfo children $w] {
      if { [regexp \.radio\(.*\)$ $cw] } {
	 set var [$cw cget -var]
	 if { $var != $varName } { continue }
	 regsub ".radio" $cw ".lframe" lf
	 if { ![winfo exists $lf] } { continue }
	 set val [$cw cget -val]
	 if { $val == $value} {
	    labelframe_status $lf normal
	 } else {
	    labelframe_status $lf disabled
	 }
      }
   }
}
# --------------------------------------------------------------------
# radio_frame --------------------------------------------------------
#    radiobutton with use of radio_labelframe widget sets
# --------------------------------------------------------------------
# w     : parent widget
# id    : identifier of this labelframe
# text  : label to show
# var   : variable name for radiobutton
# val   : value of the variable
# --------------------------------------------------------------------
proc radio_frame {w id text var val} {
   set f [radiobutton $w.radio($id) -text $text -var $var -value $val\
	      -anchor w -command "update_radio_frames $w $var"]
   return $f
}
# --------------------------------------------------------------------
# radio_labelframe ---------------------------------------------------
#    labelframe with radiobutton
# --------------------------------------------------------------------
# w     : parent widget
# id    : identifier of this labelframe
# text  : label to show
# var   : variable name for radiobutton
# val   : value of the variable
# --------------------------------------------------------------------
proc radio_labelframe {w id text var val} {
   # create widgets
   set f [labelframe $w.lframe($id)]
   radiobutton $w.radio($id) -text $text -var $var -value $val\
       -command "update_radio_frames $w $var"
   $f conf -labelwidget $w.radio($id)
   return $f
}
# --------------------------------------------------------------------
# title_frame -------------------------------------------------------
#    Create title frame for several window
# --------------------------------------------------------------------
# w  : widget name for title frame
# l  : labels to show
# --------------------------------------------------------------------
proc title_frame {w l} {
   # global --------------------------------------------------
   global sys
   
   # main --------------------------------------------------
   label $w -text $l -font bold_font -relief groove -bd 2
   return $w
}
